@@ -939,10 +939,18 @@ def group_array(array, *kfields):
 
 def multi_index(shape, axis=None):
     """
-    :param shape: a shape of lenght L with P = S1 * S2 * ... * SL
+    :param shape: a shape of lenght L
     :param axis: None or an integer in the range 0 .. L -1
     :yields:
-        P tuples of indices with a slice(None) at the axis position (if any)
+        tuples of indices with a slice(None) at the axis position (if any)
+
+    >>> for slc in multi_index((2, 3), 0): print(slc)
+    (slice(None, None, None), 0, 0)
+    (slice(None, None, None), 0, 1)
+    (slice(None, None, None), 0, 2)
+    (slice(None, None, None), 1, 0)
+    (slice(None, None, None), 1, 1)
+    (slice(None, None, None), 1, 2)
     """
     if any(s >= TWO16 for s in shape):
         raise ValueError('Shape too big: ' + str(shape))
@@ -955,10 +963,12 @@ def multi_index(shape, axis=None):
         yield tuple(lst)
 
 
-def fast_agg(indices, values=None, axis=0, factor=None):
+def fast_agg(indices, values=None, axis=0, factor=None, M=None):
     """
     :param indices: N indices in the range 0 ... M - 1 with M < N
     :param values: N values (can be arrays)
+    :param factor: if given, a multiplicate factor (or weight) for the values
+    :param M: maximum index; if None, use max(indices) + 1
     :returns: M aggregated values (can be arrays)
 
     >>> values = numpy.array([[.1, .11], [.2, .22], [.3, .33], [.4, .44]])
@@ -973,15 +983,16 @@ def fast_agg(indices, values=None, axis=0, factor=None):
         raise ValueError('There are %d values but %d indices' %
                          (N, len(indices)))
     shp = values.shape[1:]
+    if M is None:
+        M = max(indices) + 1
     if not shp:
-        return numpy.bincount(indices, values)
-    M = max(indices) + 1
+        return numpy.bincount(indices, values, M)
     lst = list(shp)
     lst.insert(axis, M)
     res = numpy.zeros(lst, values.dtype)
     for mi in multi_index(shp, axis):
         vals = values[mi] if factor is None else values[mi] * factor
-        res[mi] = numpy.bincount(indices, vals)
+        res[mi] = numpy.bincount(indices, vals, M)
     return res
 
 
@@ -1079,6 +1090,25 @@ def not_equal(array_or_none1, array_or_none2):
     return (array_or_none1 != array_or_none2).any()
 
 
+def all_equals(inputs):
+    """
+    :param inputs: a list of arrays or strings
+    :returns: True if all values are equal, False otherwise
+    """
+    inp0 = inputs[0]
+    for inp in inputs[1:]:
+        try:
+            diff = inp != inp0
+        except ValueError:  # Lengths must match to compare
+            return False
+        if isinstance(diff, numpy.ndarray):
+            if diff.any():
+                return False
+        elif diff:
+            return False
+    return True
+
+
 def humansize(nbytes, suffixes=('B', 'KB', 'MB', 'GB', 'TB', 'PB')):
     """
     Return file size in a human-friendly format
@@ -1423,6 +1453,51 @@ def agg_probs(*probs):
         acc *= 1. - prob
     return 1. - acc
 
+
+class RecordBuilder(object):
+    """
+    Builder for numpy records or arrays.
+
+    >>> rb = RecordBuilder(a=0, b=1., c="2")
+    >>> rb.dtype
+    dtype([('a', '<i8'), ('b', '<f8'), ('c', 'S1')])
+    >>> rb()
+    (0, 1., b'2')
+    """
+    def __init__(self, **defaults):
+        self.names = []
+        self.values = []
+        dtypes = []
+        for name, value in defaults.items():
+            self.names.append(name)
+            self.values.append(value)
+            if isinstance(value, (str, bytes)):
+                tp = (numpy.string_, len(value))
+            elif isinstance(value, numpy.ndarray):
+                tp = value.dtype
+            else:
+                tp = type(value)
+            dtypes.append(tp)
+        self.dtype = numpy.dtype([(n, d) for n, d in zip(self.names, dtypes)])
+
+    def zeros(self, shape):
+        return numpy.zeros(shape, self.dtype)
+
+    def dictarray(self, shape):
+        return {n: numpy.ones(shape, self.dtype[n]) for n in self.names}
+
+    def __call__(self, *args, **kw):
+        rec = numpy.zeros(1, self.dtype)[0]
+        for i, name in enumerate(self.names):
+            if name in kw:
+                rec[name] = kw[name]  # takes precedence
+                continue
+            try:
+                rec[name] = args[i]
+            except IndexError:
+                rec[name] = self.values[i]
+        return rec
+
 # #################### COMPRESSION/DECOMPRESSION ##################### #
 
 # Compressing the task outputs makes everything slower, so you should NOT
