@@ -17,8 +17,7 @@
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-import ast
-import csv
+import sys
 import inspect
 import tempfile
 import warnings
@@ -31,7 +30,7 @@ import toml
 import pandas
 import numpy
 import h5py
-from openquake.baselib import InvalidFile
+from openquake.baselib import InvalidFile, general
 from openquake.baselib.python3compat import encode, decode
 
 vbytes = h5py.special_dtype(vlen=bytes)
@@ -44,17 +43,20 @@ vfloat64 = h5py.special_dtype(vlen=numpy.float64)
 
 FLOAT = (float, numpy.float32, numpy.float64)
 INT = (int, numpy.int32, numpy.uint32, numpy.int64, numpy.uint64)
+MAX_ROWS = 10_000_000
 
 
-def maybe_encode(value):
+def sanitize(value):
     """
-    If value is a sequence of strings, encode it
+    Sanitize the value so that it can be stored as an HDF5 attribute
     """
     if isinstance(value, bytes):
         return numpy.void(value)
-    if isinstance(value, (list, tuple)):
+    elif isinstance(value, (list, tuple)):
         if value and isinstance(value[0], str):
             return encode(value)
+    elif isinstance(value, int) and value > sys.maxsize:
+        return float(value)
     return value
 
 
@@ -78,7 +80,7 @@ def create(hdf5, name, dtype, shape=(None,), compression=None,
                                    compression=compression)
     if attrs:
         for k, v in attrs.items():
-            dset.attrs[k] = maybe_encode(v)
+            dset.attrs[k] = sanitize(v)
     return dset
 
 
@@ -114,68 +116,6 @@ def extend(dset, array, **attrs):
     return newlength
 
 
-class LiteralAttrs(object):
-    """
-    A class to serialize a set of parameters in HDF5 format. The goal is to
-    store simple parameters as an HDF5 table in a readable way. Each
-    parameter can be retrieved as an attribute, given its name. The
-    implementation treats specially dictionary attributes, by storing
-    them as `attrname.keyname` strings, see the example below:
-
-    >>> class Ser(LiteralAttrs):
-    ...     def __init__(self, a, b):
-    ...         self.a = a
-    ...         self.b = b
-    >>> ser = Ser(1, dict(x='xxx', y='yyy'))
-    >>> arr, attrs = ser.__toh5__()
-    >>> for k, v in arr:
-    ...     print('%s=%s' % (k, v))
-    a=1
-    b.x='xxx'
-    b.y='yyy'
-    >>> s = object.__new__(Ser)
-    >>> s.__fromh5__(arr, attrs)
-    >>> s.a
-    1
-    >>> s.b['x']
-    'xxx'
-
-    The implementation is not recursive, i.e. there will be at most
-    one dot in the serialized names (in the example here `a`, `b.x`, `b.y`).
-
-    """
-    def __toh5__(self):
-        info_dt = numpy.dtype([('par_name', vbytes), ('par_value', vbytes)])
-        attrnames = sorted(a for a in vars(self) if not a.startswith('_'))
-        lst = []
-        for attr in attrnames:
-            value = getattr(self, attr)
-            if isinstance(value, dict):
-                for k, v in sorted(value.items()):
-                    key = '%s.%s' % (attr, k)
-                    lst.append((key, repr(v)))
-            else:
-                lst.append((attr, repr(value)))
-        return numpy.array(lst, info_dt), {}
-
-    def __fromh5__(self, array, attrs):
-        dd = collections.defaultdict(dict)
-        for (name_, literal_) in array:
-            name = decode(name_)
-            literal = decode(literal_)
-            if '.' in name:
-                k1, k2 = name.split('.', 1)
-                dd[k1][k2] = ast.literal_eval(literal)
-            else:
-                dd[name] = ast.literal_eval(literal)
-        vars(self).update(dd)
-
-    def __repr__(self):
-        names = sorted(n for n in vars(self) if not n.startswith('_'))
-        nameval = ', '.join('%s=%r' % (n, getattr(self, n)) for n in names)
-        return '<%s %s>' % (self.__class__.__name__, nameval)
-
-
 def cls2dotname(cls):
     """
     The full Python name (i.e. `pkg.subpkg.mod.cls`) of a class
@@ -193,13 +133,13 @@ def dotname2cls(dotname):
 
 def get_nbytes(dset):
     """
-    If the dataset has an attribute 'nbytes', return it. Otherwise get the size
-    of the underlying array. Returns None if the dataset is actually a group.
+    :param dset:
+        an HDF5 group or dataset
+    :returns:
+        the size of the underlying array or None if the dataset
+        is actually a group.
     """
-    if 'nbytes' in dset.attrs:
-        # look if the dataset has an attribute nbytes
-        return dset.attrs['nbytes']
-    elif hasattr(dset, 'dtype'):
+    if hasattr(dset, 'dtype'):
         # else extract nbytes from the underlying array
         return dset.size * numpy.zeros(1, dset.dtype).nbytes
 
@@ -249,6 +189,89 @@ class Group(collections.abc.Mapping):
         return len(self.dic)
 
 
+def sel(dset, filterdict):
+    """
+    Select a dataset with shape_descr. For instance
+    dstore.sel('hcurves', imt='PGA', sid=2)
+    """
+    dic = get_shape_descr(dset.attrs['json'])
+    lst = []
+    for dim in dic['shape_descr']:
+        if dim in filterdict:
+            val = filterdict[dim]
+            values = dic[dim]
+            if isinstance(val, INT) and val < 0:
+                # for instance sid=-1 means the last sid
+                idx = values[val]
+            else:
+                idx = values.index(val)
+            lst.append(slice(idx, idx + 1))
+        else:
+            lst.append(slice(None))
+    return dset[tuple(lst)]
+
+
+def dset2df(dset, indexfield, filterdict):
+    """
+    Converts an HDF5 dataset with an attribute shape_descr into a Pandas
+    dataframe. NB: this is very slow for large datasets.
+    """
+    arr = sel(dset, filterdict)
+    dic = get_shape_descr(dset.attrs['json'])
+    tags = []
+    idxs = []
+    for dim in dic['shape_descr']:
+        values = dic[dim]
+        if dim in filterdict:
+            val = filterdict[dim]
+            idx = values.index(val)
+            idxs.append([idx])
+            values = [val]
+        elif hasattr(values, 'stop'):  # a range object already
+            idxs.append(values)
+        else:
+            idxs.append(range(len(values)))
+        tags.append(values)
+    acc = general.AccumDict(accum=[])
+    index = []
+    for idx, vals in zip(itertools.product(*idxs), itertools.product(*tags)):
+        for field, val in zip(dic['shape_descr'], vals):
+            if field == indexfield:
+                index.append(val)
+            else:
+                acc[field].append(val)
+        acc['value'].append(arr[idx])
+    return pandas.DataFrame(acc, index or None)
+
+
+def extract_cols(datagrp, sel, slc, columns):
+    """
+    :param datagrp: something like and HDF5 data group
+    :param sel: dictionary column name -> value specifying a selection
+    :param slc: a slice object specifying the rows considered
+    :param columns: the full list of column names
+    :returns: a dictionary col -> array of values
+    """
+    first = columns[0]
+    nrows = len(datagrp[first])
+    if slc.start is None and slc.stop is None:  # split in slices
+        slcs = general.gen_slices(0, nrows, MAX_ROWS)
+    else:
+        slcs = [slc]
+    acc = general.AccumDict(accum=[])  # col -> arrays
+    for slc in slcs:
+        ok = slice(None)
+        dic = {col: datagrp[col][slc] for col in sel}
+        for col in sel:
+            if isinstance(ok, slice):  # first selection
+                ok = dic[col] == sel[col]
+            else:  # other selections
+                ok &= dic[col] == sel[col]
+        for col in columns:
+            acc[col].append(datagrp[col][slc][ok])
+    return {k: numpy.concatenate(vs) for k, vs in acc.items()}
+
+
 class File(h5py.File):
     """
     Subclass of :class:`h5py.File` able to store and retrieve objects
@@ -264,6 +287,9 @@ class File(h5py.File):
     3
     >>> f.close()
     """
+    class EmptyDataset(ValueError):
+        """Raised when reading an empty dataset"""
+
     def __init__(self, name, mode='r', driver=None, libver='latest',
                  userblock_size=None, swmr=True, rdcc_nslots=None,
                  rdcc_nbytes=None, rdcc_w0=None, track_order=None,
@@ -285,6 +311,82 @@ class File(h5py.File):
         self.path = path
         return self
 
+    def create_df(self, key, nametypes, compression=None, **kw):
+        """
+        Create a HDF5 datagroup readable as a pandas DataFrame
+
+        :param key:
+            name of the dataset
+        :param nametypes:
+            list of pairs (name, dtype) or (name, array) or DataFrame
+        :param compression:
+            the kind of HDF5 compression to use
+        :param kw:
+            extra attributes to store
+        """
+        if isinstance(nametypes, pandas.DataFrame):
+            nametypes = {name: nametypes[name].to_numpy()
+                         for name in nametypes.columns}.items()
+        names = []
+        for name, value in nametypes:
+            is_array = isinstance(value, numpy.ndarray)
+            if is_array and isinstance(value[0], str):
+                dt = vstr
+            elif is_array:
+                dt = value.dtype
+            else:
+                dt = value
+            dset = create(self, f'{key}/{name}', dt, (None,), compression)
+            if is_array:
+                extend(dset, value)
+            names.append(name)
+        attrs = self[key].attrs
+        attrs['__pdcolumns__'] = ' '.join(names)
+        for k, v in kw.items():
+            attrs[k] = v
+
+    def read_df(self, key, index=None, sel=(), slc=slice(None)):
+        """
+        :param key: name of the structured dataset
+        :param index: pandas index (or multi-index), possibly None
+        :param sel: dictionary used to select subsets of the dataset
+        :param slc: slice object to extract a slice of the dataset
+        :returns: pandas DataFrame associated to the dataset
+        """
+        dset = self.getitem(key)
+        if len(dset) == 0:
+            raise self.EmptyDataset('Dataset %s is empty' % key)
+        elif 'json' in dset.attrs:
+            return dset2df(dset, index, sel)
+        elif '__pdcolumns__' in dset.attrs:
+            columns = dset.attrs['__pdcolumns__'].split()
+            dic = extract_cols(dset, sel, slc, columns)
+            if index is None:
+                return pandas.DataFrame(dic)
+            else:
+                return pandas.DataFrame(dic).set_index(index)
+
+        dtlist = []
+        for name in dset.dtype.names:
+            dt = dset.dtype[name]
+            if dt.shape:  # vector field
+                templ = name + '_%d' * len(dt.shape)
+                for i, _ in numpy.ndenumerate(numpy.zeros(dt.shape)):
+                    dtlist.append((templ % i, dt.base))
+            else:  # scalar field
+                dtlist.append((name, dt))
+        data = numpy.zeros(len(dset), dtlist)
+        for name in dset.dtype.names:
+            arr = dset[name]
+            dt = dset.dtype[name]
+            if dt.shape:  # vector field
+                templ = name + '_%d' * len(dt.shape)
+                for i, _ in numpy.ndenumerate(numpy.zeros(dt.shape)):
+                    data[templ % i] = arr[(slice(None),) + i]
+            else:  # scalar field
+                data[name] = arr
+        return pandas.DataFrame.from_records(data, index=index)
+
     def save_vlen(self, key, data):  # used in SourceWriterTestCase
         """
         Save a sequence of variable-length arrays
@@ -308,7 +410,7 @@ class File(h5py.File):
             a = super().__getitem__(path).attrs
             for k, v in sorted(items):
                 try:
-                    a[k] = maybe_encode(v)
+                    a[k] = sanitize(v)
                 except Exception as exc:
                     raise TypeError(
                         'Could not store attribute %s=%s: %s' % (k, v, exc))
@@ -406,7 +508,7 @@ def array_of_vstr(lst):
 
 def dumps(dic):
     """
-    Dump in json
+    Dump a dictionary in json. Extend json.dumps to work on numpy objects.
     """
     new = {}
     for k, v in dic.items():
@@ -429,14 +531,12 @@ def dumps(dic):
                 new[k] = json.dumps(decode_array(v))
             else:
                 new[k] = json.dumps(lst)
-        elif hasattr(v, '__dict__'):
-            new[k] = {cls2dotname(v.__class__): dumps(vars(v))}
         elif isinstance(v, dict):
             new[k] = dumps(v)
-        elif isinstance(v, str):
-            new[k] = '"%s"' % v
+        elif hasattr(v, '__dict__'):
+            new[k] = {cls2dotname(v.__class__): dumps(vars(v))}
         else:
-            new[k] = v
+            new[k] = json.dumps(v)
     return "{%s}" % ','.join('\n"%s": %s' % it for it in new.items())
 
 
@@ -470,7 +570,7 @@ def get_shape_descr(json_string):
     for field in dic['shape_descr']:
         val = dic[field]
         if isinstance(val, INT):
-            dic[field] = range(val)
+            dic[field] = list(range(val))
     return dic
 
 
@@ -542,7 +642,10 @@ class ArrayWrapper(object):
             lst = ['%s=%d' % (descr, size)
                    for descr, size in zip(self.shape_descr, self.shape)]
             return '<%s(%s)>' % (self.__class__.__name__, ', '.join(lst))
-        return '<%s%s>' % (self.__class__.__name__, self.shape)
+        elif hasattr(self, 'shape'):
+            return '<%s%s>' % (self.__class__.__name__, self.shape)
+        else:
+            return '<%s %d bytes>' % (self.__class__.__name__, len(self.array))
 
     @property
     def dtype(self):
@@ -686,44 +789,51 @@ def build_dt(dtypedict, names):
     return numpy.dtype(lst)
 
 
+def check_length(field, size):
+    """
+    :param field: a bytes field in the exposure
+    :param size: maximum size of the field
+    :returns: a function checking that the value is below the size
+    """
+    def check(val):
+        if len(val) > size:
+            raise ValueError('%s=%r has length %d > %d' %
+                             (field, val, len(val), size))
+        return val
+    return check
+
+
 def _read_csv(fileobj, compositedt):
-    itemsize = [0] * len(compositedt)
-    dt = []
-    for i, name in enumerate(compositedt.names):
-        dt.append(compositedt[name])
-        if compositedt[name].kind == 'S':  # limit of the length of byte-fields
-            itemsize[i] = compositedt[name].itemsize
-    rows = []
-    for lineno, row in enumerate(csv.reader(fileobj), 3):
-        cols = []
-        for i, col in enumerate(row):
-            if itemsize[i] and len(col) > itemsize[i]:
-                raise ValueError(
-                    'line %d: %s=%r has length %d > %d' %
-                    (lineno, compositedt.names[i], col, len(col), itemsize[i]))
-            if dt[i].kind == 'b':  # boolean
-                cols.append(int(col))
-            else:
-                cols.append(col)
-        rows.append(tuple(cols))
-    return numpy.array(rows, compositedt)
+    dic = {}
+    conv = {}
+    for name in compositedt.names:
+        dic[name] = dt = compositedt[name]
+        if dt.kind == 'S':  # limit of the length of byte-fields
+            conv[name] = check_length(name, dt.itemsize)
+    df = pandas.read_csv(fileobj, names=compositedt.names, converters=conv,
+                         dtype=dic, keep_default_na=False, na_filter=False)
+    arr = numpy.zeros(len(df), compositedt)
+    for col in df.columns:
+        arr[col] = df[col].to_numpy()
+    return arr
 
 
 # NB: it would be nice to use numpy.loadtxt(
 #  f, build_dt(dtypedict, header), delimiter=sep, ndmin=1, comments=None)
 # however numpy does not support quoting, and "foo,bar" would be split :-(
 def read_csv(fname, dtypedict={None: float}, renamedict={}, sep=',',
-             index=None):
+             index=None, errors=None):
     """
     :param fname: a CSV file with an header and float fields
     :param dtypedict: a dictionary fieldname -> dtype, None -> default
     :param renamedict: aliases for the fields to rename
     :param sep: separator (default comma)
     :param index: if not None, returns a pandas DataFrame
+    :param errors: passed to the underlying open function (default None)
     :returns: an ArrayWrapper, unless there is an index
     """
     attrs = {}
-    with open(fname, encoding='utf-8-sig') as f:
+    with open(fname, encoding='utf-8-sig', errors=errors) as f:
         while True:
             first = next(f)
             if first.startswith('#'):
