@@ -104,12 +104,12 @@ Monitoring
 A major feature of the Starmap API is the ability to monitor the time spent
 in each task and the memory allocated. Such information is written into an
 HDF5 file that can be provided by the user or autogenerated. To autogenerate
-the file you can use :func:`openquake.baselib.datastore.hdf5new` which
+the file you can use :func:`openquake.commonlib.datastore.hdf5new` which
 will create a file named ``calc_XXX.hdf5`` in your $OQ_DATA directory
 (if the environment variable is not set, the engine will use $HOME/oqdata).
 Here is an example of usage:
 
->>> from openquake.baselib.datastore import hdf5new
+>>> from openquake.commonlib.datastore import hdf5new
 >>> h5 = hdf5new()
 >>> smap = Starmap(count, [['hello'], ['world']], h5=h5)
 >>> print(sorted(smap.reduce().items()))
@@ -206,6 +206,7 @@ except ImportError:
         "Do nothing"
 
 from openquake.baselib import config, hdf5, workerpool, version
+from openquake.baselib.python3compat import decode
 from openquake.baselib.zeromq import zmq, Socket
 from openquake.baselib.performance import (
     Monitor, memory_rss, init_performance)
@@ -412,6 +413,10 @@ class Result(object):
                     'The master is at version %s while the worker %s is at '
                     'version %s' % (mon.version, socket.gethostname(),
                                     version))
+            if mon.config.dbserver.host != config.dbserver.host:
+                raise RuntimeError(
+                    'The master has dbserver.host=%s while the worker has %s'
+                    % (mon.config.dbserver.host, config.dbserver.host))
             with mon:
                 val = func(*args)
         except StopIteration:
@@ -444,6 +449,7 @@ def check_mem_usage(soft_percent=None, hard_percent=None):
 
 dummy_mon = Monitor()
 dummy_mon.version = version
+dummy_mon.config = config
 dummy_mon.backurl = None
 
 
@@ -556,7 +562,7 @@ class IterResult(object):
                 # measure only the memory used by the main process
                 mem_gb = memory_rss(os.getpid()) / GB
             if result.msg == 'TASK_ENDED':
-                task_sent = ast.literal_eval(self.h5['task_sent'][()])
+                task_sent = ast.literal_eval(decode(self.h5['task_sent'][()]))
                 task_sent.update(self.sent)
                 del self.h5['task_sent']
                 self.h5['task_sent'] = str(task_sent)
@@ -578,7 +584,8 @@ class IterResult(object):
             nb = {k: humansize(v) for k, v in reversed(items)}
             msg = nb if len(nb) < 10 else {
                 'tot': humansize(sum(self.nbytes.values()))}
-            logging.info('Received %s in %d seconds', msg, time.time() - t0)
+            logging.info('Received %s in %d seconds from %s',
+                         msg, time.time() - t0, self.name)
 
     def reduce(self, agg=operator.add, acc=None):
         if acc is None:
@@ -780,6 +787,7 @@ class Starmap(object):
             monitor.backurl = 'tcp://%s:%s' % (
                 config.dbserver.host, self.socket.port)
             monitor.version = version
+            monitor.config = config
         OQ_TASK_NO = os.environ.get('OQ_TASK_NO')
         if OQ_TASK_NO is not None and self.task_no != int(OQ_TASK_NO):
             self.task_no += 1
@@ -850,8 +858,8 @@ class Starmap(object):
 
         nbytes = sum(self.sent[self.task_func.__name__].values())
         if nbytes > 1E6:
-            logging.info('Sent %d tasks, %s in %d seconds', len(self.tasks),
-                         humansize(nbytes), time.time() - self.t0)
+            logging.info('Sent %d %s tasks, %s in %d seconds', len(self.tasks),
+                         self.name, humansize(nbytes), time.time() - self.t0)
 
         isocket = iter(self.socket)
         self.todo = len(self.tasks)
@@ -860,7 +868,7 @@ class Starmap(object):
             res = next(isocket)
             if self.calc_id != res.mon.calc_id:
                 logging.warning('Discarding a result from job %s, since this '
-                                'is job %d', res.mon.calc_id, self.calc_id)
+                                'is job %s', res.mon.calc_id, self.calc_id)
             elif res.msg == 'TASK_ENDED':
                 self.busytime += {res.workerid: res.mon.duration}
                 self.todo -= 1
@@ -954,9 +962,9 @@ def workers_start():
     """
     if OQDIST in 'no processpool':
         return
-    sched = config.distribution.dask_scheduler
     for host, cores, args in ssh_args():
         if OQDIST == 'dask':
+            sched = config.distribution.dask_scheduler
             args += ['-m', 'distributed.cli.dask_worker', sched,
                      '--nprocs', cores, '--nthreads', '1',
                      '--memory-limit', '1e11']
@@ -980,10 +988,23 @@ def workers_stop():
     elif OQDIST == 'celery':
         app.control.shutdown()
     elif OQDIST == 'zmq':
-        workerpool.WorkerMaster().kill()
+        workerpool.WorkerMaster().stop()
     return 'stopped'
 
 
+def workers_kill():
+    """
+    Kill all the workers
+    """
+    if OQDIST == 'dask':
+        Client(config.distribution.dask_scheduler).retire_workers()
+    elif OQDIST == 'celery':
+        app.control.shutdown()
+    elif OQDIST == 'zmq':
+        workerpool.WorkerMaster().kill()
+    return 'killed'
+
+
 def workers_status():
     """
     :returns: a list [(host name, running, total), ...]
@@ -1025,23 +1046,3 @@ def workers_wait(seconds=30):
         else:
             raise TimeoutError(status)
         return status
-
-
-def workers_kill():
-    code = '''import getpass, psutil
-user = getpass.getuser()
-for proc in psutil.process_iter(['name', 'username']):
-    if proc.username() == user:
-        cmdline = proc.cmdline()
-        if ('workerpool' in cmdline or 'celery' in cmdline or
-            'distributed.cli.dask_worker' in cmdline or
-            'distributed.cli.dask_scheduler' in cmdline):
-            print('killing %s' % ' '.join(cmdline))
-            proc.kill()
-'''
-    hosts = []
-    for host, cores, args in ssh_args():
-        c = '"%s"' % code if 'ssh' in args else code
-        out = subprocess.check_output(args + ['-c', c]).decode('utf8')
-        hosts.append('%s: %s' % (host, out))
-    return '\n'.join(hosts)
