@@ -18,7 +18,6 @@
 import io
 import time
 import psutil
-import pprint
 import logging
 import operator
 import numpy
@@ -33,13 +32,13 @@ from openquake.baselib.general import (
     get_nbytes_msg)
 from openquake.hazardlib.source.point import (
     PointSource, grid_point_sources, msr_name)
-from openquake.hazardlib.source.base import EPS
+from openquake.hazardlib.source.base import EPS, get_code2cls
 from openquake.hazardlib.sourceconverter import SourceGroup
-from openquake.hazardlib.contexts import ContextMaker, get_effect
+from openquake.hazardlib.contexts import ContextMaker, read_cmakers
 from openquake.hazardlib.calc.filters import split_source, SourceFilter
 from openquake.hazardlib.calc.hazard_curve import classical as hazclassical
 from openquake.hazardlib.probability_map import ProbabilityMap
-from openquake.commonlib import calc, util, readinput
+from openquake.commonlib import calc, readinput, datastore
 from openquake.calculators import getters
 from openquake.calculators import base
 
@@ -48,13 +47,13 @@ U32 = numpy.uint32
 F32 = numpy.float32
 F64 = numpy.float64
 TWO32 = 2 ** 32
-BUFFER = 1.5  # enlarge the pointsource_distance sphere to fix the weight
+BUFFER = 1.5  # enlarge the pointsource_distance sphere to fix the weight;
 # with BUFFER = 1 we would have lots of apparently light sources
 # collected together in an extra-slow task, as it happens in SHARE
 # with ps_grid_spacing=50
 get_weight = operator.attrgetter('weight')
-grp_extreme_dt = numpy.dtype([('et_id', U16), ('grp_trt', hdf5.vstr),
-                             ('extreme_poe', F32)])
+grp_extreme_dt = numpy.dtype([('grp_id', U16), ('grp_trt', hdf5.vstr),
+                              ('extreme_poe', F32), ('smrs', hdf5.vuint16)])
 
 
 def get_source_id(src):  # used in submit_tasks
@@ -77,8 +76,6 @@ def run_preclassical(csm, oqparam, h5):
     :param oqparam: the parameters in job.ini file
     :param h5: a DataStore instance
     """
-    logging.info('Sending %s', csm.sitecol)
-
     # do nothing for atomic sources except counting the ruptures
     for src in csm.get_sources(atomic=True):
         src.num_ruptures = src.count_ruptures()
@@ -95,10 +92,23 @@ def run_preclassical(csm, oqparam, h5):
     srcfilter = SourceFilter(
         csm.sitecol.reduce(10000) if csm.sitecol else None,
         oqparam.maximum_distance)
+    if csm.sitecol:
+        logging.info('Sending %s', srcfilter.sitecol)
+    if oqparam.ps_grid_spacing:
+        # produce a preclassical task for each group
+        allargs = ((srcs, srcfilter, param)
+                   for srcs in sources_by_grp.values())
+    else:
+        # produce many preclassical task
+        maxw = sum(len(srcs) for srcs in sources_by_grp.values()) / (
+            oqparam.concurrent_tasks or 1)
+        allargs = ((blk, srcfilter, param)
+                   for srcs in sources_by_grp.values()
+                   for blk in block_splitter(srcs, maxw))
     res = parallel.Starmap(
-        preclassical,
-        ((srcs, srcfilter, param) for srcs in sources_by_grp.values()),
-        h5=h5, distribute=None if len(sources_by_grp) > 1 else 'no').reduce()
+        preclassical, allargs,  h5=h5,
+        distribute=None if len(sources_by_grp) > 1 else 'no'
+    ).reduce()
 
     if res and res['before'] != res['after']:
         logging.info('Reduced the number of sources from {:_d} -> {:_d}'.
@@ -107,12 +117,19 @@ def run_preclassical(csm, oqparam, h5):
     if res and h5:
         csm.update_source_info(res['calc_times'], nsites=True)
 
+    acc = AccumDict(accum=0)
+    code2cls = get_code2cls()
     for grp_id, srcs in res.items():
         # srcs can be empty if the minimum_magnitude filter is on
         if srcs and not isinstance(grp_id, str):
             newsg = SourceGroup(srcs[0].tectonic_region_type)
             newsg.sources = srcs
             csm.src_groups[grp_id] = newsg
+            for src in srcs:
+                acc[src.code] += int(src.num_ruptures)
+    for val, key in sorted((val, key) for key, val in acc.items()):
+        cls = code2cls[key].__name__
+        logging.info('{} ruptures: {:_d}'.format(cls, val))
 
     # sanity check
     for sg in csm.src_groups:
@@ -138,7 +155,6 @@ def store_ctxs(dstore, rupdata, grp_id):
     Store contexts with the same magnitude in the datastore
     """
     nr = len(rupdata['mag'])
-    rupdata['nsites'] = numpy.array([len(s) for s in rupdata['sids_']])
     rupdata['grp_id'] = numpy.repeat(grp_id, nr)
     nans = numpy.repeat(numpy.nan, nr)
     for par in dstore['rup']:
@@ -212,12 +228,12 @@ def preclassical(srcs, srcfilter, params, monitor):
     return dic
 
 
-def classical(srcs, rlzs_by_gsim, params, monitor):
+def classical(srcs, cmaker, monitor):
     """
     Read the SourceFilter and call the classical calculator in hazardlib
     """
-    srcfilter = monitor.read('srcfilter')
-    return hazclassical(srcs, srcfilter, rlzs_by_gsim, params, monitor)
+    cmaker.init_monitoring(monitor)
+    return hazclassical(srcs, monitor.read('sitecol'), cmaker)
 
 
 class Hazard:
@@ -227,49 +243,55 @@ class Hazard:
     def __init__(self, dstore, full_lt, pgetter, srcidx):
         self.datastore = dstore
         self.full_lt = full_lt
-        self.et_ids = dstore['et_ids'][:]
-        self.rlzs_by_gsim_list = full_lt.get_rlzs_by_gsim_list(self.et_ids)
-        self.slice_by_g = getters.get_slice_by_g(self.rlzs_by_gsim_list)
+        self.cmakers = read_cmakers(dstore, full_lt)
         self.get_hcurves = pgetter.get_hcurves
         self.imtls = pgetter.imtls
         self.sids = pgetter.sids
         self.srcidx = srcidx
-        self.data = []
+        extreme = []
+        n = len(full_lt.sm_rlzs)
+        for grp_id, indices in enumerate(dstore['trt_smrs']):
+            trti, smrs = numpy.divmod(indices, n)
+            trt = full_lt.trts[trti[0]]
+            extreme.append((grp_id, trt, 0, smrs))
+        self.extreme = numpy.array(extreme, grp_extreme_dt)
 
     def init(self, pmaps, grp_id):
         """
         Initialize the pmaps dictionary with zeros, if needed
         """
         if grp_id not in pmaps:
-            L, G = self.imtls.size, len(self.rlzs_by_gsim_list[grp_id])
+            L, G = self.imtls.size, len(self.cmakers[grp_id].gsims)
             pmaps[grp_id] = ProbabilityMap.build(L, G, self.sids)
+            pmaps[grp_id].grp_id = grp_id
 
     def store_poes(self, grp_id, pmap):
         """
         Store the pmap of the given group inside the _poes dataset
         """
-        trt = self.full_lt.trt_by_et[self.et_ids[grp_id][0]]
+        cmaker = self.cmakers[grp_id]
+        dset = self.datastore['_poes']
         base.fix_ones(pmap)  # avoid saving PoEs == 1, fast
         arr = numpy.array([pmap[sid].array for sid in pmap]).transpose(2, 0, 1)
-        self.datastore['_poes'][self.slice_by_g[grp_id]] = arr  # shape GNL
+        for g, mat in enumerate(arr):
+            dset[cmaker.start + g] = mat  # shape NL
         extreme = max(
             get_extreme_poe(pmap[sid].array, self.imtls)
             for sid in pmap)
-        self.data.append((grp_id, trt, extreme))
+        self.extreme[grp_id]['extreme_poe'] = extreme
 
     def store_disagg(self, pmaps=None):
         """
-        Store data inside disagg_by_src/disagg_by_grp
+        Store data inside disagg_by_grp (optionally disagg_by_src)
         """
+        self.datastore['disagg_by_grp'] = self.extreme
         if pmaps:  # called inside a loop
             for key, pmap in pmaps.items():
-                # contains only string keys in case of disaggregation
-                rlzs_by_gsim = self.rlzs_by_gsim_list[pmap.grp_id]
-                self.datastore['disagg_by_src'][..., self.srcidx[key]] = (
-                    self.get_hcurves(pmap, rlzs_by_gsim))
-        else:  # called at the end of the loop
-            self.datastore['disagg_by_grp'] = numpy.array(
-                sorted(self.data), grp_extreme_dt)
+                if isinstance(key, str):
+                    # contains only string keys in case of disaggregation
+                    rlzs_by_gsim = self.cmakers[pmap.grp_id].gsims
+                    self.datastore['disagg_by_src'][..., self.srcidx[key]] = (
+                        self.get_hcurves(pmap, rlzs_by_gsim))
 
 
 @base.calculators.add('classical', 'preclassical', 'ucerf_classical')
@@ -291,8 +313,7 @@ class ClassicalCalculator(base.HazardCalculator):
         # for an OOM it can become None, thus giving a very confusing error
         if dic is None:
             raise MemoryError('You ran out of memory!')
-        pmap = dic['pmap']
-        extra = dic['extra']
+
         ctimes = dic['calc_times']  # srcid -> eff_rups, eff_sites, dt
         self.calc_times += ctimes
         srcids = set()
@@ -303,27 +324,27 @@ class ClassicalCalculator(base.HazardCalculator):
             eff_rups += rec[0]
             if rec[0]:
                 eff_sites += rec[1] / rec[0]
-        self.by_task[extra['task_no']] = (
+        self.by_task[dic['task_no']] = (
             eff_rups, eff_sites, sorted(srcids))
-        self.rel_ruptures[extra.pop('trt')] += eff_rups
-        grp_id = extra['grp_id']
-        self.counts[grp_id] -= 1
-        if self.oqparam.disagg_by_src:
-            # store the poes for the given source
-            pmap.grp_id = grp_id
-            acc[extra['source_id'].split(':')[0]] = pmap
-
-        self.maxradius = max(self.maxradius, extra.pop('maxradius'))
-        with self.monitor('aggregate curves'):
-            if pmap:
-                self.haz.init(acc, grp_id)
-                acc[grp_id] |= pmap
+        grp_id = dic.pop('grp_id')
+        self.rel_ruptures[grp_id] += eff_rups
 
         # store rup_data if there are few sites
         if self.few_sites and len(dic['rup_data']['src_id']):
             with self.monitor('saving rup_data'):
                 store_ctxs(self.datastore, dic['rup_data'], grp_id)
+        with self.monitor('aggregate curves'):
+            pmap = dic['pmap']
+            pmap.grp_id = grp_id
+            source_id = dic.pop('source_id', None)
+            if source_id:
+                # store the poes for the given source
+                acc[source_id.split(':')[0]] = pmap
+            if pmap:
+                self.haz.init(acc, grp_id)
+                acc[grp_id] |= pmap
 
+        self.counts[grp_id] -= 1
         if self.counts[grp_id] == 0:
             with self.monitor('saving probability maps'):
                 if grp_id in acc:
@@ -335,7 +356,7 @@ class ClassicalCalculator(base.HazardCalculator):
         Store some empty datasets in the datastore
         """
         params = {'grp_id', 'occurrence_rate', 'clon_', 'clat_', 'rrup_',
-                  'nsites', 'probs_occur_', 'sids_', 'src_id'}
+                  'probs_occur_', 'sids_', 'src_id'}
         gsims_by_trt = self.full_lt.get_gsims_by_trt()
         for trt, gsims in gsims_by_trt.items():
             cm = ContextMaker(trt, gsims, dict(imtls=self.oqparam.imtls))
@@ -357,16 +378,15 @@ class ClassicalCalculator(base.HazardCalculator):
                     dt = hdf5.vfloat32
                 elif param == 'src_id':
                     dt = U32
-                elif param in {'nsites', 'grp_id'}:
+                elif param == 'grp_id':
                     dt = U16
                 else:
                     dt = F32
                 descr.append((param, dt))
-            self.datastore.create_dframe('rup', descr, 'gzip')
+            self.datastore.create_df('rup', descr, 'gzip')
         self.by_task = {}  # task_no => src_ids
-        self.maxradius = 0
         self.Ns = len(self.csm.source_info)
-        self.rel_ruptures = AccumDict(accum=0)  # trt -> rel_ruptures
+        self.rel_ruptures = AccumDict(accum=0)  # grp_id -> rel_ruptures
         # NB: the relevant ruptures are less than the effective ruptures,
         # which are a preclassical concept
         if self.oqparam.disagg_by_src:
@@ -404,12 +424,12 @@ class ClassicalCalculator(base.HazardCalculator):
         super().init()
         if self.oqparam.hazard_calculation_id:
             full_lt = self.datastore.parent['full_lt']
-            et_ids = self.datastore.parent['et_ids'][:]
+            trt_smrs = self.datastore.parent['trt_smrs'][:]
         else:
             full_lt = self.csm.full_lt
-            et_ids = self.csm.get_et_ids()
-        self.grp_ids = numpy.arange(len(et_ids))
-        rlzs_by_gsim_list = full_lt.get_rlzs_by_gsim_list(et_ids)
+            trt_smrs = self.csm.get_trt_smrs()
+        self.grp_ids = numpy.arange(len(trt_smrs))
+        rlzs_by_gsim_list = full_lt.get_rlzs_by_gsim_list(trt_smrs)
         rlzs_by_g = []
         for rlzs_by_gsim in rlzs_by_gsim_list:
             for rlzs in rlzs_by_gsim.values():
@@ -432,7 +452,7 @@ class ClassicalCalculator(base.HazardCalculator):
                 'You have only %s of free RAM' % humansize(avail))
         elif avail < size:
             logging.warning('You have only %s of free RAM' % humansize(avail))
-        self.ct = (self.oqparam.concurrent_tasks or 1) * 2.5
+        self.ct = self.oqparam.concurrent_tasks * 1.5 or 1
         # NB: it is CRITICAL for performance to have shape GNL and not NLG
         # dset[g, :, :] = XXX is fast, dset[:, :, g] = XXX is ultra-slow
         self.datastore.create_dset('_poes', F64, poes_shape)
@@ -447,9 +467,9 @@ class ClassicalCalculator(base.HazardCalculator):
         """
         oq = self.oqparam
         if oq.hazard_calculation_id and not oq.compare_with_classical:
-            with util.read(self.oqparam.hazard_calculation_id) as parent:
+            with datastore.read(self.oqparam.hazard_calculation_id) as parent:
                 self.full_lt = parent['full_lt']
-            self.calc_stats()  # post-processing
+            self.store_stats()  # post-processing
             return {}
 
         assert oq.max_sites_per_tile > oq.max_sites_disagg, (
@@ -472,26 +492,27 @@ class ClassicalCalculator(base.HazardCalculator):
         weights = [rlz.weight for rlz in self.realizations]
         pgetter = getters.PmapGetter(
             self.datastore, weights, self.sitecol.sids, oq.imtls)
-        srcidx = {rec[0]: i for i, rec in enumerate(
-            self.csm.source_info.values())}
+        srcidx = {
+            rec[0]: i for i, rec in enumerate(self.csm.source_info.values())}
         self.haz = Hazard(self.datastore, self.full_lt, pgetter, srcidx)
-        blocks = list(block_splitter(grp_ids, len(self.grp_ids)))
-        for b, block in enumerate(blocks, 1):
-            args = self.get_args(block, self.haz)
-            logging.info('Sending %d tasks', len(args))
-            smap = parallel.Starmap(classical, args, h5=self.datastore.hdf5)
-            smap.monitor.save('srcfilter', self.src_filter())
-            self.datastore.swmr_on()
-            smap.h5 = self.datastore.hdf5
-            pmaps = smap.reduce(self.agg_dicts)
-            logging.debug("busy time: %s", smap.busytime)
-            self.haz.store_disagg(pmaps)
+        args = self.get_args(grp_ids, self.haz.cmakers)
+        logging.info('Sending %d tasks', len(args))
+        smap = parallel.Starmap(classical, args, h5=self.datastore.hdf5)
+        smap.monitor.save('sitecol', self.sitecol)
+        self.datastore.swmr_on()
+        smap.h5 = self.datastore.hdf5
+        pmaps = smap.reduce(self.agg_dicts)
+        logging.debug("busy time: %s", smap.busytime)
+        self.haz.store_disagg(pmaps)
         if not oq.hazard_calculation_id:
             self.haz.store_disagg()
         self.store_info(psd)
         return True
 
     def store_info(self, psd):
+        """
+        Store full_lt, source_info and by_task
+        """
         self.store_rlz_info(self.rel_ruptures)
         source_ids = self.store_source_info(self.calc_times)
         if self.by_task:
@@ -515,14 +536,9 @@ class ClassicalCalculator(base.HazardCalculator):
             numsites = sum(arr[1] for arr in self.calc_times.values())
             logging.info('Total number of contexts: {:_d}'.
                          format(int(self.numctxs)))
-            logging.info('Average number of sites per context: %d',
-                         numsites / self.numctxs)
-        if psd:
-            psdist = max(max(psd.ddic[trt].values()) for trt in psd.ddic)
-            if psdist and self.maxradius >= psdist / 2:
-                logging.warning('The pointsource_distance of %d km is too '
-                                'small compared to a maxradius of %d km',
-                                psdist, self.maxradius)
+            if self.numctxs:
+                logging.info('Average number of sites per context: %d',
+                             numsites / self.numctxs)
         self.calc_times.clear()  # save a bit of memory
 
     def set_psd(self):
@@ -533,7 +549,6 @@ class ClassicalCalculator(base.HazardCalculator):
         mags = self.datastore['source_mags']  # by TRT
         if len(mags) == 0:  # everything was discarded
             raise RuntimeError('All sources were discarded!?')
-        gsims_by_trt = self.full_lt.get_gsims_by_trt()
         mags_by_trt = {}
         for trt in mags:
             mags_by_trt[trt] = mags[trt][()]
@@ -544,35 +559,25 @@ class ClassicalCalculator(base.HazardCalculator):
                 # the sum is zero for {'default': [(1, 0), (10, 0)]}
                 if sum(dic.values()):
                     it = list(dic.items())
-                    md = '%s->%d ... %s->%d' % (it[0] + it[-1])
-                    logging.info('ps_dist %s: %s', trt, md)
-        imts_with_period = [imt for imt in oq.imtls
-                            if imt == 'PGA' or imt.startswith('SA')]
-        imts_ok = len(imts_with_period) == len(oq.imtls)
-        if (imts_ok and psd and psd.suggested()) or (
-                imts_ok and oq.minimum_intensity):
-            aw = get_effect(mags_by_trt, self.sitecol.one(), gsims_by_trt, oq)
-            if psd:
-                dic = {trt: [(float(mag), int(dst))
-                             for mag, dst in psd.ddic[trt].items()]
-                       for trt in psd.ddic if trt != 'default'}
-                logging.info('pointsource_distance=\n%s', pprint.pformat(dic))
-            if len(vars(aw)) > 1:  # more than _extra
-                self.datastore['effect_by_mag_dst'] = aw
+                    dists = {i[1] for i in it}
+                    if len(set(dists)) > 1:
+                        md = '%s->%d ... %s->%d' % (it[0] + it[-1])
+                        logging.info('ps_dist %s: %s', trt, md)
         hint = 1 if self.N <= oq.max_sites_disagg else numpy.ceil(
             self.N / oq.max_sites_per_tile)
         self.params = dict(
             truncation_level=oq.truncation_level,
+            investigation_time=oq.investigation_time,
             imtls=oq.imtls, reqv=oq.get_reqv(),
             pointsource_distance=oq.pointsource_distance,
             shift_hypo=oq.shift_hypo,
             min_weight=oq.min_weight,
-            collapse_level=oq.collapse_level, hint=hint,
+            collapse_level=int(oq.collapse_level), hint=hint,
             max_sites_disagg=oq.max_sites_disagg,
             split_sources=oq.split_sources, af=self.af)
         return psd
 
-    def get_args(self, grp_ids, hazard):
+    def get_args(self, grp_ids, cmakers):
         """
         :returns: a list of Starmap arguments
         """
@@ -581,10 +586,10 @@ class ClassicalCalculator(base.HazardCalculator):
         src_groups = self.csm.src_groups
         tot_weight = 0
         for grp_id in grp_ids:
-            rlzs_by_gsim = hazard.rlzs_by_gsim_list[grp_id]
+            gsims = cmakers[grp_id].gsims
             sg = src_groups[grp_id]
             for src in sg:
-                src.ngsims = len(rlzs_by_gsim)
+                src.ngsims = len(gsims)
                 tot_weight += src.weight
                 if src.code == b'C' and src.num_ruptures > 20_000:
                     msg = ('{} is suspiciously large, containing {:_d} '
@@ -598,22 +603,21 @@ class ClassicalCalculator(base.HazardCalculator):
             int(tot_weight), int(max_weight)))
         self.counts = AccumDict(accum=0)
         for grp_id in grp_ids:
-            rlzs_by_gsim = hazard.rlzs_by_gsim_list[grp_id]
             sg = src_groups[grp_id]
             if sg.atomic:
                 # do not split atomic groups
                 self.counts[grp_id] += 1
-                allargs.append((sg, rlzs_by_gsim, self.params))
+                allargs.append((sg, cmakers[grp_id]))
             else:  # regroup the sources in blocks
-                blks = (groupby(sg, get_source_id).values() if oq.disagg_by_src
-                        else block_splitter(
-                                sg, max_weight, get_weight, sort=True))
+                blks = (groupby(sg, get_source_id).values()
+                        if oq.disagg_by_src else
+                        block_splitter(sg, max_weight, get_weight, sort=True))
                 blocks = list(blks)
                 self.counts[grp_id] += len(blocks)
                 for block in blocks:
                     logging.debug('Sending %d source(s) with weight %d',
                                   len(block), sum(src.weight for src in block))
-                    allargs.append((block, rlzs_by_gsim, self.params))
+                    allargs.append((block, cmakers[grp_id]))
         return allargs
 
     def save_hazard(self, acc, pmap_by_kind):
@@ -659,13 +663,18 @@ class ClassicalCalculator(base.HazardCalculator):
         if (self.oqparam.hazard_calculation_id is None
                 and '_poes' in self.datastore):
             self.datastore.swmr_on()  # needed
-            self.calc_stats()
+            self.store_stats()
 
-    def calc_stats(self):
+    def store_stats(self):
         oq = self.oqparam
+        N = len(self.sitecol.complete)
+        R = len(self.realizations)
+        if oq.individual_curves is None:  # not specified in the job.ini
+            individual_curves = (N == 1) * (R > 1)
+        else:
+            individual_curves = oq.individual_curves
         hstats = oq.hazard_stats()
         # initialize datasets
-        N = len(self.sitecol.complete)
         P = len(oq.poes)
         M = self.M = len(oq.imtls)
         imts = list(oq.imtls)
@@ -674,9 +683,8 @@ class ClassicalCalculator(base.HazardCalculator):
         else:
             L = oq.imtls.size
         L1 = self.L1 = L // M
-        R = len(self.realizations)
         S = len(hstats)
-        if R > 1 and oq.individual_curves or not hstats:
+        if R > 1 and individual_curves or not hstats:
             self.datastore.create_dset('hcurves-rlzs', F32, (N, R, M, L1))
             self.datastore.set_shape_descr(
                 'hcurves-rlzs', site_id=N, rlz_id=R, imt=imts, lvl=L1)
@@ -703,7 +711,7 @@ class ClassicalCalculator(base.HazardCalculator):
         allargs = [  # this list is very fast to generate
             (getters.PmapGetter(
                 dstore, self.weights, t.sids, oq.imtls, oq.poes),
-             N, hstats, oq.individual_curves, oq.max_sites_disagg,
+             N, hstats, individual_curves, oq.max_sites_disagg,
              self.amplifier)
             for t in self.sitecol.split_in_tiles(ct)]
         if self.few_sites:
@@ -808,26 +816,25 @@ def build_hazard(pgetter, N, hstats, individual_curves,
     compute_mon = monitor('compute stats', measuremem=False)
     for sid in pgetter.sids:
         with combine_mon:
-            pcurves = pgetter.get_pcurves(sid)
+            pc = pgetter.get_pcurve(sid)  # shape (L, R)
             if amplifier:
-                pcurves = amplifier.amplify(ampcode[sid], pcurves)
-                # NB: the pcurves have soil levels != IMT levels
-        if sum(pc.array.sum() for pc in pcurves) == 0:  # no data
+                pc = amplifier.amplify(ampcode[sid], pc)
+                # NB: the pcurve have soil levels != IMT levels
+        if pc.array.sum() == 0:  # no data
             continue
         with compute_mon:
             if hstats:
-                arr = numpy.array([pc.array for pc in pcurves])
                 for s, (statname, stat) in enumerate(hstats.items()):
-                    pc = getters.build_stat_curve(arr, imtls, stat, weights)
-                    pmap_by_kind['hcurves-stats'][s][sid] = pc
+                    sc = getters.build_stat_curve(pc, imtls, stat, weights)
+                    pmap_by_kind['hcurves-stats'][s][sid] = sc
                     if poes:
-                        hmap = calc.make_hmap(pc, imtls, poes, sid)
+                        hmap = calc.make_hmap(sc, imtls, poes, sid)
                         pmap_by_kind['hmaps-stats'][s].update(hmap)
             if R > 1 and individual_curves or not hstats:
-                for pmap, pc in zip(pmap_by_kind['hcurves-rlzs'], pcurves):
-                    pmap[sid] = pc
+                for r, pmap in enumerate(pmap_by_kind['hcurves-rlzs']):
+                    pmap[sid] = pc.extract(r)
                 if poes:
-                    for r, pc in enumerate(pcurves):
-                        hmap = calc.make_hmap(pc, imtls, poes, sid)
+                    for r in range(R):
+                        hmap = calc.make_hmap(pc.extract(r), imtls, poes, sid)
                         pmap_by_kind['hmaps-rlzs'][r].update(hmap)
     return pmap_by_kind
