@@ -39,16 +39,21 @@ def classical_damage(riskinputs, param, monitor):
         dictionaries asset_ordinal -> damage(R, L, D)
     """
     crmodel = monitor.read('crmodel')
+    mon = monitor('getting hazard', measuremem=False)
     for ri in riskinputs:
         R = ri.hazard_getter.num_rlzs
         L = len(crmodel.lti)
         D = len(crmodel.damage_states)
         result = AccumDict(accum=numpy.zeros((R, L, D), F32))
-        for out in ri.gen_outputs(crmodel, monitor):
-            r = out.rlzi
-            for l, loss_type in enumerate(crmodel.loss_types):
-                for a, frac in zip(ri.assets['ordinal'], out[loss_type]):
-                    result[a][r, l] = frac
+        with mon:
+            haz = ri.hazard_getter.get_hazard()
+        for taxo, assets in ri.asset_df.groupby('taxonomy'):
+            for rlz in range(R):
+                pcurve = haz.extract(rlz)
+                out = crmodel.get_output(taxo, assets, pcurve, rlz=rlz)
+                for li, loss_type in enumerate(crmodel.loss_types):
+                    for a, frac in zip(assets.ordinal, out[loss_type]):
+                        result[a][rlz, li] = frac
         yield result
 
 
@@ -76,4 +81,5 @@ class ClassicalDamageCalculator(classical_risk.ClassicalRiskCalculator):
                              assets=self.assetcol['id'],
                              loss_type=self.oqparam.loss_names,
                              dmg_state=self.crmodel.damage_states)
-        logging.info('\n' + views.view('portfolio_damage', self.datastore))
+        dmg = views.view('portfolio_damage', self.datastore)
+        logging.info('\n' + views.text_table(dmg, ext='org'))
