@@ -31,9 +31,7 @@ from openquake.baselib.python3compat import encode
 from openquake.hazardlib import stats
 from openquake.hazardlib.calc import disagg
 from openquake.hazardlib.imt import from_string
-from openquake.hazardlib.gsim.base import ContextMaker
-from openquake.hazardlib.contexts import read_ctxs, RuptureContext
-from openquake.hazardlib.tom import PoissonTOM
+from openquake.hazardlib.contexts import read_cmakers
 from openquake.commonlib import util, calc
 from openquake.calculators import getters
 from openquake.calculators import base
@@ -94,7 +92,7 @@ def output(mat6):
     return pprod(mat6, axis=(1, 2)), pprod(mat6, axis=(0, 3))
 
 
-def compute_disagg(dstore, slc, cmaker, hmap4, trti, magi, bin_edges, monitor):
+def compute_disagg(dstore, slc, cmaker, hmap4, magi, bin_edges, monitor):
     # see https://bugs.launchpad.net/oq-engine/+bug/1279247 for an explanation
     # of the algorithm used
     """
@@ -106,8 +104,6 @@ def compute_disagg(dstore, slc, cmaker, hmap4, trti, magi, bin_edges, monitor):
         a :class:`openquake.hazardlib.gsim.base.ContextMaker` instance
     :param hmap4:
         an ArrayWrapper of shape (N, M, P, Z)
-    :param trti:
-        tectonic region type index
     :param magi:
         magnitude bin indices
     :param bin_egdes:
@@ -117,12 +113,9 @@ def compute_disagg(dstore, slc, cmaker, hmap4, trti, magi, bin_edges, monitor):
     :returns:
         a dictionary sid, imti -> 6D-array
     """
-    RuptureContext.temporal_occurrence_model = PoissonTOM(
-        cmaker.investigation_time)
     with monitor('reading contexts', measuremem=True):
         dstore.open('r')
-        allctxs, close_ctxs = read_ctxs(
-            dstore, slc, req_site_params=cmaker.REQUIRES_SITES_PARAMETERS)
+        allctxs = cmaker.read_ctxs(dstore, slc)
         for magidx, ctx in zip(magi, allctxs):
             ctx.magi = magidx
     dis_mon = monitor('disaggregate', measuremem=False)
@@ -136,15 +129,14 @@ def compute_disagg(dstore, slc, cmaker, hmap4, trti, magi, bin_edges, monitor):
     eps3 = disagg._eps3(cmaker.trunclevel, cmaker.num_epsilon_bins)
     imts = [from_string(im) for im in cmaker.imtls]
     for magi, ctxs in groupby(allctxs, operator.attrgetter('magi')).items():
-        res = {'trti': trti, 'magi': magi}
+        res = {'trti': cmaker.trti, 'magi': magi}
         with ms_mon:
-            # compute mean and std for a single IMT to save memory
-            # the size is N * U * G * 16 bytes
-            disagg.set_mean_std(ctxs, imts, cmaker.gsims)
+            # compute mean and std (N * U * M * G * 16 bytes)
+            disagg.set_mean_std(ctxs, cmaker)
 
         # disaggregate by site, IMT
         for s, iml3 in enumerate(hmap4):
-            close = [ctx for ctx in close_ctxs[s] if ctx.magi == magi]
+            close = [ctx for ctx in ctxs if ctx.magi == magi and s in ctx.sids]
             if not g_by_z[s] or not close:
                 # g_by_z[s] is empty in test case_7
                 continue
@@ -154,8 +146,8 @@ def compute_disagg(dstore, slc, cmaker, hmap4, trti, magi, bin_edges, monitor):
             iml2 = dict(zip(imts, iml3))
             with dis_mon:
                 # 7D-matrix #distbins, #lonbins, #latbins, #epsbins, M, P, Z
-                matrix = disagg.disaggregate(
-                    close, g_by_z[s], iml2, eps3, s, bins)  # 7D-matrix
+                matrix = disagg.disaggregate(close, cmaker.tom, g_by_z[s],
+                                             iml2, eps3, s, bins)  # 7D-matrix
                 for m in range(M):
                     mat6 = matrix[..., m, :, :]
                     if mat6.any():
@@ -241,9 +233,9 @@ class DisaggregationCalculator(base.HazardCalculator):
         :returns: a list of Z arrays of PoEs
         """
         poes = []
-        pcurves = self.pgetter.get_pcurves(sid)
+        pcurve = self.pgetter.get_pcurve(sid)
         for z, rlz in enumerate(rlzs):
-            pc = pcurves[rlz]
+            pc = pcurve.extract(rlz)
             if z == 0:
                 self.curves.append(pc.array[:, 0])
             poes.append(pc.convert(self.oqparam.imtls))
@@ -273,12 +265,12 @@ class DisaggregationCalculator(base.HazardCalculator):
             rlzs = numpy.zeros((self.N, Z), int)
             if self.R > 1:
                 for sid in self.sitecol.sids:
-                    curves = numpy.array(
-                        [pc.array for pc in self.pgetter.get_pcurves(sid)])
+                    pcurve = self.pgetter.get_pcurve(sid)
                     mean = getters.build_stat_curve(
-                        curves, oq.imtls, stats.mean_curve, ws)
+                        pcurve, oq.imtls, stats.mean_curve, ws)
                     # get the closest realization to the mean
-                    rlzs[sid] = util.closest_to_ref(curves, mean.array)[:Z]
+                    rlzs[sid] = util.closest_to_ref(
+                        pcurve.array.T, mean.array)[:Z]
             self.datastore['best_rlzs'] = rlzs
         else:
             Z = len(oq.rlz_index)
@@ -323,50 +315,41 @@ class DisaggregationCalculator(base.HazardCalculator):
         rdata['magi'] = magi
         rdata['idx'] = numpy.arange(totrups)
         rdata['grp_id'] = dstore['rup/grp_id'][:]
-        rdata['nsites'] = dstore['rup/nsites'][:]
+        rdata['nsites'] = [len(sids) for sids in dstore['rup/sids_']]
         totweight = rdata['nsites'].sum()
-        et_ids = dstore['et_ids'][:]
-        rlzs_by_gsim = self.full_lt.get_rlzs_by_gsim_list(et_ids)
+        trt_smrs = dstore['trt_smrs'][:]
+        rlzs_by_gsim = self.full_lt.get_rlzs_by_gsim_list(trt_smrs)
         G = max(len(rbg) for rbg in rlzs_by_gsim)
         maxw = 2 * 1024**3 / (16 * G * self.M)  # at max 2 GB
         maxweight = min(
             numpy.ceil(totweight / (oq.concurrent_tasks or 1)), maxw)
-        num_eff_rlzs = len(self.full_lt.sm_rlzs)
         task_inputs = []
         U = 0
         self.datastore.swmr_on()
         smap = parallel.Starmap(compute_disagg, h5=self.datastore.hdf5)
-        # ABSURDLY IMPORTANT!! we rely on the fact that the classical part
+        # IMPORTANT!! we rely on the fact that the classical part
         # of the calculation stores the ruptures in chunks of constant
         # grp_id, therefore it is possible to build (start, stop) slices;
         # we are NOT grouping by operator.itemgetter('grp_id', 'magi'):
         # that would break the ordering of the indices causing an incredibly
         # worse performance, but visible only in extra-large calculations!
+        cmakers = read_cmakers(self.datastore)
         for block in block_splitter(rdata, maxweight,
                                     operator.itemgetter('nsites'),
                                     operator.itemgetter('grp_id')):
             grp_id = block[0]['grp_id']
-            trti = et_ids[grp_id][0] // num_eff_rlzs
-            trt = self.trts[trti]
-            cmaker = ContextMaker(
-                trt, rlzs_by_gsim[grp_id],
-                {'truncation_level': oq.truncation_level,
-                 'maximum_distance': oq.maximum_distance,
-                 'collapse_level': oq.collapse_level,
-                 'num_epsilon_bins': oq.num_epsilon_bins,
-                 'investigation_time': oq.investigation_time,
-                 'imtls': oq.imtls})
+            cmaker = cmakers[grp_id]
             U = max(U, block.weight)
             slc = slice(block[0]['idx'], block[-1]['idx'] + 1)
-            smap.submit((dstore, slc, cmaker, self.hmap4, trti, magi[slc],
-                         self.bin_edges))
-            task_inputs.append((trti, slc.stop-slc.start))
+            smap.submit((dstore, slc, cmaker, self.hmap4,
+                         magi[slc], self.bin_edges))
+            task_inputs.append((cmaker.trti, slc.stop-slc.start))
 
         nbytes, msg = get_nbytes_msg(dict(M=self.M, G=G, U=U, F=2))
         logging.info('Maximum mean_std per task:\n%s', msg)
 
         s = self.shapedic
-        Ta = numpy.ceil(len(task_inputs))
+        Ta = len(task_inputs)
         nbytes = s['N'] * s['M'] * s['P'] * s['Z'] * Ta * 8
         data_transfer = (s['dist'] * s['eps'] + s['lon'] * s['lat']) * nbytes
         if data_transfer > oq.max_data_transfer:
@@ -454,11 +437,13 @@ class DisaggregationCalculator(base.HazardCalculator):
         _disagg_trt = numpy.zeros(self.N, [(trt, float) for trt in self.trts])
         vcurves = []  # hazard curves with a vertical section for large poes
         for (s, m, k), mat6 in sorted(results.items()):
+            # NB: k is an index with value 0 (MagDistEps) or 1 (MagLonLat)
             imt = self.imts[m]
             for p, poe in enumerate(self.poes_disagg):
                 mat5 = mat6[..., p, :]
+                # mat5 has shape (T, Ma, D, E, Z) for k == 0
+                # and (T, Ma, Lo, La, Z) for k == 1
                 if k == 0 and m == 0 and poe == self.poes_disagg[-1]:
-                    # mat5 has shape (T, Ma, D, E, Z)
                     _disagg_trt[s] = tuple(pprod(mat5[..., 0], axis=(1, 2, 3)))
                 poe2 = pprod(mat5, axis=(0, 1, 2, 3))
                 self.datastore['poe4'][s, m, p] = poe2  # shape Z
