@@ -19,7 +19,6 @@
 import os.path
 import logging
 import numpy
-import psutil
 
 from openquake.baselib import hdf5, parallel
 from openquake.baselib.general import AccumDict, copyobj, humansize
@@ -30,12 +29,13 @@ from openquake.hazardlib.gsim.base import ContextMaker
 from openquake.hazardlib.calc.filters import nofilter
 from openquake.hazardlib import InvalidFile
 from openquake.hazardlib.calc.stochastic import get_rup_array, rupture_dt
-from openquake.hazardlib.source.rupture import EBRupture
-from openquake.commonlib import calc, util, logs, readinput, logictree
+from openquake.hazardlib.source.rupture import EBRupture, get_ruptures
+from openquake.commonlib import (
+    calc, util, logs, readinput, logictree, datastore)
 from openquake.risklib.riskinput import str2rsi
 from openquake.calculators import base, views
 from openquake.calculators.getters import (
-    GmfGetter, gen_rupture_getters, sig_eps_dt, time_dt)
+    GmfGetter, get_rupture_getters, sig_eps_dt, time_dt)
 from openquake.calculators.classical import ClassicalCalculator
 from openquake.engine import engine
 
@@ -63,6 +63,13 @@ def get_mean_curves(dstore, imt):
 # ########################################################################## #
 
 
+def count_ruptures(src):
+    """
+    Count the number of ruptures on a heavy source
+    """
+    return {src.source_id: src.count_ruptures()}
+
+
 def compute_gmfs(rupgetter, param, monitor):
     """
     Compute GMFs and optionally hazard curves
@@ -119,23 +126,36 @@ class EventBasedCalculator(base.HazardCalculator):
 
     def acc0(self):
         """
-        Initial accumulator, a dictionary (et_id, gsim) -> curves
+        Initial accumulator, a dictionary rlz -> ProbabilityMap
         """
         self.L = self.oqparam.imtls.size
-        zd = {r: ProbabilityMap(self.L) for r in range(self.R)}
-        return zd
+        return {r: ProbabilityMap(self.L) for r in range(self.R)}
 
     def build_events_from_sources(self):
         """
         Prefilter the composite source model and store the source_info
         """
+        oq = self.oqparam
+        params = dict(maximum_distance=oq.maximum_distance,
+                      imtls=oq.imtls,
+                      ses_per_logic_tree_path=oq.ses_per_logic_tree_path,
+                      ses_seed=oq.ses_seed)
         gsims_by_trt = self.csm.full_lt.get_gsims_by_trt()
-        logging.info('Building ruptures')
-        for src in self.csm.get_sources():
+        sources = self.csm.get_sources()
+        # weighting the heavy sources
+        nrups = parallel.Starmap(
+            count_ruptures, [(src,) for src in sources if src.code in b'AMC'],
+            progress=logging.debug
+        ).reduce()
+        for src in sources:
             src.nsites = 1  # avoid 0 weight
+            try:
+                src.num_ruptures = nrups[src.source_id]
+            except KeyError:
+                src.num_ruptures = src.count_ruptures()
         maxweight = sum(sg.weight for sg in self.csm.src_groups) / (
             self.oqparam.concurrent_tasks or 1)
-        eff_ruptures = AccumDict(accum=0)  # trt => potential ruptures
+        eff_ruptures = AccumDict(accum=0)  # grp_id => potential ruptures
         calc_times = AccumDict(accum=numpy.zeros(3, F32))  # nr, ns, dt
         allargs = []
         if self.oqparam.is_ucerf():
@@ -146,18 +166,18 @@ class EventBasedCalculator(base.HazardCalculator):
             srcfilter = nofilter  # otherwise it would be ultra-slow
         else:
             srcfilter = self.srcfilter
+        logging.info('Building ruptures')
         for sg in self.csm.src_groups:
             if not sg.sources:
                 continue
             logging.info('Sending %s', sg)
-            par = self.param.copy()
-            par['gsims'] = gsims_by_trt[sg.trt]
+            cmaker = ContextMaker(sg.trt, gsims_by_trt[sg.trt], params)
             for src_group in sg.split(maxweight):
-                allargs.append((src_group, srcfilter, par))
+                allargs.append((src_group, cmaker, srcfilter.sitecol))
         smap = parallel.Starmap(
             sample_ruptures, allargs, h5=self.datastore.hdf5)
         mon = self.monitor('saving ruptures')
-        self.nruptures = 0
+        self.nruptures = 0  # estimated classical ruptures within maxdist
         for dic in smap:
             # NB: dic should be a dictionary, but when the calculation dies
             # for an OOM it can become None, thus giving a very confusing error
@@ -186,7 +206,8 @@ class EventBasedCalculator(base.HazardCalculator):
         self.store_source_info(calc_times)
         imp = calc.RuptureImporter(self.datastore)
         with self.monitor('saving ruptures and events'):
-            imp.import_rups(self.datastore.getitem('ruptures')[()])
+            imp.import_rups_events(
+                self.datastore.getitem('ruptures')[()], get_rupture_getters)
 
     def agg_dicts(self, acc, result):
         """
@@ -200,10 +221,12 @@ class EventBasedCalculator(base.HazardCalculator):
         with sav_mon:
             df = result.pop('gmfdata')
             if len(df):
+                dset = self.datastore['gmf_data/sid']
                 times = result.pop('times')
+                [task_no] = numpy.unique(times['task_no'])
                 rupids = list(times['rup_id'])
                 self.datastore['gmf_data/time_by_rup'][rupids] = times
-                hdf5.extend(self.datastore['gmf_data/sid'], df.sid.to_numpy())
+                hdf5.extend(dset, df.sid.to_numpy())
                 hdf5.extend(self.datastore['gmf_data/eid'], df.eid.to_numpy())
                 for m in range(len(primary)):
                     hdf5.extend(self.datastore[f'gmf_data/gmv_{m}'],
@@ -244,24 +267,41 @@ class EventBasedCalculator(base.HazardCalculator):
         oq = self.oqparam
         gsim_lt = readinput.get_gsim_lt(self.oqparam)
         G = gsim_lt.get_num_paths()
-        if oq.inputs['rupture_model'].endswith('.xml'):
+        if oq.calculation_mode.startswith('scenario'):
             ngmfs = oq.number_of_ground_motion_fields
-            self.gsims = [gsim_rlz.value[0] for gsim_rlz in gsim_lt]
+        if oq.inputs['rupture_model'].endswith('.xml'):
+            # check the number of branchsets
+            bsets = len(gsim_lt._ltnode)
+            if bsets > 1:
+                raise InvalidFile(
+                    '%s for a scenario calculation must contain a single '
+                    'branchset, found %d!' % (oq.inputs['job_ini'], bsets))
+            [(trt, rlzs_by_gsim)] = gsim_lt.get_rlzs_by_gsim_trt().items()
             self.cmaker = ContextMaker(
-                '*', self.gsims, {'maximum_distance': oq.maximum_distance,
-                                  'imtls': oq.imtls})
+                trt, rlzs_by_gsim, {'maximum_distance': oq.maximum_distance,
+                                    'minimum_distance': oq.minimum_distance,
+                                    'truncation_level': oq.truncation_level,
+                                    'imtls': oq.imtls})
             rup = readinput.get_rupture(oq)
             if self.N > oq.max_sites_disagg:  # many sites, split rupture
                 ebrs = [EBRupture(copyobj(rup, rup_id=rup.rup_id + i),
-                                  0, 0, G, e0=i * G) for i in range(ngmfs)]
+                                  'NA', 0, G, e0=i * G, scenario=True)
+                        for i in range(ngmfs)]
             else:  # keep a single rupture with a big occupation number
-                ebrs = [EBRupture(rup, 0, 0, G * ngmfs, rup.rup_id)]
+                ebrs = [EBRupture(rup, 'NA', 0, G * ngmfs, rup.rup_id,
+                                  scenario=True)]
             aw = get_rup_array(ebrs, self.srcfilter)
             if len(aw) == 0:
-                raise RuntimeError('The rupture is too far from the sites!')
+                raise RuntimeError(
+                    'The rupture is too far from the sites! Please check the '
+                    'maximum_distance and the position of the rupture')
         elif oq.inputs['rupture_model'].endswith('.csv'):
-            aw = readinput.get_ruptures(oq.inputs['rupture_model'])
-            aw.array['n_occ'] = G
+            aw = get_ruptures(oq.inputs['rupture_model'])
+            if oq.calculation_mode.startswith('scenario'):
+                # rescale n_occ by ngmfs and nrlzs
+                aw['n_occ'] *= ngmfs * gsim_lt.get_num_paths()
+        else:
+            raise InvalidFile("Something wrong in %s" % oq.inputs['job_ini'])
         rup_array = aw.array
         hdf5.extend(self.datastore['rupgeoms'], aw.geom)
 
@@ -271,26 +311,21 @@ class EventBasedCalculator(base.HazardCalculator):
                 ' of %s km from the rupture' % oq.maximum_distance(
                     rup.tectonic_region_type, rup.mag))
 
-        # check the number of branchsets
-        branchsets = len(gsim_lt._ltnode)
-        if len(rup_array) == 1 and branchsets > 1:
-            raise InvalidFile(
-                '%s for a scenario calculation must contain a single '
-                'branchset, found %d!' % (oq.inputs['job_ini'], branchsets))
-
         fake = logictree.FullLogicTree.fake(gsim_lt)
         self.realizations = fake.get_realizations()
         self.datastore['full_lt'] = fake
         self.store_rlz_info({})  # store weights
         self.save_params()
-        calc.RuptureImporter(self.datastore).import_rups(rup_array)
+        imp = calc.RuptureImporter(self.datastore)
+        imp.import_rups_events(rup_array, get_rupture_getters)
 
     def execute(self):
         oq = self.oqparam
+        dstore = self.datastore
         self.set_param()
         self.offset = 0
         if oq.hazard_calculation_id:  # from ruptures
-            self.datastore.parent = util.read(oq.hazard_calculation_id)
+            dstore.parent = datastore.read(oq.hazard_calculation_id)
         elif hasattr(self, 'csm'):  # from sources
             self.build_events_from_sources()
             if (oq.ground_motion_fields is False and
@@ -301,40 +336,35 @@ class EventBasedCalculator(base.HazardCalculator):
                 'There is no rupture_model, the calculator will just '
                 'import data without performing any calculation')
             fake = logictree.FullLogicTree.fake()
-            self.datastore['full_lt'] = fake  # needed to expose the outputs
-            self.datastore['weights'] = [1.]
+            dstore['full_lt'] = fake  # needed to expose the outputs
+            dstore['weights'] = [1.]
             return {}
         else:  # scenario
             self._read_scenario_ruptures()
             if (oq.ground_motion_fields is False and
                     oq.hazard_curves_from_gmfs is False):
                 return {}
-        N = len(self.sitecol.complete)
+
         if oq.ground_motion_fields:
-            M = len(oq.get_primary_imtls())
-            nrups = len(self.datastore['ruptures'])
-            base.create_gmf_data(self.datastore, M, oq.get_sec_imts())
-            self.datastore.create_dset('gmf_data/sigma_epsilon',
-                                       sig_eps_dt(oq.imtls))
-            self.datastore.create_dset('gmf_data/events_by_sid', U32, (N,))
-            self.datastore.create_dset('gmf_data/time_by_rup',
-                                       time_dt, (nrups,), fillvalue=None)
+            imts = oq.get_primary_imtls()
+            nrups = len(dstore['ruptures'])
+            base.create_gmf_data(dstore, imts, oq.get_sec_imts())
+            dstore.create_dset('gmf_data/sigma_epsilon', sig_eps_dt(oq.imtls))
+            dstore.create_dset('gmf_data/time_by_rup',
+                               time_dt, (nrups,), fillvalue=None)
 
         # compute_gmfs in parallel
-        nr = len(self.datastore['ruptures'])
+        nr = len(dstore['ruptures'])
         logging.info('Reading {:_d} ruptures'.format(nr))
-        allargs = [(rgetter, self.param)
-                   for rgetter in gen_rupture_getters(
-                           self.datastore, oq.concurrent_tasks)]
-        # reading the args is fast since we are not prefiltering the ruptures,
-        # nor reading the geometries; using an iterator would cause the usual
-        # damned h5py error, last seen on macos
-        self.datastore.swmr_on()
+        rgetters = get_rupture_getters(dstore, oq.concurrent_tasks * 1.25,
+                                       srcfilter=self.srcfilter)
+        allargs = [(rgetter, self.param) for rgetter in rgetters]
+        dstore.swmr_on()
         smap = parallel.Starmap(
-            self.core_task.__func__, allargs, h5=self.datastore.hdf5)
+            self.core_task.__func__, allargs, h5=dstore.hdf5)
         smap.monitor.save('srcfilter', self.srcfilter)
         acc = smap.reduce(self.agg_dicts, self.acc0())
-        if 'gmf_data' not in self.datastore:
+        if 'gmf_data' not in dstore:
             return acc
         if oq.ground_motion_fields:
             with self.monitor('saving avg_gmf', measuremem=True):
@@ -347,9 +377,9 @@ class EventBasedCalculator(base.HazardCalculator):
         """
         size = self.datastore.getsize('gmf_data')
         logging.info(f'Stored {humansize(size)} of GMFs')
-        if size > 1024**3:
+        if size > 100 * 1024**2:
             logging.warning(
-                'There are more than 1 GB of GMFs, not computing avg_gmf')
+                'There are more than 100 MB of GMFs, not computing avg_gmf')
             return numpy.unique(self.datastore['gmf_data/eid'][:])
 
         rlzs = self.datastore['events']['rlz_id']
@@ -386,13 +416,11 @@ class EventBasedCalculator(base.HazardCalculator):
         L = oq.imtls.size
         L1 = L // (M or 1)
         # check seed dependency unless the number of GMFs is huge
-        if ('gmf_data' in self.datastore and
-                self.datastore.getsize('gmf_data') < 1E9):
-            logging.info('Checking seed dependency')
-            err = views.view('gmf_error', self.datastore)
-            if err > .05:
-                logging.warning('Your results are expected to have a large '
-                                'dependency from ses_seed')
+        if 'gmf_data' in self.datastore and self.datastore.getsize(
+                'gmf_data/gmv_0') < 4E9:
+            logging.info('Checking stored GMFs')
+            msg = views.view('extreme_gmvs', self.datastore)
+            logging.warning(msg)
         if oq.hazard_curves_from_gmfs:
             rlzs = self.datastore['full_lt'].get_realizations()
             # compute and save statistics; this is done in process and can
@@ -466,19 +494,20 @@ class EventBasedCalculator(base.HazardCalculator):
             if not os.path.exists(export_dir):
                 os.makedirs(export_dir)
             oq.export_dir = export_dir
-            job_id = logs.init('job')
             oq.calculation_mode = 'classical'
-            self.cl = ClassicalCalculator(oq, job_id)
-            # TODO: perhaps it is possible to avoid reprocessing the source
-            # model, however usually this is quite fast and do not dominate
-            # the computation
-            self.cl.run()
-            engine.expose_outputs(self.datastore)
-            for imt in oq.imtls:
-                cl_mean_curves = get_mean_curves(self.datastore, imt)
-                eb_mean_curves = get_mean_curves(self.datastore, imt)
-                self.rdiff, index = util.max_rel_diff_index(
-                    cl_mean_curves, eb_mean_curves)
-                logging.warning('Relative difference with the classical '
-                                'mean curves: %d%% at site index %d, imt=%s',
-                                self.rdiff * 100, index, imt)
+            with logs.init('job', vars(oq)) as log:
+                self.cl = ClassicalCalculator(oq, log.calc_id)
+                # TODO: perhaps it is possible to avoid reprocessing the source
+                # model, however usually this is quite fast and do not dominate
+                # the computation
+                self.cl.run()
+                engine.expose_outputs(self.cl.datastore)
+                for imt in oq.imtls:
+                    cl_mean_curves = get_mean_curves(self.datastore, imt)
+                    eb_mean_curves = get_mean_curves(self.datastore, imt)
+                    self.rdiff, index = util.max_rel_diff_index(
+                        cl_mean_curves, eb_mean_curves)
+                    logging.warning(
+                        'Relative difference with the classical '
+                        'mean curves: %d%% at site index %d, imt=%s',
+                        self.rdiff * 100, index, imt)
