@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 from openquake.baselib.python3compat import decode
-from openquake.commonlib import writers
+from openquake.baselib import writers
 from openquake.calculators.post_risk import get_loss_builder
 
 
