@@ -21,8 +21,9 @@ import logging
 import itertools
 import numpy
 
-from openquake.baselib import general, datastore, parallel, python3compat
+from openquake.baselib import general, parallel, python3compat
 from openquake.hazardlib.stats import set_rlzs_stats
+from openquake.commonlib import datastore
 from openquake.risklib import scientific
 from openquake.calculators import base, views
 
@@ -83,14 +84,27 @@ def get_loss_builder(dstore, return_periods=None, loss_dt=None):
     """
     oq = dstore['oqparam']
     weights = dstore['weights'][()]
-    eff_time = oq.investigation_time * oq.ses_per_logic_tree_path
-    num_events = numpy.bincount(dstore['events']['rlz_id'])
+    try:
+        haz_time = dstore['gmf_data'].attrs['effective_time']
+    except KeyError:
+        haz_time = None
+    eff_time = oq.investigation_time * oq.ses_per_logic_tree_path * (
+        len(weights) if oq.collect_rlzs else 1)
+    if oq.collect_rlzs:
+        if haz_time and haz_time != eff_time:
+            raise ValueError('The effective time stored in gmf_data is %d, '
+                             'which is inconsistent with %d' %
+                             (haz_time, eff_time))
+        num_events = numpy.array([len(dstore['events'])])
+        weights = numpy.ones(1)
+    else:
+        num_events = numpy.bincount(dstore['events']['rlz_id'])
     periods = return_periods or oq.return_periods or scientific.return_periods(
         eff_time, num_events.max())
     return scientific.LossCurvesMapsBuilder(
         oq.conditional_loss_poes, numpy.array(periods),
         loss_dt or oq.loss_dt(), weights, dict(enumerate(num_events)),
-        eff_time, oq.risk_investigation_time)
+        eff_time, oq.risk_investigation_time or oq.investigation_time)
 
 
 def get_src_loss_table(dstore, L):
@@ -98,8 +112,8 @@ def get_src_loss_table(dstore, L):
     :returns:
         (source_ids, array of losses of shape (Ns, L))
     """
-    K = dstore['agg_loss_table'].attrs.get('K', 0)
-    alt = dstore.read_df('agg_loss_table', 'agg_id', dict(agg_id=K))
+    K = dstore['risk_by_event'].attrs.get('K', 0)
+    alt = dstore.read_df('risk_by_event', 'agg_id', dict(agg_id=K))
     eids = alt.event_id.to_numpy()
     evs = dstore['events'][:][eids]
     rlz_ids = evs['rlz_id']
@@ -107,20 +121,19 @@ def get_src_loss_table(dstore, L):
     source_id = python3compat.decode(dstore['ruptures']['source_id'][rup_ids])
     w = dstore['weights'][:]
     acc = general.AccumDict(accum=numpy.zeros(L, F32))
-    del alt['event_id']
-    all_losses = numpy.array(alt)
-    for source_id, rlz_id, losses in zip(source_id, rlz_ids, all_losses):
-        acc[source_id] += losses * w[rlz_id]
+    for source_id, rlz_id, loss_id, loss in zip(
+            source_id, rlz_ids, alt.loss_id.to_numpy(), alt.loss.to_numpy()):
+        acc[source_id][loss_id] += loss * w[rlz_id]
     return zip(*sorted(acc.items()))
 
 
-def post_risk(builder, kr_losses, monitor):
+def post_risk(builder, krl_losses, monitor):
     """
-    :returns: dictionary kr -> L loss curves
+    :returns: dictionary krl -> loss curve
     """
     res = {}
-    for k, r, losses in kr_losses:
-        res[k, r] = builder.build_curves(losses, r)
+    for k, r, l, losses in krl_losses:
+        res[k, r, l] = builder.build_curve(losses, r)
     return res
 
 
@@ -139,7 +152,7 @@ class PostRiskCalculator(base.RiskCalculator):
             self.aggkey = base.save_agg_values(
                 ds, assetcol, oq.loss_names, oq.aggregate_by)
             aggby = ds.parent['oqparam'].aggregate_by
-            self.reaggreate = oq.aggregate_by != aggby
+            self.reaggreate = aggby and oq.aggregate_by != aggby
             if self.reaggreate:
                 self.num_tags = dict(
                     zip(aggby, assetcol.tagcol.agg_shape(aggby)))
@@ -147,8 +160,8 @@ class PostRiskCalculator(base.RiskCalculator):
             assetcol = ds['assetcol']
             self.aggkey = assetcol.tagcol.get_aggkey(oq.aggregate_by)
         self.L = len(oq.loss_names)
-        size = general.humansize(ds.getsize('agg_loss_table'))
-        logging.info('Stored %s in the agg_loss_table', size)
+        size = general.humansize(ds.getsize('risk_by_event'))
+        logging.info('Stored %s in the risk_by_event', size)
 
     def execute(self):
         oq = self.oqparam
@@ -173,46 +186,42 @@ class PostRiskCalculator(base.RiskCalculator):
         P = len(builder.return_periods)
         # do everything in process since it is really fast
         rlz_id = self.datastore['events']['rlz_id']
-        alt_df = self.datastore.read_df('agg_loss_table')
+        if oq.collect_rlzs:
+            rlz_id = numpy.zeros_like(rlz_id)
+        alt_df = self.datastore.read_df('risk_by_event')
         if self.reaggreate:
             idxs = numpy.concatenate([
                 reagg_idxs(self.num_tags, oq.aggregate_by),
                 numpy.array([K], int)])
             alt_df['agg_id'] = idxs[alt_df['agg_id'].to_numpy()]
-            alt_df = alt_df.groupby(['event_id', 'agg_id']).sum().reset_index()
+            alt_df = alt_df.groupby(
+                ['event_id', 'loss_id', 'agg_id']).sum().reset_index()
         alt_df['rlz_id'] = rlz_id[alt_df.event_id.to_numpy()]
         units = self.datastore['cost_calculator'].get_units(oq.loss_names)
-        dist = ('no' if os.environ.get('OQ_DISTRIBUTE') == 'no'
-                else 'processpool')  # use only the local cores
-        smap = parallel.Starmap(post_risk, h5=self.datastore.hdf5,
-                                distribute=dist)
+        smap = parallel.Starmap(post_risk, h5=self.datastore.hdf5)
         # producing concurrent_tasks/2 = num_cores tasks
         blocksize = int(numpy.ceil(
             (K + 1) * self.R / (oq.concurrent_tasks // 2 or 1)))
-        kr_losses = []
+        krl_losses = []
         agg_losses = numpy.zeros((K + 1, self.R, self.L), F32)
         agg_curves = numpy.zeros((K + 1, self.R, self.L, P), F32)
-        gb = alt_df.groupby([alt_df.agg_id, alt_df.rlz_id])
+        gb = alt_df.groupby([alt_df.agg_id, alt_df.rlz_id, alt_df.loss_id])
         # NB: in the future we may use multiprocessing.shared_memory
-        for (k, r), df in gb:
-            arr = numpy.zeros((self.L, len(df)), F32)
-            for lni, ln in enumerate(oq.loss_names):
-                arr[lni] = df[ln].to_numpy()
-            agg_losses[k, r] = arr.sum(axis=1)
-            kr_losses.append((k, r, arr))
-            if len(kr_losses) >= blocksize:
-                size = sum(ls.nbytes for k, r, ls in kr_losses)
-                logging.info('Sending %s of losses',
-                             general.humansize(size))
-                smap.submit((builder, kr_losses))
-                kr_losses[:] = []
-        if kr_losses:
-            smap.submit((builder, kr_losses))
-        for (k, r), curve in smap.reduce().items():
-            agg_curves[k, r] = curve
-        self.datastore['agg_losses-rlzs'] = agg_losses * oq.ses_ratio
+        for (k, r, lni), df in gb:
+            agg_losses[k, r, lni] = df.loss.sum()
+            krl_losses.append((k, r, lni, df.loss.to_numpy()))
+            if len(krl_losses) >= blocksize:
+                smap.submit((builder, krl_losses))
+                krl_losses[:] = []
+        if krl_losses:
+            smap.submit((builder, krl_losses))
+        for krl, curve in smap.reduce().items():
+            agg_curves[krl] = curve
+        R = len(self.datastore['weights'])
+        time_ratio = oq.time_ratio / R if oq.collect_rlzs else oq.time_ratio
+        self.datastore['agg_losses-rlzs'] = agg_losses * time_ratio
         set_rlzs_stats(self.datastore, 'agg_losses',
-                       agg_id=K + 1, loss_types=oq.loss_names, units=units)
+                       agg_id=K + 1, loss_type=oq.loss_names, units=units)
         self.datastore['agg_curves-rlzs'] = agg_curves
         set_rlzs_stats(self.datastore, 'agg_curves',
                        agg_id=K + 1, lti=self.L,
@@ -222,14 +231,22 @@ class PostRiskCalculator(base.RiskCalculator):
 
     def post_execute(self, dummy):
         """
-        Sanity check on tot_losses
+        Sanity checks
         """
         logging.info('Total portfolio loss\n' +
                      views.view('portfolio_loss', self.datastore))
+        for li, ln in enumerate(self.oqparam.loss_names):
+            dloss = views.view('delta_loss:%d' % li, self.datastore)
+            if dloss['delta'].mean() > .1:  # more than 10% variation
+                logging.warning(
+                    'A big variation in the %s loss curve is expected: try\n'
+                    '$ oq show delta_loss:%d %d', ln, li,
+                    self.datastore.calc_id)
         if not self.aggkey:
             return
         logging.info('Sanity check on agg_losses')
         for kind in 'rlzs', 'stats':
+            avg = 'avg_losses-' + kind
             agg = 'agg_losses-' + kind
             if agg not in self.datastore:
                 return
@@ -237,14 +254,25 @@ class PostRiskCalculator(base.RiskCalculator):
                 kinds = ['rlz-%d' % rlz for rlz in range(self.R)]
             else:
                 kinds = self.oqparam.hazard_stats()
-            for l in range(self.L):
-                ln = self.oqparam.loss_names[l]
+            for li in range(self.L):
+                ln = self.oqparam.loss_names[li]
                 for r, k in enumerate(kinds):
-                    tot_losses = self.datastore[agg][-1, r, l]
-                    agg_losses = self.datastore[agg][:-1, r, l].sum()
+                    tot_losses = self.datastore[agg][-1, r, li]
+                    agg_losses = self.datastore[agg][:-1, r, li].sum()
                     if kind == 'rlzs' or k == 'mean':
-                        ok = numpy.allclose(agg_losses, tot_losses, rtol=.001)
-                        if not ok:
+                        if not numpy.allclose(
+                                agg_losses, tot_losses, rtol=.001):
                             logging.warning(
                                 'Inconsistent total losses for %s, %s: '
                                 '%s != %s', ln, k, agg_losses, tot_losses)
+                        try:
+                            avg_losses = self.datastore[avg][:, r, li]
+                        except KeyError:
+                            continue
+                        # check on the sum of the average losses
+                        sum_losses = avg_losses.sum()
+                        if not numpy.allclose(
+                                sum_losses, tot_losses, rtol=.001):
+                            logging.warning(
+                                'Inconsistent sum_losses for %s, %s: '
+                                '%s != %s', ln, k, sum_losses, tot_losses)
