@@ -18,7 +18,7 @@
 import sys
 import collections
 import numpy
-from openquake.commonlib import util
+from openquake.commonlib import datastore
 from openquake.calculators.extract import Extractor
 from openquake.calculators import views
 
@@ -64,7 +64,7 @@ class Comparator(object):
         for ex in self.extractors:
             time = ex.get('performance_data')['time_sec'].sum()
             data.append((ex.calc_id, time))
-        print(views.rst_table(data, ['calc_id', 'time']))
+        print(views.text_table(data, ['calc_id', 'time']))
 
     def getsids(self, samplesites):
         sids = self.sitecol['sids']
@@ -137,7 +137,7 @@ class Comparator(object):
         if len(diff_idxs) == 0:
             print('There are no differences within the tolerances '
                   'atol=%s, rtol=%d%%, sids=%s' % (atol, rtol * 100, sids))
-            return
+            return []
         arr = arrays.transpose(1, 0, 2)  # shape (N, C, L)
         for sid, array in sorted(zip(sids[diff_idxs], arr[diff_idxs])):
             for ex, cols in zip(self.extractors, array):
@@ -149,10 +149,10 @@ class Comparator(object):
             fdict = {ex.calc_id: open('%s.txt' % ex.calc_id, 'w')
                      for ex in self.extractors}
             for calc_id, f in fdict.items():
-                f.write(views.rst_table(rows[calc_id], header))
+                f.write(views.text_table(rows[calc_id], header))
                 print('Generated %s' % f.name)
         else:
-            print(views.rst_table(rows['all'], header))
+            print(views.text_table(rows['all'], header))
         return arrays
 
 
@@ -161,7 +161,7 @@ def compare_rups(calc_1: int, calc_2: int):
     """
     Compare the ruptures of two calculations as pandas DataFrames
     """
-    with util.read(calc_1) as ds1, util.read(calc_2) as ds2:
+    with datastore.read(calc_1) as ds1, datastore.read(calc_2) as ds2:
         df1 = ds1.read_df('rup').sort_values(['src_id', 'mag'])
         df2 = ds2.read_df('rup').sort_values(['src_id', 'mag'])
     cols = [col for col in df1.columns if col not in
@@ -187,12 +187,12 @@ def compare_hmaps(imt, calc_ids: int, files=False, *,
     """
     c = Comparator(calc_ids)
     arrays = c.compare('hmaps', imt, files, samplesites, atol, rtol)
-    if len(calc_ids) == 2:
+    if len(arrays) and len(calc_ids) == 2:
         ms = numpy.mean((arrays[0] - arrays[1])**2, axis=0)  # P
         maxdiff = numpy.abs(arrays[0] - arrays[1]).max(axis=0)  # P
         rows = [(str(poe), rms, md) for poe, rms, md in zip(
             c.oq.poes, numpy.sqrt(ms), maxdiff)]
-        print(views.rst_table(rows, ['poe', 'rms-diff', 'max-diff']))
+        print(views.text_table(rows, ['poe', 'rms-diff', 'max-diff']))
 
 
 def compare_hcurves(imt, calc_ids: int, files=False, *,
