@@ -23,7 +23,7 @@ import numpy
 from openquake.hazardlib.geo.utils import cross_idl
 from openquake.hazardlib.contexts import Effect, get_effect_by_mag
 from openquake.hazardlib.calc.filters import getdefault, MagDepDistance
-from openquake.calculators.extract import Extractor, WebExtractor
+from openquake.calculators.extract import Extractor, WebExtractor, clusterize
 
 
 def make_figure_hcurves(extractors, what):
@@ -56,6 +56,43 @@ def make_figure_hcurves(extractors, what):
     return plt
 
 
+def make_figure_uhs_cluster(extractors, what):
+    """
+    $ oq plot "uhs_cluster?k=12"
+    """
+    import matplotlib.pyplot as plt
+    import matplotlib.cm as cm
+    kstr = what.split('?')[1]
+    k = int(kstr.split('=')[1])
+    fig, ax = plt.subplots()
+    [ex] = extractors
+    hmaps = ex.get('hmaps?kind=rlzs')
+    rlzs = ex.get('realizations').array
+    labels = []
+    for p, poe in enumerate(ex.oqparam.poes):
+        for imt in ex.oqparam.imtls:
+            labels.append('%s' % imt)
+    xs = numpy.arange(len(labels))
+    ax.set_xticks(xs)
+    ax.set_xticklabels(labels)
+    ax.set_ylabel('IML')
+    obs = [getattr(hmaps, 'rlz-%03d' % rlz)[0] for rlz in range(len(rlzs))]
+    arr, cluster = clusterize(numpy.array(obs), rlzs, k)
+    colors = cm.rainbow(numpy.linspace(0, 1, len(arr)))
+    paths = [p.decode('ascii') for p in arr['branch_paths']]
+    for rlz in range(len(rlzs)):
+        # ush-rlz has shape NMP
+        ys = getattr(hmaps, 'rlz-%03d' % rlz)[0].T.flatten()
+        ax.plot(xs, ys, '-', color=colors[cluster[rlz]])
+    for c, curve in enumerate(arr['centroid']):
+        lbl = '%s:%s' % (c + 1, paths[c])
+        print(lbl)
+        ax.plot(xs, curve, '--', color=colors[c], label=lbl)
+    ax.grid(True)
+    ax.legend()
+    return plt
+
+
 def make_figure_avg_gmf(extractors, what):
     """
     $ oq plot "avg_gmf?imt=gmv_0"
@@ -579,13 +616,18 @@ def make_figure_agg_curves(extractors, what):
     got = {}  # (calc_id, kind) -> curves
     for i, ex in enumerate(extractors):
         aw = ex.get(what + '&absolute=1')
+        if isinstance(aw.json, numpy.ndarray):  # from webui
+            js = bytes(aw.json).decode('utf8')
+        else:
+            js = aw.json
+        vars(aw).update(json.loads(js))
         agg_curve = aw.array.squeeze()
         got[ex.calc_id, aw.kind[0]] = agg_curve
     oq = ex.oqparam
     periods = aw.return_period
     ax = fig.add_subplot(1, 1, 1)
     ax.set_xlabel('risk_inv_time=%dy' % oq.risk_investigation_time)
-    ax.set_ylabel('PoE')
+    ax.set_ylabel('loss')
     for ck, arr in got.items():
         ax.loglog(periods, agg_curve, '-', label='%s_%s' % ck)
         ax.loglog(periods, agg_curve, '.')
@@ -594,21 +636,39 @@ def make_figure_agg_curves(extractors, what):
     return plt
 
 
+def make_figure_csq_curves(extractors, what):
+    """
+    $ oq plot "csq_curves?agg_id=0&loss_type=structural&consequence=losses" -1
+    """
+    import matplotlib.pyplot as plt
+    fig = plt.figure()
+    got = {}  # (calc_id, limit_state) -> curve
+    for i, ex in enumerate(extractors):
+        aw = ex.get(what)
+        P, C = aw.shape
+        if P < 2:
+            raise RuntimeError('Not enough return periods: %d' % P)
+        for c, csq in enumerate(aw.consequences):
+            if csq in what:
+                got[ex.calc_id, csq] = aw[:, c]
+    oq = ex.oqparam
+    periods = aw.return_period
+    ax = fig.add_subplot(1, 1, 1)
+    ax.set_xlabel('risk_inv_time=%dy' % oq.risk_investigation_time)
+    ax.set_ylabel(csq)
+    for ck, arr in got.items():
+        ax.loglog(periods, arr, '-', label=ck[0])
+        ax.loglog(periods, arr, '.')
+    ax.grid(True)
+    ax.legend()
+    return plt
+
+
 def make_figure_tot_curves(extractors, what):
     """
     $ oq plot "tot_curves?loss_type=structural&kind=rlz-000&absolute=1"
     """
-    import matplotlib.pyplot as plt
-    fig = plt.figure()
-    [ex] = extractors
-    tot = ex.get(what)
-    ax = fig.add_subplot(1, 1, 1)
-    ax.set_xlabel('return periods')
-    ax.set_ylabel('PoE')
-    ax.loglog(tot.return_period, tot[:, 0], '-', label='tot_curves')
-    ax.grid(True)
-    ax.legend()
-    return plt
+    return make_figure_agg_curves(extractors, what)
 
 
 def plot_wkt(wkt_string):
