@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2016-2020 GEM Foundation
+# Copyright (C) 2016-2021 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -21,7 +21,7 @@ import sys
 import subprocess
 import webbrowser
 
-from openquake.baselib import sap, config
+from openquake.baselib import config
 from openquake.server import dbserver
 from openquake.server.utils import check_webserver_running
 
@@ -47,8 +47,7 @@ def rundjango(subcmd, hostport=None, skip_browser=False):
         sys.exit(p.returncode)
 
 
-@sap.script
-def webui(cmd, hostport='127.0.0.1:8800', skip_browser=False):
+def main(cmd, hostport='127.0.0.1:8800', skip_browser: bool = False):
     """
     start the webui server in foreground or perform other operation on the
     django application
@@ -64,6 +63,6 @@ def webui(cmd, hostport='127.0.0.1:8800', skip_browser=False):
         rundjango(cmd)
 
 
-webui.arg('cmd', 'webui command', choices=commands)
-webui.arg('hostport', 'a string of the form <hostname:port>')
-webui.flg('skip_browser', 'do not automatically open the browser')
+main.cmd = dict(help='webui command', choices=commands)
+main.hostport = 'a string of the form <hostname:port>'
+main.skip_browser = 'do not automatically open the browser'
