@@ -20,14 +20,12 @@ import sys
 import getpass
 from openquake.baselib import config, parallel as p
 
-ro_commands = ('status', 'inspect')
-
 
 def main(cmd):
     """
     start/stop the workers, or return their status
     """
-    if (cmd not in ro_commands and config.dbserver.multi_user and
+    if (cmd != 'status' and config.dbserver.multi_user and
             getpass.getuser() not in 'openquake'):
         sys.exit('oq workers only works in single user mode')
     if p.OQDIST in ('dask', 'celery', 'zmq'):
