@@ -24,30 +24,26 @@ A logic tree object must be iterable and yielding realizations, i.e. objects
 with attributes `value`, `weight`, `lt_path` and `ordinal`.
 """
 
-import io
 import os
 import re
+import json
 import time
-import string
 import logging
 import functools
 import itertools
 import collections
 import operator
-from collections import namedtuple
-import toml
 import numpy
 from openquake.baselib import hdf5
 from openquake.baselib.python3compat import decode
-from openquake.baselib.node import node_from_elem, Node as N, context
-from openquake.baselib.general import groupby, duplicated, AccumDict
-from openquake.hazardlib.gsim.mgmpe.avg_gmpe import AvgGMPE
-from openquake.hazardlib.gsim.base import CoeffsTable
-from openquake.hazardlib.imt import from_string
-from openquake.hazardlib import valid, nrml, InvalidFile, pmf
+from openquake.baselib.node import node_from_elem, context
+from openquake.baselib.general import groupby, AccumDict
+from openquake.hazardlib import nrml, InvalidFile, pmf
 from openquake.hazardlib.sourceconverter import SourceGroup
+from openquake.hazardlib.gsim_lt import (
+    GsimLogicTree, Realization, bsnodes, fix_bytes, keyno)
 from openquake.hazardlib.lt import (
-    Branch, BranchSet, LogicTreeError, parse_uncertainty, sample, random)
+    Branch, BranchSet, LogicTreeError, parse_uncertainty, random)
 
 TRT_REGEX = re.compile(r'tectonicRegion="([^"]+?)"')
 ID_REGEX = re.compile(r'id="([^"]+?)"')
@@ -72,7 +68,7 @@ source_model_dt = numpy.dtype([
 ])
 
 src_group_dt = numpy.dtype(
-    [('et_id', U32),
+    [('trt_smr', U32),
      ('name', hdf5.vstr),
      ('trti', U16),
      ('effrup', I32),
@@ -97,42 +93,6 @@ def unique(objects, key=None):
     return objects
 
 
-class Realization(object):
-    """
-    Generic Realization object with attributes value, weight, ordinal, lt_path,
-    samples.
-    """
-    def __init__(self, value, weight, ordinal, lt_path, samples=1):
-        self.value = value
-        self.weight = weight
-        self.ordinal = ordinal
-        self.lt_path = lt_path
-        self.samples = samples
-
-    @property
-    def pid(self):
-        return '~'.join(self.lt_path)  # path ID
-
-    @property
-    def name(self):
-        """
-        Compact representation for the names
-        """
-        names = self.value.split()
-        if len(names) == 1:
-            return names[0]
-        elif len(names) == 2:
-            return ' '.join(names)
-        else:
-            return ' '.join([names[0], '...', names[-1]])
-
-    def __repr__(self):
-        samples = ', samples=%d' % self.samples if self.samples > 1 else ''
-        return '<%s #%d %s, path=%s, weight=%s%s>' % (
-            self.__class__.__name__, self.ordinal, self.value,
-            '~'.join(self.lt_path), self.weight, samples)
-
-
 @functools.lru_cache()
 def get_effective_rlzs(rlzs):
     """
@@ -152,21 +112,6 @@ def get_effective_rlzs(rlzs):
     return effective
 
 
-# manage the legacy logicTreeBranchingLevel nodes
-def _bsnodes(fname, branchinglevel):
-    if branchinglevel.tag.endswith('logicTreeBranchingLevel'):
-        if len(branchinglevel) > 1:
-            raise InvalidLogicTree(
-                '%s: Branching level %s has multiple branchsets'
-                % (fname, branchinglevel['branchingLevelID']))
-        return branchinglevel.nodes
-    elif branchinglevel.tag.endswith('logicTreeBranchSet'):
-        return [branchinglevel]
-    else:
-        raise ValueError('Expected BranchingLevel/BranchSet, got %s' %
-                         branchinglevel)
-
-
 Info = collections.namedtuple('Info', 'smpaths, applytosources')
 
 
@@ -190,7 +135,7 @@ def collect_info(smlt):
     paths = set()
     applytosources = collections.defaultdict(list)  # branchID -> source IDs
     for blevel in blevels:
-        for bset in _bsnodes(smlt, blevel):
+        for bset in bsnodes(smlt, blevel):
             if 'applyToSources' in bset.attrib:
                 applytosources[bset.get('applyToBranches')].extend(
                         bset['applyToSources'].split())
@@ -232,23 +177,6 @@ def read_source_groups(fname):
     return src_groups
 
 
-def keyno(branch_id, no, fname='',
-          chars=string.digits + string.ascii_uppercase):
-    """
-    :param branch_id: a branch ID string
-    :param no: number of the branch in the branchset (starting from 0)
-    :returns: a 1-char string for the branch_id based on the branch number
-    """
-    try:
-        valid.branch_id(branch_id)
-    except ValueError as ex:
-        raise ValueError('%s %s' % (ex, fname))
-    try:
-        return chars[no]
-    except IndexError:
-        return branch_id
-
-
 def shorten(path, shortener):
     """
     :path:  sequence of strings
@@ -258,6 +186,75 @@ def shorten(path, shortener):
     return ''.join(shortener.get(key, key) for key in path)
 
 
+# useful to print reduced logic trees
+def collect_paths(paths, b1=ord('['), b2=ord(']'), til=ord('~')):
+    """
+    Collect branch paths belonging to the same cluster
+
+    >>> collect_paths([b'0~A0', b'0~A1'])
+    b'[0]~[A][01]'
+    """
+    n = len(paths[0])
+    for path in paths[1:]:
+        assert len(path) == n, (len(path), n)
+    sets = [set() for _ in range(n)]
+    for c, s in enumerate(sets):
+        for path in paths:
+            s.add(path[c])
+    ints = []
+    for s in sets:
+        chars = sorted(s)
+        if chars != [til]:
+            ints.append(b1)
+        ints.extend(chars)
+        if chars != [til]:
+            ints.append(b2)
+    return bytes(ints)
+
+
+def reducible(lt, cluster_paths):
+    """
+    :param lt: a logic tree with B branches
+    :param cluster_paths: list of paths for a realization cluster
+    :returns: a list [filename, (branchSetID, branchIDs), ...]
+    """
+    longener = {short: long for long, short in lt.shortener.items()}
+    bsets = [set() for _ in lt.bsetdict]
+    for path in cluster_paths:
+        for b, chars in enumerate(path.strip('][').split('][')):
+            bsets[b].add(chars)
+    res = [lt.filename]
+    for bs, bset in zip(sorted(lt.bsetdict), bsets):
+        # a branch is reducible if there the same combinations for all paths
+        try:
+            [br_ids] = bset
+        except ValueError:
+            continue
+        res.append((bs, [longener[c] for c in br_ids]))
+    return res
+
+
+# this is used in oq reduce_lt
+def reduce_full(full_lt, rlz_clusters):
+    """
+    :param full_lt: a FullLogicTree instance
+    :param rlz_clusters: list of paths for a realization cluster
+    :returns: a dictionary with what can be reduced
+    """
+    smrlz_clusters = []
+    gsrlz_clusters = []
+    for path in rlz_clusters:
+        smr, gsr = decode(path).split('~')
+        smrlz_clusters.append(smr)
+        gsrlz_clusters.append(gsr)
+    f1, *p1 = reducible(full_lt.source_model_lt, smrlz_clusters)
+    f2, *p2 = reducible(full_lt.gsim_lt, gsrlz_clusters)
+    before = (full_lt.source_model_lt.get_num_paths() *
+              full_lt.gsim_lt.get_num_paths())
+    after = before / numpy.prod([len(p[1]) for p in p1 + p2])
+    return {f1: dict(p1), f2: dict(p2), 'size_before_after': (before, after)}
+
+
 class SourceModelLogicTree(object):
     """
     Source model logic tree parser.
@@ -274,6 +271,13 @@ class SourceModelLogicTree(object):
                'applyToSources',
                'applyToSourceType')
 
+    ABSOLUTE_UNCERTAINTIES = ('abGRAbsolute', 'bGRAbsolute',
+                              'maxMagGRAbsolute',
+                              'simpleFaultGeometryAbsolute',
+                              'truncatedGRFromSlipAbsolute',
+                              'complexFaultGeometryAbsolute',
+                              'setMSRAbsolute')
+
     @classmethod
     def fake(cls):
         """
@@ -283,18 +287,20 @@ class SourceModelLogicTree(object):
         arr = numpy.array([('bs0', 'b0', 'sourceModel', 'fake.xml', 1)],
                           branch_dt)
         dic = dict(filename='fake.xml', seed=0, num_samples=0,
-                   sampling_method='early_weights')
+                   sampling_method='early_weights', num_paths=1,
+                   bsetdict='{"bs0": {"uncertaintyType": "sourceModel"}}')
         self.__fromh5__(arr, dic)
         return self
 
     def __init__(self, filename, seed=0, num_samples=0,
-                 sampling_method='early_weights'):
+                 sampling_method='early_weights', test_mode=False):
         self.filename = filename
         self.basepath = os.path.dirname(filename)
         # NB: converting the random_seed into an integer is needed on Windows
         self.seed = int(seed)
         self.num_samples = num_samples
         self.sampling_method = sampling_method
+        self.test_mode = test_mode
         self.branches = {}  # branch_id -> branch
         self.bsetdict = {}
         self.previous_branches = []
@@ -328,7 +334,7 @@ class SourceModelLogicTree(object):
         self.source_ids = collections.defaultdict(list)
         t0 = time.time()
         for depth, blnode in enumerate(tree_node.nodes):
-            [bsnode] = _bsnodes(self.filename, blnode)
+            [bsnode] = bsnodes(self.filename, blnode)
             self.parse_branchset(bsnode, depth)
         dt = time.time() - t0
         bname = os.path.basename(self.filename)
@@ -377,6 +383,12 @@ class SourceModelLogicTree(object):
         self.previous_branches = branchset.branches
         self.num_paths *= len(branchset.branches)
 
+    def get_num_paths(self):
+        """
+        :returns: the number of paths in the logic tree
+        """
+        return self.num_samples if self.num_samples else self.num_paths
+
     def parse_branches(self, branchset_node, branchset):
         """
         Create and attach branches at ``branchset_node`` to ``branchset``.
@@ -405,7 +417,8 @@ class SourceModelLogicTree(object):
             if branchset.uncertainty_type in ('sourceModel', 'extendModel'):
                 try:
                     for fname in value_node.text.strip().split():
-                        if fname.endswith(('.xml', '.nrml')):  # except UCERF
+                        if (fname.endswith(('.xml', '.nrml'))  # except UCERF
+                                and not self.test_mode):
                             self.collect_source_model_data(
                                 branchnode['branchID'], fname)
                 except Exception as exc:
@@ -508,9 +521,7 @@ class SourceModelLogicTree(object):
                     "source models don't define sources of tectonic region "
                     "type '%s'" % filters['applyToTectonicRegionType'])
 
-        if uncertainty_type in ('abGRAbsolute', 'maxMagGRAbsolute',
-                                'simpleFaultGeometryAbsolute',
-                                'complexFaultGeometryAbsolute'):
+        if uncertainty_type in self.ABSOLUTE_UNCERTAINTIES:
             if not filters or not list(filters) == ['applyToSources'] \
                     or not len(filters['applyToSources'].split()) == 1:
                 raise LogicTreeError(
@@ -639,36 +650,34 @@ class SourceModelLogicTree(object):
         """
         return self.root_branchset.get_bset_values(sm_rlz.lt_path)[1:]
 
-    def _tomldict(self):
-        out = {}
-        for key, dic in self.bsetdict.items():
-            out[key] = toml.dumps({k: v.strip() for k, v in dic.items()
-                                   if k != 'uncertaintyType'}).strip()
-        return out
-
+    # SourceModelLogicTree
     def __toh5__(self):
         tbl = []
         for brid, br in self.branches.items():
             dic = self.bsetdict[br.bs_id].copy()
-            utype = dic.pop('uncertaintyType')
-            tbl.append((br.bs_id, brid, utype, br.value, br.weight))
-        attrs = self._tomldict()
+            utype = dic['uncertaintyType']
+            tbl.append((br.bs_id, brid, utype, str(br.value), br.weight))
+        attrs = dict(bsetdict=json.dumps(self.bsetdict))
         attrs['seed'] = self.seed
         attrs['num_samples'] = self.num_samples
         attrs['sampling_method'] = self.sampling_method
         attrs['filename'] = self.filename
+        attrs['num_paths'] = self.num_paths
         return numpy.array(tbl, branch_dt), attrs
 
+    # SourceModelLogicTree
     def __fromh5__(self, array, attrs):
         # this is rather tricky; to understand it, run the test
         # SerializeSmltTestCase which has a logic tree with 3 branchsets
         # with the form b11[b21[b31, b32], b22[b31, b32]] and 1 x 2 x 2 rlzs
+        vars(self).update(attrs)
         bsets = []
         self.branches = {}
-        self.bsetdict = {}
+        self.bsetdict = json.loads(attrs['bsetdict'])
         self.shortener = {}
         acc = AccumDict(accum=[])  # bsid -> rows
         for rec in array:
+            rec = fix_bytes(rec)
             # NB: it is important to keep the order of the branchsets
             acc[rec['branchset']].append(rec)
         for ordinal, (bsid, rows) in enumerate(acc.items()):
@@ -682,435 +691,19 @@ class SourceModelLogicTree(object):
                     br.branch_id, no, attrs['filename'])
                 bset.branches.append(br)
             bsets.append(bset)
-            self.bsetdict[bsid] = {'uncertaintyType': utype}
         # bsets [<b11>, <b21 b22>, <b31 b32>]
         self.root_branchset = bsets[0]
         for i, childset in enumerate(bsets[1:]):
-            dic = toml.loads(attrs[childset.id])
+            dic = self.bsetdict[childset.id]
             atb = dic.get('applyToBranches')
             for branch in bsets[i].branches:  # parent branches
                 if not atb or branch.branch_id in atb:
                     branch.bset = childset
-        self.seed = attrs['seed']
-        self.num_samples = attrs['num_samples']
-        self.sampling_method = attrs['sampling_method']
-        self.filename = attrs['filename']
 
     def __str__(self):
         return '<%s%s>' % (self.__class__.__name__, repr(self.root_branchset))
 
 
-# used in GsimLogicTree
-BranchTuple = namedtuple('BranchTuple', 'trt id gsim weight effective')
-
-
-class InvalidLogicTree(Exception):
-    pass
-
-
-class ImtWeight(object):
-    """
-    A composite weight by IMTs extracted from the gsim_logic_tree_file
-    """
-    def __init__(self, branch, fname):
-        with context(fname, branch.uncertaintyWeight):
-            nodes = list(branch.getnodes('uncertaintyWeight'))
-            if 'imt' in nodes[0].attrib:
-                raise InvalidLogicTree('The first uncertaintyWeight has an imt'
-                                       ' attribute')
-            self.dic = {'weight': float(nodes[0].text)}
-            imts = []
-            for n in nodes[1:]:
-                self.dic[n['imt']] = float(n.text)
-                imts.append(n['imt'])
-            if len(set(imts)) < len(imts):
-                raise InvalidLogicTree(
-                    'There are duplicated IMTs in the weights')
-
-    def __mul__(self, other):
-        new = object.__new__(self.__class__)
-        if isinstance(other, self.__class__):
-            keys = set(self.dic) | set(other.dic)
-            new.dic = {k: self[k] * other[k] for k in keys}
-        else:  # assume a float
-            new.dic = {k: self.dic[k] * other for k in self.dic}
-        return new
-
-    __rmul__ = __mul__
-
-    def __add__(self, other):
-        new = object.__new__(self.__class__)
-        if isinstance(other, self.__class__):
-            new.dic = {k: self.dic[k] + other[k] for k in self.dic}
-        else:  # assume a float
-            new.dic = {k: self.dic[k] + other for k in self.dic}
-        return new
-
-    __radd__ = __add__
-
-    def __truediv__(self, other):
-        new = object.__new__(self.__class__)
-        if isinstance(other, self.__class__):
-            new.dic = {k: self.dic[k] / other[k] for k in self.dic}
-        else:  # assume a float
-            new.dic = {k: self.dic[k] / other for k in self.dic}
-        return new
-
-    def is_one(self):
-        """
-        Check that all the inner weights are 1 up to the precision
-        """
-        return all(abs(v - 1.) < pmf.PRECISION for v in self.dic.values() if v)
-
-    def __getitem__(self, imt):
-        try:
-            return self.dic[imt]
-        except KeyError:
-            return self.dic['weight']
-
-    def __repr__(self):
-        return '<%s %s>' % (self.__class__.__name__, self.dic)
-
-
-class GsimLogicTree(object):
-    """
-    A GsimLogicTree instance is an iterable yielding `Realization`
-    tuples with attributes `value`, `weight` and `lt_path`, where
-    `value` is a dictionary {trt: gsim}, `weight` is a number in the
-    interval 0..1 and `lt_path` is a tuple with the branch ids of the
-    given realization.
-
-    :param str fname:
-        full path of the gsim_logic_tree file
-    :param tectonic_region_types:
-        a sequence of distinct tectonic region types
-    :param ltnode:
-        usually None, but it can also be a
-        :class:`openquake.hazardlib.nrml.Node` object describing the
-        GSIM logic tree XML file, to avoid reparsing it
-    """
-    @classmethod
-    def from_(cls, gsim):
-        """
-        Generate a trivial GsimLogicTree from a single GSIM instance.
-        """
-        ltbranch = N('logicTreeBranch', {'branchID': 'b1'},
-                     nodes=[N('uncertaintyModel', text=str(gsim)),
-                            N('uncertaintyWeight', text='1.0')])
-        lt = N('logicTree', {'logicTreeID': 'lt1'},
-               nodes=[N('logicTreeBranchingLevel', {'branchingLevelID': 'bl1'},
-                        nodes=[N('logicTreeBranchSet',
-                                 {'applyToTectonicRegionType': '*',
-                                  'branchSetID': 'bs1',
-                                  'uncertaintyType': 'gmpeModel'},
-                                 nodes=[ltbranch])])])
-        return cls('fake/' + gsim.__class__.__name__, ['*'], ltnode=lt)
-
-    def __init__(self, fname, tectonic_region_types=['*'], ltnode=None):
-        # tectonic_region_types usually comes from the source models
-        self.filename = fname
-        trts = sorted(tectonic_region_types)
-        if len(trts) > len(set(trts)):
-            raise ValueError(
-                'The given tectonic region types are not distinct: %s' %
-                ','.join(trts))
-        self.values = collections.defaultdict(list)  # {trt: gsims}
-        self._ltnode = ltnode or nrml.read(fname).logicTree
-        self.bs_id_by_trt = {}
-        self.shortener = {}
-        self.branches = self._build_trts_branches(trts)  # sorted by trt
-        if trts != ['*']:
-            # reduce self.values to the listed TRTs
-            values = {}
-            for trt in trts:
-                values[trt] = self.values[trt]
-                if not values[trt]:
-                    raise InvalidLogicTree('%s is missing the TRT %r' %
-                                           (fname, trt))
-            self.values = values
-        if trts and not self.branches:
-            raise InvalidLogicTree(
-                '%s is missing in %s' % (set(tectonic_region_types), fname))
-
-    @property
-    def req_site_params(self):
-        site_params = set()
-        for trt in self.values:
-            for gsim in self.values[trt]:
-                site_params.update(gsim.REQUIRES_SITES_PARAMETERS)
-        return site_params
-
-    def check_imts(self, imts):
-        """
-        Make sure the IMTs are recognized by all GSIMs in the logic tree
-        """
-        for trt in self.values:
-            for gsim in self.values[trt]:
-                for attr in dir(gsim):
-                    coeffs = getattr(gsim, attr)
-                    if not isinstance(coeffs, CoeffsTable):
-                        continue
-                    for imt in imts:
-                        if imt.startswith('SA'):
-                            try:
-                                coeffs[from_string(imt)]
-                            except KeyError:
-                                raise ValueError(
-                                    '%s is out of the period range defined '
-                                    'for %s' % (imt, gsim))
-
-    def __toh5__(self):
-        weights = set()
-        for branch in self.branches:
-            weights.update(branch.weight.dic)
-        dt = [('trt', hdf5.vstr), ('branch', hdf5.vstr),
-              ('uncertainty', hdf5.vstr)] + [
-            (weight, float) for weight in sorted(weights)]
-        branches = [(b.trt, b.id, repr(b.gsim)) +
-                    tuple(b.weight[weight] for weight in sorted(weights))
-                    for b in self.branches if b.effective]
-        dic = {}
-        if hasattr(self, 'filename'):
-            # missing in EventBasedRiskTestCase case_1f
-            dirname = os.path.dirname(self.filename)
-            for gsims in self.values.values():
-                for gsim in gsims:
-                    for k, v in gsim.kwargs.items():
-                        if k.endswith(('_file', '_table')):
-                            fname = os.path.join(dirname, v)
-                            with open(fname, 'rb') as f:
-                                dic[os.path.basename(v)] = f.read()
-        return numpy.array(branches, dt), dic
-
-    def __fromh5__(self, array, dic):
-        self.branches = []
-        self.shortener = {}
-        self.values = collections.defaultdict(list)
-        for no, branch in enumerate(array):
-            br_id = branch['branch']
-            gsim = valid.gsim(branch['uncertainty'])
-            for k, v in gsim.kwargs.items():
-                if k.endswith(('_file', '_table')):
-                    arr = numpy.asarray(dic[os.path.basename(v)][()])
-                    gsim.kwargs[k] = io.BytesIO(bytes(arr))
-            gsim.__init__(**gsim.kwargs)
-            self.values[branch['trt']].append(gsim)
-            weight = object.__new__(ImtWeight)
-            # branch has dtype ('trt', 'branch', 'uncertainty', 'weight', ...)
-            weight.dic = {w: branch[w] for w in branch.dtype.names[3:]}
-            if len(weight.dic) > 1:
-                gsim.weight = weight
-            bt = BranchTuple(branch['trt'], br_id, gsim, weight, True)
-            self.branches.append(bt)
-            self.shortener[br_id] = keyno(br_id, no)
-
-    def reduce(self, trts):
-        """
-        Reduce the GsimLogicTree.
-
-        :param trts: a subset of tectonic region types
-        :returns: a reduced GsimLogicTree instance
-        """
-        new = object.__new__(self.__class__)
-        vars(new).update(vars(self))
-        if trts != {'*'}:
-            new.branches = []
-            for br in self.branches:
-                branch = BranchTuple(br.trt, br.id, br.gsim, br.weight,
-                                     br.trt in trts)
-                new.branches.append(branch)
-        return new
-
-    def collapse(self, branchset_ids):
-        """
-        Collapse the GsimLogicTree by using AgvGMPE instances if needed
-
-        :param branchset_ids: branchset ids to collapse
-        :returns: a collapse GsimLogicTree instance
-        """
-        new = object.__new__(self.__class__)
-        vars(new).update(vars(self))
-        new.branches = []
-        for trt, grp in itertools.groupby(self.branches, lambda b: b.trt):
-            bs_id = self.bs_id_by_trt[trt]
-            brs = []
-            gsims = []
-            weights = []
-            for br in grp:
-                brs.append(br.id)
-                gsims.append(br.gsim)
-                weights.append(br.weight)
-            if len(gsims) > 1 and bs_id in branchset_ids:
-                kwargs = {}
-                for brid, gsim, weight in zip(brs, gsims, weights):
-                    kw = gsim.kwargs.copy()
-                    kw['weight'] = weight.dic['weight']
-                    kwargs[brid] = {gsim.__class__.__name__: kw}
-                _toml = toml.dumps({'AvgGMPE': kwargs})
-                gsim = AvgGMPE(**kwargs)
-                gsim._toml = _toml
-                new.values[trt] = [gsim]
-                branch = BranchTuple(trt, bs_id, gsim, sum(weights), True)
-                new.branches.append(branch)
-            else:
-                new.branches.append(br)
-        return new
-
-    def get_num_branches(self):
-        """
-        Return the number of effective branches for tectonic region type,
-        as a dictionary.
-        """
-        num = {}
-        for trt, branches in itertools.groupby(
-                self.branches, operator.attrgetter('trt')):
-            num[trt] = sum(1 for br in branches if br.effective)
-        return num
-
-    def get_num_paths(self):
-        """
-        Return the effective number of paths in the tree.
-        """
-        num_branches = self.get_num_branches()
-        if not sum(num_branches.values()):
-            return 0
-        num = 1
-        for val in num_branches.values():
-            if val:  # the branch is effective
-                num *= val
-        return num
-
-    def _build_trts_branches(self, tectonic_region_types):
-        # do the parsing, called at instantiation time to populate .values
-        trts = []
-        branches = []
-        branchsetids = set()
-        basedir = os.path.dirname(self.filename)
-        for blnode in self._ltnode:
-            [branchset] = _bsnodes(self.filename, blnode)
-            if branchset['uncertaintyType'] != 'gmpeModel':
-                raise InvalidLogicTree(
-                    '%s: only uncertainties of type "gmpeModel" '
-                    'are allowed in gmpe logic tree' % self.filename)
-            bsid = branchset['branchSetID']
-            if bsid in branchsetids:
-                raise InvalidLogicTree(
-                    '%s: Duplicated branchSetID %s' %
-                    (self.filename, bsid))
-            else:
-                branchsetids.add(bsid)
-            trt = branchset.get('applyToTectonicRegionType')
-            if trt:  # missing in logictree_test.py
-                self.bs_id_by_trt[trt] = bsid
-                trts.append(trt)
-            self.bs_id_by_trt[trt] = bsid
-            # NB: '*' is used in scenario calculations to disable filtering
-            effective = (tectonic_region_types == ['*'] or
-                         trt in tectonic_region_types)
-            weights = []
-            branch_ids = []
-            for no, branch in enumerate(branchset):
-                weight = ImtWeight(branch, self.filename)
-                weights.append(weight)
-                branch_id = branch['branchID']
-                branch_ids.append(branch_id)
-                try:
-                    gsim = valid.gsim(branch.uncertaintyModel, basedir)
-                except Exception as exc:
-                    raise ValueError(
-                        "%s in file %s" % (exc, self.filename)) from exc
-                if gsim in self.values[trt]:
-                    raise InvalidLogicTree('%s: duplicated gsim %s' %
-                                           (self.filename, gsim))
-                if len(weight.dic) > 1:
-                    gsim.weight = weight
-                self.values[trt].append(gsim)
-                bt = BranchTuple(
-                    branchset['applyToTectonicRegionType'],
-                    branch_id, gsim, weight, effective)
-                if effective:
-                    branches.append(bt)
-                    self.shortener[branch_id] = keyno(
-                        branch_id, no, self.filename)
-            tot = sum(weights)
-            assert tot.is_one(), '%s in branch %s' % (tot, branch_id)
-            if duplicated(branch_ids):
-                raise InvalidLogicTree(
-                    'There where duplicated branchIDs in %s' %
-                    self.filename)
-        if len(trts) > len(set(trts)):
-            raise InvalidLogicTree(
-                '%s: Found duplicated applyToTectonicRegionType=%s' %
-                (self.filename, trts))
-        branches.sort(key=lambda b: (b.trt, b.id))
-        # TODO: add an .idx to each GSIM ?
-        return branches
-
-    def get_gsims(self, trt):
-        """
-        :param trt: tectonic region type
-        :returns: sorted list of available GSIMs for that trt
-        """
-        if trt == '*' or trt == b'*':  # fake logictree
-            [trt] = self.values
-        return sorted(self.values[trt])
-
-    def sample(self, n, seed, sampling_method):
-        """
-        :param n: number of samples
-        :param seed: random seed
-        :param sampling_method: by default 'early_weights'
-        :returns: n Realization objects
-        """
-        m = len(self.values)  # number of TRTs
-        probs = random((n, m), seed, sampling_method)
-        brlists = [sample([b for b in self.branches if b.trt == trt],
-                          probs[:, i], sampling_method)
-                   for i, trt in enumerate(self.values)]
-        rlzs = []
-        for i in range(n):
-            weight = 1
-            lt_path = []
-            lt_uid = []
-            value = []
-            for brlist in brlists:  # there is branch list for each TRT
-                branch = brlist[i]
-                lt_path.append(branch.id)
-                lt_uid.append(branch.id if branch.effective else '@')
-                weight *= branch.weight
-                value.append(branch.gsim)
-            rlz = Realization(tuple(value), weight, i, tuple(lt_uid))
-            rlzs.append(rlz)
-        return rlzs
-
-    def __iter__(self):
-        """
-        Yield :class:`openquake.commonlib.logictree.Realization` instances
-        """
-        groups = []
-        # NB: branches are already sorted
-        for trt in self.values:
-            groups.append([b for b in self.branches if b.trt == trt])
-        # with T tectonic region types there are T groups and T branches
-        for i, branches in enumerate(itertools.product(*groups)):
-            weight = 1
-            lt_path = []
-            lt_uid = []
-            value = []
-            for trt, branch in zip(self.values, branches):
-                lt_path.append(branch.id)
-                lt_uid.append(branch.id if branch.effective else '@')
-                weight *= branch.weight
-                value.append(branch.gsim)
-            yield Realization(tuple(value), weight, i, tuple(lt_uid))
-
-    def __repr__(self):
-        lines = ['%s,%s,%s,w=%s' %
-                 (b.trt, b.id, b.gsim, b.weight['weight'])
-                 for b in self.branches if b.effective]
-        return '<%s\n%s>' % (self.__class__.__name__, '\n'.join(lines))
-
-
 def capitalize(words):
     """
     Capitalize words separated by spaces.
@@ -1185,7 +778,7 @@ class FullLogicTree(object):
     def __init__(self, source_model_lt, gsim_lt):
         self.source_model_lt = source_model_lt
         self.gsim_lt = gsim_lt
-        self.init()  # set .sm_rlzs and .trt_by_et
+        self.init()  # set .sm_rlzs and .trts
 
     def init(self):
         if self.source_model_lt.num_samples:
@@ -1199,22 +792,22 @@ class FullLogicTree(object):
                 sm_rlz.samples = samples
                 self.sm_rlzs.append(sm_rlz)
         self.trti = {trt: i for i, trt in enumerate(self.gsim_lt.values)}
+        self.trts = list(self.gsim_lt.values)
 
-    def get_eri_by_ltp(self):
+    def get_smr_by_ltp(self):
         """
         :returns: a dictionary sm_lt_path -> effective realization index
         """
         return {'~'.join(sm_rlz.lt_path): i
                 for i, sm_rlz in enumerate(self.sm_rlzs)}
 
-    @property
-    def trt_by_et(self):
+    def trt_by(self, trt_smr):
         """
-        :returns: a list of TRTs, one for each et_id
+        :returns: the TRT associated to trt_smr
         """
-        e = len(self.sm_rlzs)
-        trts = list(self.gsim_lt.values)
-        return [trts[et_id // e] for et_id in range(e*len(trts))]
+        if len(self.trts) == 1:
+            return self.trts[0]
+        return self.trts[trt_smr // len(self.sm_rlzs)]
 
     @property
     def seed(self):
@@ -1237,27 +830,27 @@ class FullLogicTree(object):
         """
         return self.source_model_lt.sampling_method
 
-    def get_trti_eri(self, et_id):
+    def get_trti_smr(self, trt_smr):
         """
-        :returns: (trti, eri)
+        :returns: (trti, smr)
         """
-        return divmod(et_id, len(self.sm_rlzs))
+        return divmod(trt_smr, len(self.sm_rlzs))
 
-    def get_et_id(self, trt, eri):
+    def get_trt_smr(self, trt, smr):
         """
-        :returns: et_id
+        :returns: trt_smr
         """
-        gid = self.trti[trt] * len(self.sm_rlzs) + int(eri)
+        gid = self.trti[trt] * len(self.sm_rlzs) + int(smr)
         return gid
 
-    def et_ids(self, eri):
+    def get_trt_smrs(self, smr):
         """
-        :param eri: effective realization index
+        :param smr: effective realization index
         :returns: array of T group IDs, being T the number of TRTs
         """
         nt = len(self.gsim_lt.values)
         ns = len(self.sm_rlzs)
-        return eri + numpy.arange(nt) * ns
+        return smr + numpy.arange(nt) * ns
 
     def gsim_by_trt(self, rlz):
         """
@@ -1308,75 +901,75 @@ class FullLogicTree(object):
                     rlz.weight = rlz.weight / tot_weight
         return rlzs
 
-    def get_rlzs_by_eri(self):
+    def get_rlzs_by_smr(self):
         """
-        :returns: a dict eri -> rlzs
+        :returns: a dict smr -> rlzs
         """
         smltpath = operator.attrgetter('sm_lt_path')
-        eri_by_ltp = self.get_eri_by_ltp()
+        smr_by_ltp = self.get_smr_by_ltp()
         rlzs = self.get_realizations()
-        dic = {eri_by_ltp['~'.join(ltp)]: rlzs for ltp, rlzs in groupby(
+        dic = {smr_by_ltp['~'.join(ltp)]: rlzs for ltp, rlzs in groupby(
             rlzs, smltpath).items()}
         return dic
 
-    def get_rlzs_by_gsim(self, et_id):
+    def _rlzs_by_gsim(self, grp_id):
         """
         :returns: a dictionary gsim -> array of rlz indices
         """
         if not hasattr(self, '_rlzs_by_grp'):
-            eri_by_ltp = self.get_eri_by_ltp()
+            smr_by_ltp = self.get_smr_by_ltp()
             rlzs = self.get_realizations()
-            acc = AccumDict(accum=AccumDict(accum=[]))  # et_id->gsim->rlzs
+            acc = AccumDict(accum=AccumDict(accum=[]))  # trt_smr->gsim->rlzs
             for sm in self.sm_rlzs:
-                for gid in self.et_ids(sm.ordinal):
-                    trti, eri = divmod(gid, len(self.sm_rlzs))
+                for gid in self.get_trt_smrs(sm.ordinal):
+                    trti, smr = divmod(gid, len(self.sm_rlzs))
                     for rlz in rlzs:
-                        idx = eri_by_ltp['~'.join(rlz.sm_lt_path)]
-                        if idx == eri:
+                        idx = smr_by_ltp['~'.join(rlz.sm_lt_path)]
+                        if idx == smr:
                             acc[gid][rlz.gsim_rlz.value[trti]].append(
                                 rlz.ordinal)
             self._rlzs_by_grp = {}
             for gid, dic in acc.items():
                 self._rlzs_by_grp[gid] = {
                     gsim: U32(rlzs) for gsim, rlzs in sorted(dic.items())}
-        return self._rlzs_by_grp[et_id]
+        return self._rlzs_by_grp[grp_id]
 
-    def get_rlzs_by_gsim_grp(self):
+    def get_rlzs_by_gsim(self):
         """
-        :returns: a dictionary et_id -> gsim -> rlzs
+        :returns: a dictionary trt_smr -> gsim -> rlzs
         """
         dic = {}
         for sm in self.sm_rlzs:
-            for et_id in self.et_ids(sm.ordinal):
-                dic[et_id] = self.get_rlzs_by_gsim(et_id)
+            for trt_smr in self.get_trt_smrs(sm.ordinal):
+                dic[trt_smr] = self._rlzs_by_gsim(trt_smr)
         return dic
 
     def get_rlzs_by_grp(self):
         """
-        :returns: a dictionary et_id -> [rlzis, ...]
+        :returns: a dictionary grp_id -> [rlzis, ...]
         """
         dic = {}
         for sm in self.sm_rlzs:
-            for et_id in self.et_ids(sm.ordinal):
-                grp = 'grp-%02d' % et_id
-                dic[grp] = list(self.get_rlzs_by_gsim(et_id).values())
-        return {et_id: dic[et_id] for et_id in sorted(dic)}
+            for trt_smr in self.get_trt_smrs(sm.ordinal):
+                grp = 'grp-%02d' % trt_smr
+                dic[grp] = list(self._rlzs_by_gsim(trt_smr).values())
+        return {grp_id: dic[grp_id] for grp_id in sorted(dic)}
 
-    def get_rlzs_by_gsim_list(self, list_of_et_ids):
+    def get_rlzs_by_gsim_list(self, list_of_trt_smrs):
         """
         :returns: a list of dictionaries rlzs_by_gsim, one for each grp_id
         """
         out = []
-        for grp_id, et_ids in enumerate(list_of_et_ids):
+        for grp_id, trt_smrs in enumerate(list_of_trt_smrs):
             dic = AccumDict(accum=[])
-            for et_id in et_ids:
-                for gsim, rlzs in self.get_rlzs_by_gsim(et_id).items():
+            for trt_smr in trt_smrs:
+                for gsim, rlzs in self._rlzs_by_gsim(trt_smr).items():
                     dic[gsim].extend(rlzs)
             out.append(dic)
         return out
 
+    # FullLogicTree
     def __toh5__(self):
-        # save full_lt/sm_data in the datastore
         sm_data = []
         for sm in self.sm_rlzs:
             sm_data.append((sm.value, sm.weight, '~'.join(sm.lt_path),
@@ -1388,6 +981,7 @@ class FullLogicTree(object):
                 dict(seed=self.seed, num_samples=self.num_samples,
                      trts=hdf5.array_of_vstr(self.gsim_lt.values)))
 
+    # FullLogicTree
     def __fromh5__(self, dic, attrs):
         # TODO: this is called more times than needed, maybe we should cache it
         sm_data = dic['sm_data']
@@ -1442,10 +1036,10 @@ class FullLogicTree(object):
 
     def get_sm_by_grp(self):
         """
-        :returns: a dictionary et_id -> sm_id
+        :returns: a dictionary trt_smr -> sm_id
         """
-        return {et_id: sm.ordinal for sm in self.sm_rlzs
-                for et_id in self.et_ids(sm.ordinal)}
+        return {trt_smr: sm.ordinal for sm in self.sm_rlzs
+                for trt_smr in self.get_trt_smrs(sm.ordinal)}
 
     def __repr__(self):
         info_by_model = {}
