@@ -16,7 +16,27 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
 
-from openquake.baselib import __version__
+
+import os
+import re
+import ast
+import json
+import inspect
+import logging
+import functools
+import collections
+import multiprocessing
+import numpy
+
+from openquake.baselib import __version__, hdf5, python3compat
+from openquake.baselib.general import DictArray, AccumDict
+from openquake.hazardlib.imt import from_string
+from openquake.hazardlib.shakemap.maps import get_array
+from openquake.hazardlib import correlation, stats, calc
+from openquake.hazardlib import valid, InvalidFile, site
+from openquake.sep.classes import SecondaryPeril
+from openquake.commonlib import logictree
+from openquake.risklib.riskmodels import get_risk_files
 
 __doc__ = """\
 Full list of configuration parameters
@@ -94,6 +114,12 @@ collapse_gsim_logic_tree:
 collapse_level:
   INTERNAL
 
+collect_rlzs:
+  Collect all realizations into a single effective realizations. Used in
+  event_based_risk calculations with sampling.
+  Example: *collect_rlzs=true*.
+  Default: False
+
 compare_with_classical:
   Used in event based calculation to perform also a classical calculation,
   so that the hazard curves can be compared.
@@ -123,6 +149,15 @@ float_dmg_dist:
   Example: *float_dmg_dist = true*.
   Default: False
 
+cholesky_limit:
+  When generating the GMFs from a ShakeMap the engine needs to perform a
+  Cholesky decomposition of a matrix of size (M x N)^2, being M the number
+  of intensity measure types and N the number of sites. The decomposition
+  can become ultra-slow, run out of memory, or produce bogus negative
+  eigenvalues, therefore there is a limit on the maximum size of M x N.
+  Example: *cholesky_limit = 1000*.
+  Default: 10,000
+
 continuous_fragility_discretization:
   Used when discretizing continuuos fragility functions.
   Example: *continuous_fragility_discretization = 10*.
@@ -142,7 +177,7 @@ cross_correlation:
 description:
   A string describing the calculation.
   Example: *description = Test calculation*.
-  Default: no default
+  Default: "no description"
 
 disagg_by_src:
   Flag used to enable disaggregation by source when possible.
@@ -173,6 +208,17 @@ distance_bin_width:
 ebrisk_maxsize:
   INTERNAL
 
+ignore_encoding_errors:
+  If set, skip characters with non-UTF8 encoding
+  Example: *ignore_encoding_errors = true*.
+  Default: False
+
+ignore_master_seed:
+  If set, estimate analytically the uncertainty on the losses due to the
+  uncertainty on the vulnerability functions.
+  Example: *ignore_master_seed = vulnerability*.
+  Default: None
+
 export_dir:
   Set the export directory.
   Example: *export_dir = /tmp*.
@@ -266,6 +312,11 @@ investigation_time:
   Example: *investigation_time = 50*.
   Default: no default
 
+limit_states:
+   Limit states used in damage calculations.
+   Example: *limit_states = moderate, complete*
+   Default: no default
+
 lrem_steps_per_interval:
   Used in the vulnerability functions.
   Example: *lrem_steps_per_interval  = 1*.
@@ -281,7 +332,7 @@ master_seed:
   calculations with vulnerability functions with nonzero coefficients of
   variation.
   Example: *master_seed = 1234*.
-  Default: 0
+  Default: 123456789
 
 max:
   Compute the maximum across realizations. Akin to mean and quantiles.
@@ -339,6 +390,11 @@ minimum_asset_loss:
   Example: *minimum_asset_loss = {"structural": 1000}*.
   Default: empty dictionary
 
+minimum_distance:
+   If set, distances below the minimum are rounded up.
+   Example: *minimum_distance = 5*
+   Default: 0
+
 minimum_intensity:
   If set, ground motion values below the *minimum_intensity* are
   considered zeros.
@@ -359,7 +415,7 @@ modal_damage_state:
 num_epsilon_bins:
   Number of epsilon bins in disaggregation calculations.
   Example: *num_epsilon_bins = 3*.
-  Default: no default
+  Default: 1
 
 num_rlzs_disagg:
   Used in disaggregation calculation to specify how many outputs will be
@@ -428,17 +484,10 @@ reference_depth_to_2pt5km_per_sec:
   Example: *reference_depth_to_2pt5km_per_sec = 5*.
   Default: no default
 
-reference_siteclass:
-  Used when there is no site model to specify a global site class.
-  The siteclass is a one-character letter used in some GMPEs, like the
-  McVerry (2006), and has values "A", "B", "C" or "D".
-  Example: *reference_siteclass = "A"*.
-  Default: "D"
-
 reference_vs30_type:
   Used when there is no site model to specify a global vs30 type.
   The choices are "inferred" or "measured"
-  Example: *reference_vs30_type = inferred".
+  Example: *reference_vs30_type = inferred"*.
   Default: "measured"
 
 reference_vs30_value:
@@ -522,6 +571,15 @@ shakemap_id:
   Example: *shakemap_id = usp000fjta*.
   Default: no default
 
+shakemap_uri:
+  Dictionary used in ShakeMap calculations to specify a ShakeMap. Must contain
+  a key named "kind" with values "usgs_id", "usgs_xml" or "file_npy".
+  Example: shakemap_uri = {
+  "kind": "usgs_xml",
+  "grid_url": "file:///home/michele/usp000fjta/grid.xml",
+  "uncertainty_url": "file:///home/michele/usp000fjta/uncertainty.xml"}.
+  Default: empty dictionary
+
 shift_hypo:
   Used in classical calculations to shift the rupture hypocenter.
   Example: *shift_hypo = true*.
@@ -595,20 +653,6 @@ width_of_mfd_bin:
   Example: *width_of_mfd_bin = 0.2*.
   Default: None
 """ % __version__
-import os
-import re
-import logging
-import functools
-import multiprocessing
-import numpy
-
-from openquake.baselib.general import DictArray, AccumDict
-from openquake.hazardlib.imt import from_string
-from openquake.hazardlib import correlation, stats, calc
-from openquake.hazardlib import valid, InvalidFile
-from openquake.sep.classes import SecondaryPeril
-from openquake.commonlib import logictree, util
-from openquake.risklib.riskmodels import get_risk_files
 
 GROUND_MOTION_CORRELATION_MODELS = ['JB2009', 'HM2018']
 TWO16 = 2 ** 16  # 65536
@@ -666,27 +710,21 @@ class OqParam(valid.ParamSet):
                     'structural_vulnerability_retrofitted',
                     'occupants_vulnerability'}
     hazard_imtls = {}
-    siteparam = dict(
-        vs30measured='reference_vs30_type',
-        vs30='reference_vs30_value',
-        z1pt0='reference_depth_to_1pt0km_per_sec',
-        z2pt5='reference_depth_to_2pt5km_per_sec',
-        siteclass='reference_siteclass',
-        backarc='reference_backarc')
     aggregate_by = valid.Param(valid.namelist, [])
     amplification_method = valid.Param(
         valid.Choice('convolution', 'kernel'), None)
     minimum_asset_loss = valid.Param(valid.floatdict, {'default': 0})
     area_source_discretization = valid.Param(
         valid.NoneOr(valid.positivefloat), None)
-    asset_correlation = valid.Param(valid.NoneOr(valid.FloatRange(0, 1)), 0)
+    asset_correlation = valid.Param(valid.Choice('0', '1'), 0)
     asset_life_expectancy = valid.Param(valid.positivefloat)
     assets_per_site_limit = valid.Param(valid.positivefloat, 1000)
     avg_losses = valid.Param(valid.boolean, True)
     base_path = valid.Param(valid.utf8, '.')
     calculation_mode = valid.Param(valid.Choice())  # -> get_oqparam
     collapse_gsim_logic_tree = valid.Param(valid.namelist, [])
-    collapse_level = valid.Param(valid.Choice('0', '1', '2', '3'), 0)
+    collapse_level = valid.Param(valid.Choice('0', '1', '2', '3'), '0')
+    collect_rlzs = valid.Param(valid.boolean, False)
     coordinate_bin_width = valid.Param(valid.positivefloat)
     compare_with_classical = valid.Param(valid.boolean, False)
     concurrent_tasks = valid.Param(
@@ -694,8 +732,9 @@ class OqParam(valid.ParamSet):
     conditional_loss_poes = valid.Param(valid.probabilities, [])
     continuous_fragility_discretization = valid.Param(valid.positiveint, 20)
     cross_correlation = valid.Param(valid.Choice('yes', 'no', 'full'), 'yes')
+    cholesky_limit = valid.Param(valid.positiveint, 10_000)
     cachedir = valid.Param(valid.utf8, '')
-    description = valid.Param(valid.utf8_not_empty)
+    description = valid.Param(valid.utf8_not_empty, "no description")
     disagg_by_src = valid.Param(valid.boolean, False)
     disagg_outputs = valid.Param(valid.disagg_outputs,
                                  list(calc.disagg.pmf_map))
@@ -704,6 +743,8 @@ class OqParam(valid.ParamSet):
     distance_bin_width = valid.Param(valid.positivefloat)
     float_dmg_dist = valid.Param(valid.boolean, False)
     mag_bin_width = valid.Param(valid.positivefloat)
+    ignore_encoding_errors = valid.Param(valid.boolean, False)
+    ignore_master_seed = valid.Param(valid.boolean, False)
     export_dir = valid.Param(valid.utf8, '.')
     exports = valid.Param(valid.export_formats, ())
     ground_motion_correlation_model = valid.Param(
@@ -717,7 +758,7 @@ class OqParam(valid.ParamSet):
     ignore_missing_costs = valid.Param(valid.namelist, [])
     ignore_covs = valid.Param(valid.boolean, False)
     iml_disagg = valid.Param(valid.floatdict, {})  # IMT -> IML
-    individual_curves = valid.Param(valid.boolean, False)
+    individual_curves = valid.Param(valid.boolean, None)
     inputs = valid.Param(dict, {})
     ash_wet_amplification_factor = valid.Param(valid.positivefloat, 1.0)
     intensity_measure_types = valid.Param(valid.intensity_measure_types, '')
@@ -725,9 +766,10 @@ class OqParam(valid.ParamSet):
         valid.intensity_measure_types_and_levels, None)
     interest_rate = valid.Param(valid.positivefloat)
     investigation_time = valid.Param(valid.positivefloat, None)
+    limit_states = valid.Param(valid.namelist, [])
     lrem_steps_per_interval = valid.Param(valid.positiveint, 0)
     steps_per_interval = valid.Param(valid.positiveint, 1)
-    master_seed = valid.Param(valid.positiveint, 0)
+    master_seed = valid.Param(valid.positiveint, 123456789)
     maximum_distance = valid.Param(valid.MagDepDistance.new)  # km
     asset_hazard_distance = valid.Param(valid.floatdict, {'default': 15})  # km
     max = valid.Param(valid.boolean, False)
@@ -739,12 +781,13 @@ class OqParam(valid.ParamSet):
     max_sites_disagg = valid.Param(valid.positiveint, 10)
     mean_hazard_curves = mean = valid.Param(valid.boolean, True)
     std = valid.Param(valid.boolean, False)
+    minimum_distance = valid.Param(valid.positivefloat, 0)
     minimum_intensity = valid.Param(valid.floatdict, {})  # IMT -> minIML
     minimum_magnitude = valid.Param(valid.floatdict, {'default': 0})  # by TRT
     modal_damage_state = valid.Param(valid.boolean, False)
     number_of_ground_motion_fields = valid.Param(valid.positiveint)
     number_of_logic_tree_samples = valid.Param(valid.positiveint, 0)
-    num_epsilon_bins = valid.Param(valid.positiveint)
+    num_epsilon_bins = valid.Param(valid.positiveint, 1)
     num_rlzs_disagg = valid.Param(valid.positiveint, None)
     poes = valid.Param(valid.probabilities, [])
     poes_disagg = valid.Param(valid.probabilities, [])
@@ -760,7 +803,6 @@ class OqParam(valid.ParamSet):
         valid.Choice('measured', 'inferred'), 'measured')
     reference_vs30_value = valid.Param(
         valid.positivefloat, numpy.nan)
-    reference_siteclass = valid.Param(valid.Choice('A', 'B', 'C', 'D'), 'D')
     reference_backarc = valid.Param(valid.boolean, False)
     region = valid.Param(valid.wkt_polygon, None)
     region_grid_spacing = valid.Param(valid.positivefloat, None)
@@ -784,6 +826,7 @@ class OqParam(valid.ParamSet):
         valid.compose(valid.nonzero, valid.positiveint), 1)
     ses_seed = valid.Param(valid.positiveint, 42)
     shakemap_id = valid.Param(valid.nice_string, None)
+    shakemap_uri = valid.Param(valid.dictionary, {})
     shift_hypo = valid.Param(valid.boolean, False)
     site_effects = valid.Param(valid.boolean, False)  # shakemap amplification
     sites = valid.Param(valid.NoneOr(valid.coordinates), None)
@@ -829,8 +872,8 @@ class OqParam(valid.ParamSet):
                 for key, value in self.inputs['reqv'].items()}
 
     def __init__(self, **names_vals):
-        if '_job_id' in names_vals:  # called from engine
-            del names_vals['_job_id']
+        if '_log' in names_vals:  # called from engine
+            del names_vals['_log']
 
         # support legacy names
         for name in list(names_vals):
@@ -854,9 +897,6 @@ class OqParam(valid.ParamSet):
                 'region_constraint is obsolete, use region instead')
             self.region = valid.wkt_polygon(
                 names_vals.pop('region_constraint'))
-        self.risk_investigation_time = (
-            self.risk_investigation_time or self.investigation_time)
-        self.collapse_level = int(self.collapse_level)
         if ('intensity_measure_types_and_levels' in names_vals and
                 'intensity_measure_types' in names_vals):
             logging.warning('Ignoring intensity_measure_types since '
@@ -891,6 +931,20 @@ class OqParam(valid.ParamSet):
                               'pointsource_distance!' % self.inputs['job_ini'])
 
         self._risk_files = get_risk_files(self.inputs)
+        if self.risk_files:
+            # checks for risk_files
+            hc = self.hazard_calculation_id
+            if 'damage' in self.calculation_mode and not hc:
+                ok = any('fragility' in key for key in self._risk_files)
+                if not ok:
+                    raise InvalidFile('Missing fragility files in %s' %
+                                      self.inputs['job_ini'])
+            elif ('risk' in self.calculation_mode and
+                  self.calculation_mode != 'multi_risk' and not hc):
+                ok = any('vulnerability' in key for key in self._risk_files)
+                if not ok:
+                    raise InvalidFile('Missing vulnerability files in %s' %
+                                      self.inputs['job_ini'])
 
         if self.hazard_precomputed() and self.job_type == 'risk':
             self.check_missing('site_model', 'debug')
@@ -925,6 +979,11 @@ class OqParam(valid.ParamSet):
             raise ValueError('Unknown key %s_file in %s' %
                              (unknown.pop(), self.inputs['job_ini']))
 
+        # check return_periods vs poes
+        if self.return_periods and not self.poes and self.investigation_time:
+            self.poes = 1 - numpy.exp(
+                - self.investigation_time / numpy.array(self.return_periods))
+
         # checks for disaggregation
         if self.calculation_mode == 'disaggregation':
             if not self.poes_disagg and self.poes:
@@ -961,22 +1020,6 @@ class OqParam(valid.ParamSet):
                     '%s: conditional_loss_poes are not defined '
                     'for classical_damage calculations' % job_ini)
 
-        # checks for event_based_risk
-        if (self.calculation_mode == 'event_based_risk' and
-                self.asset_correlation not in (0, 1)):
-            raise ValueError('asset_correlation != {0, 1} is no longer'
-                             ' supported in %s' % job_ini)
-        elif (self.calculation_mode == 'event_based_risk' and
-              not self.ground_motion_fields):
-            raise ValueError('ground_motion_fields must be set to true in %s'
-                             % job_ini)
-
-        # checks for ebrisk
-        if self.calculation_mode == 'ebrisk':
-            if self.risk_investigation_time is None:
-                raise InvalidFile('Please set the risk_investigation_time in'
-                                  ' %s' % job_ini)
-
         # check for GMFs from file
         if (self.inputs.get('gmfs', '').endswith('.csv')
                 and 'sites' not in self.inputs and self.sites is None):
@@ -1011,16 +1054,48 @@ class OqParam(valid.ParamSet):
             check_same_levels(self.imtls)
 
         if ('amplification' in self.inputs and
-            self.amplification_method == 'convolution' and
-            not self.soil_intensities):
-                raise InvalidFile('%s: The soil_intensities must be defined'
-                     % job_ini)
+            self.amplification_method == 'convolution' and not
+                self.soil_intensities):
+            raise InvalidFile('%s: The soil_intensities must be defined'
+                              % job_ini)
+
+    def validate(self):
+        """
+        Set self.loss_names
+        """
+        from openquake.commonlib import datastore  # avoid circular import
+        if self.hazard_calculation_id:
+            with datastore.read(self.hazard_calculation_id) as ds:
+                self._parent = ds['oqparam']
+        else:
+            self._parent = None
+        # set all_cost_types
+        # rt has the form 'vulnerability/structural', 'fragility/...', ...
+        costtypes = set(rt.rsplit('/')[1] for rt in self.risk_files)
+        if not costtypes and self.hazard_calculation_id:
+            try:
+                self._risk_files = rfs = get_risk_files(self._parent.inputs)
+                costtypes = set(rt.rsplit('/')[1] for rt in rfs)
+            except OSError:  # FileNotFound for wrong hazard_calculation_id
+                pass
+        self.all_cost_types = sorted(costtypes)
+
+        # fix minimum_asset_loss
+        self.minimum_asset_loss = {
+            ln: calc.filters.getdefault(self.minimum_asset_loss, ln)
+            for ln in self.loss_names}
+
+        super().validate()
+        self.check_source_model()
 
     def check_gsims(self, gsims):
         """
         :param gsims: a sequence of GSIM instances
         """
-        imts = set(from_string(imt).name for imt in self.imtls)
+        imts = set()
+        for imt in self.imtls:
+            im = from_string(imt)
+            imts.add("SA" if imt.startswith("SA") else im.string)
         for gsim in gsims:
             if hasattr(gsim, 'weight'):  # disable the check
                 continue
@@ -1041,13 +1116,14 @@ class OqParam(valid.ParamSet):
                 for param in gsim.REQUIRES_SITES_PARAMETERS:
                     if param in ('lon', 'lat'):  # no check
                         continue
-                    param_name = self.siteparam[param]
-                    param_value = getattr(self, param_name)
-                    if (isinstance(param_value, float) and
-                            numpy.isnan(param_value)):
-                        raise ValueError(
-                            'Please set a value for %r, this is required by '
-                            'the GSIM %s' % (param_name, gsim))
+                    elif param in site.param:  # mandatory params
+                        param_name = site.param[param]
+                        param_value = getattr(self, param_name)
+                        if (isinstance(param_value, float) and
+                                numpy.isnan(param_value)):
+                            raise ValueError(
+                                'Please set a value for %r, this is required '
+                                'by the GSIM %s' % (param_name, gsim))
 
     @property
     def tses(self):
@@ -1059,7 +1135,7 @@ class OqParam(valid.ParamSet):
                 (self.number_of_logic_tree_samples or 1))
 
     @property
-    def ses_ratio(self):
+    def time_ratio(self):
         """
         The ratio
 
@@ -1079,21 +1155,6 @@ class OqParam(valid.ParamSet):
         imtls = self.hazard_imtls or self.risk_imtls
         return DictArray(imtls) if imtls else {}
 
-    @property
-    def all_cost_types(self):
-        """
-        Return the cost types of the computation (including `occupants`
-        if it is there) in order.
-        """
-        # rt has the form 'vulnerability/structural', 'fragility/...', ...
-        costtypes = set(rt.rsplit('/')[1] for rt in self.risk_files)
-        if not costtypes and self.hazard_calculation_id:
-            with util.read(self.hazard_calculation_id) as ds:
-                parent = ds['oqparam']
-            self._risk_files = get_risk_files(parent.inputs)
-            costtypes = set(rt.rsplit('/')[1] for rt in self.risk_files)
-        return sorted(costtypes)
-
     @property
     def min_iml(self):
         """
@@ -1105,9 +1166,7 @@ class OqParam(valid.ParamSet):
                 try:
                     mini[imt] = calc.filters.getdefault(mini, imt)
                 except KeyError:
-                    raise ValueError(
-                        'The parameter `minimum_intensity` in the job.ini '
-                        'file is missing the IMT %r' % imt)
+                    mini[imt] = 0
         if 'default' in mini:
             del mini['default']
         return numpy.array([mini.get(imt) or 1E-10 for imt in self.imtls])
@@ -1212,25 +1271,26 @@ class OqParam(valid.ParamSet):
         """
         Loss types plus insured types, if any
         """
+        if not hasattr(self, "all_cost_types"):  # for hazard
+            return []
         names = []
-        for lt, _ in self.loss_dt_list():
+        for lt in self.all_cost_types:
             names.append(lt)
         for name in self.inputs.get('insurance', []):
             names.append(lt + '_ins')
         return names
 
-    def loss_dt(self, dtype=F32):
+    def loss_dt(self, dtype=F64):
         """
         :returns: a composite dtype based on the loss types including occupants
         """
         return numpy.dtype(self.loss_dt_list(dtype))
 
-    def loss_dt_list(self, dtype=F32):
+    def loss_dt_list(self, dtype=F64):
         """
         :returns: a data type list [(loss_name, dtype), ...]
         """
-        loss_types = self.all_cost_types
-        dts = [(str(lt), dtype) for lt in loss_types]
+        dts = [(str(lt), dtype) for lt in self.loss_names]
         return dts
 
     def loss_maps_dt(self, dtype=F32):
@@ -1365,7 +1425,21 @@ class OqParam(valid.ParamSet):
         """
         hazard_calculation_id must be set if shakemap_id is set
         """
-        return self.hazard_calculation_id if self.shakemap_id else True
+        if self.shakemap_uri:
+            kind = self.shakemap_uri['kind']
+            sig = inspect.signature(get_array[kind])
+            # parameters without default value
+            params = [p.name for p in list(
+                sig.parameters.values()) if p.default is p.empty]
+            all_params = list(sig.parameters)
+            if not all(p in list(self.shakemap_uri) for p in params) or \
+                    not all(p in all_params for p in list(self.shakemap_uri)):
+                raise ValueError(
+                    'Error in shakemap_uri: Expected parameters %s, '
+                    'valid parameters %s, got %s' %
+                    (params, all_params, list(self.shakemap_uri)))
+        return self.hazard_calculation_id if (
+            self.shakemap_id or self.shakemap_uri) else True
 
     def is_valid_truncation_level(self):
         """
@@ -1541,6 +1615,32 @@ class OqParam(valid.ParamSet):
             self.complex_fault_mesh_spacing = self.rupture_mesh_spacing
         return True
 
+    def is_valid_collect_rlzs(self):
+        """
+        sampling_method must be early_weights, only the mean is available,
+        and number_of_logic_tree_samples must be greater than 1.
+        """
+        if self.calculation_mode == 'event_based_damage':
+            if not self.investigation_time:
+                ini = self.inputs['job_ini']
+                raise InvalidFile('Missing investigation_time in %s' % ini)
+            self.collect_rlzs = True
+            return True
+        elif self.collect_rlzs is False:
+            return True
+        elif self.hazard_calculation_id:
+            n = self._parent.number_of_logic_tree_samples
+            if n == 0:
+                raise ValueError('collect_rlzs=true can only be specified if '
+                                 'the parent hazard calculation used sampling')
+            elif n != self.number_of_logic_tree_samples:
+                raise ValueError('Please specify number_of_logic_tree_samples'
+                                 '=%d' % n)
+        hstats = list(self.hazard_stats())
+        nostats = not hstats or hstats == ['mean']
+        return nostats and self.number_of_logic_tree_samples > 1 and (
+            self.sampling_method == 'early_weights')
+
     def check_uniform_hazard_spectra(self):
         ok_imts = [imt for imt in self.imtls if imt == 'PGA' or
                    imt.startswith('SA')]
@@ -1597,3 +1697,21 @@ class OqParam(valid.ParamSet):
             name = name.split()[-1]
             dic[name] = doc
         return dic
+
+    def __toh5__(self):
+        return hdf5.dumps(vars(self)), {}
+
+    def __fromh5__(self, array, attrs):
+        if isinstance(array, numpy.ndarray):  # old format <= 3.11
+            dd = collections.defaultdict(dict)
+            for (name_, literal_) in array:
+                name = python3compat.decode(name_)
+                literal = python3compat.decode(literal_)
+                if '.' in name:
+                    k1, k2 = name.split('.', 1)
+                    dd[k1][k2] = ast.literal_eval(literal)
+                else:
+                    dd[name] = ast.literal_eval(literal)
+            vars(self).update(dd)
+        else:  # new format >= 3.12
+            vars(self).update(json.loads(python3compat.decode(array)))
