@@ -25,6 +25,7 @@ import zlib
 import pickle
 import shutil
 import zipfile
+import pathlib
 import logging
 import tempfile
 import functools
@@ -45,11 +46,9 @@ from openquake.hazardlib.calc.filters import SourceFilter
 from openquake.hazardlib.calc.gmf import CorrelationButNoInterIntraStdDevs
 from openquake.hazardlib import (
     source, geo, site, imt, valid, sourceconverter, nrml, InvalidFile, pmf)
-from openquake.hazardlib.source import rupture
-from openquake.hazardlib.calc.stochastic import rupture_dt
 from openquake.hazardlib.probability_map import ProbabilityMap
 from openquake.hazardlib.geo.utils import BBoxError, cross_idl
-from openquake.risklib import asset, riskmodels
+from openquake.risklib import asset, riskmodels, scientific
 from openquake.risklib.riskmodels import get_risk_functions
 from openquake.commonlib.oqvalidation import OqParam
 from openquake.commonlib.source_reader import get_csm
@@ -144,11 +143,23 @@ def unzip_rename(zpath, name):
     return xpath
 
 
+def normpath(fnames, base_path):
+    vals = []
+    for fname in fnames:
+        val = os.path.normpath(os.path.join(base_path, fname))
+        if not os.path.exists(val):
+            raise OSError('No such file: %s' % val)
+        vals.append(val)
+    return vals
+
+
 def normalize(key, fnames, base_path):
     input_type, _ext = key.rsplit('_', 1)
     filenames = []
     for val in fnames:
-        if '://' in val:
+        if isinstance(val, list):
+            val = normpath(val, base_path)
+        elif '://' in val:
             # get the data from an URL
             resp = requests.get(val)
             _, val = val.rsplit('/', 1)
@@ -156,7 +167,7 @@ def normalize(key, fnames, base_path):
                 f.write(resp.content)
         elif os.path.isabs(val):
             raise ValueError('%s=%s is an absolute path' % (key, val))
-        if val.endswith('.zip'):
+        elif val.endswith('.zip'):
             zpath = os.path.normpath(os.path.join(base_path, val))
             if key == 'exposure_file':
                 name = 'exposure.xml'
@@ -167,7 +178,7 @@ def normalize(key, fnames, base_path):
             val = unzip_rename(zpath, name)
         else:
             val = os.path.normpath(os.path.join(base_path, val))
-        if not os.path.exists(val):
+        if isinstance(val, str) and not os.path.exists(val):
             # tested in archive_err_2
             raise OSError('No such file: %s' % val)
         filenames.append(val)
@@ -220,6 +231,8 @@ def get_params(job_ini, kw={}):
     :returns:
         A dictionary of parameters
     """
+    if isinstance(job_ini, pathlib.Path):
+        job_ini = str(job_ini)
     if job_ini.startswith(('http://', 'https://')):
         resp = requests.get(job_ini)
         job_ini = gettemp(suffix='.zip')
@@ -254,12 +267,13 @@ def get_params(job_ini, kw={}):
     return params
 
 
-def get_oqparam(job_ini, pkg=None, calculators=None, kw={}, validate=1):
+def get_oqparam(job_ini, pkg=None, calculators=None, kw={}):
     """
     Parse a dictionary of parameters from an INI-style config file.
 
     :param job_ini:
-        Path to configuration file/archive or dictionary of parameters
+        Path to configuration file/archive or
+        dictionary of parameters with a key "calculation_mode"
     :param pkg:
         Python package where to find the configuration file (optional)
     :param calculators:
@@ -267,11 +281,9 @@ def get_oqparam(job_ini, pkg=None, calculators=None, kw={}, validate=1):
         valid choices for `calculation_mode`
     :param kw:
         Dictionary of strings to override the job parameters
-    :param validate:
-        Flag. By default it is true and the parameters are validated
     :returns:
         An :class:`openquake.commonlib.oqvalidation.OqParam` instance
-        containing the validate and casted parameters/values parsed from
+        containing the validated and casted parameters/values parsed from
         the job.ini file as well as a subdictionary 'inputs' containing
         absolute paths to all of the files referenced in the job.ini, keyed by
         the parameter name.
@@ -305,9 +317,7 @@ def get_oqparam(job_ini, pkg=None, calculators=None, kw={}, validate=1):
                 {imt: imtls[imt]})
         job_ini['save_disk_space'] = 'true'
     oqparam = OqParam(**job_ini)
-    if validate and '_job_id' not in job_ini:
-        oqparam.check_source_model()
-        oqparam.validate()
+    oqparam.validate()
     return oqparam
 
 
@@ -368,7 +378,7 @@ def get_mesh(oqparam, h5=None):
         c = (coords[start:stop] if header[0] == 'site_id'
              else sorted(coords[start:stop]))
         # NB: Notice the sort=False below
-        # Calculations starting from ground motion fields input by the user
+        # Calculations starting from predefined ground motion fields
         # require at least two input files related to the gmf data:
         #   1. A sites.csv file, listing {site_id, lon, lat} tuples
         #   2. A gmfs.csv file, listing {event_id, site_id, gmv[IMT1],
@@ -491,6 +501,25 @@ def get_site_model(oqparam):
     return numpy.concatenate(arrays)
 
 
+def count_old_style(gsim_lt):
+    # count the number of old style GMPEs
+    old = 0
+    for gsims in gsim_lt.values.values():
+        for gsim in gsims:
+            old += 'get_mean_and_stddevs' in gsim.__class__.__dict__
+    return old
+
+
+def count_no_vect(gsim_lt):
+    # count the number of not vectorize GMPEs
+    no = 0
+    for gsims in gsim_lt.values.values():
+        for gsim in gsims:
+            compute = getattr(gsim.__class__, 'compute')
+            no += 'ctx' not in compute.__annotations__
+    return no
+
+
 def get_site_collection(oqparam, h5=None):
     """
     Returns a SiteCollection instance by looking at the points and the
@@ -552,6 +581,8 @@ def get_gsim_lt(oqparam, trts=('*',)):
     gmfcorr = oqparam.correl_model
     for trt, gsims in gsim_lt.values.items():
         for gsim in gsims:
+            # NB: gsim.DEFINED_FOR_TECTONIC_REGION_TYPE can be != trt,
+            # but it is not an error, it is actually the most common case!
             if gmfcorr and (gsim.DEFINED_FOR_STANDARD_DEVIATION_TYPES ==
                             {StdDev.TOTAL}):
                 raise CorrelationButNoInterIntraStdDevs(gmfcorr, gsim)
@@ -572,68 +603,28 @@ def get_gsim_lt(oqparam, trts=('*',)):
         logging.info('Collapsing the gsim logic tree')
         gsim_lt = gsim_lt.collapse(oqparam.collapse_gsim_logic_tree)
     gsim_lt_cache[key] = gsim_lt
+    if trts != ('*',):  # not in get_input_files
+        old_style = count_old_style(gsim_lt)
+        no_vect = count_no_vect(gsim_lt)
+        if old_style:
+            logging.info('There are %d old style GMPEs', old_style)
+        if no_vect:
+            logging.info('There are %d not vectorized GMPEs', no_vect)
     return gsim_lt
 
 
-def get_ruptures(fname_csv):
-    """
-    Read ruptures in CSV format and return an ArrayWrapper
-    """
-    if not rupture.BaseRupture._code:
-        rupture.BaseRupture.init()  # initialize rupture codes
-    code = rupture.BaseRupture.str2code
-    aw = hdf5.read_csv(fname_csv, rupture.rupture_dt)
-    trts = aw.trts
-    rups = []
-    geoms = []
-    n_occ = 1
-    for u, row in enumerate(aw.array):
-        hypo = row['lon'], row['lat'], row['dep']
-        dic = json.loads(row['extra'])
-        mesh = F32(json.loads(row['mesh']))
-        s1, s2 = mesh.shape[1:]
-        rec = numpy.zeros(1, rupture_dt)[0]
-        rec['seed'] = row['seed']
-        rec['minlon'] = minlon = mesh[0].min()
-        rec['minlat'] = minlat = mesh[1].min()
-        rec['maxlon'] = maxlon = mesh[0].max()
-        rec['maxlat'] = maxlat = mesh[1].max()
-        rec['mag'] = row['mag']
-        rec['hypo'] = hypo
-        rate = dic.get('occurrence_rate', numpy.nan)
-        tup = (u, row['seed'], 'no-source', trts.index(row['trt']),
-               code[row['kind']], n_occ, row['mag'], row['rake'], rate,
-               minlon, minlat, maxlon, maxlat, hypo, u, 0, 0)
-        rups.append(tup)
-        points = mesh.flatten()  # lons + lats + deps
-        # FIXME: extend to MultiSurfaces
-        geoms.append(numpy.concatenate([[1], [s1, s2], points]))
-    if not rups:
-        return ()
-    dic = dict(geom=numpy.array(geoms, object))
-    # NB: PMFs for nonparametric ruptures are missing
-    return hdf5.ArrayWrapper(numpy.array(rups, rupture_dt), dic)
-
-
 def get_rupture(oqparam):
     """
-    Read the `rupture_model` file and by filter the site collection
+    Read the `rupture_model` XML file and by filter the site collection
 
     :param oqparam:
         an :class:`openquake.commonlib.oqvalidation.OqParam` instance
     :returns:
         an hazardlib rupture
     """
-    rup_model = oqparam.inputs['rupture_model']
-    if rup_model.endswith('.csv'):
-        return rupture.from_array(hdf5.read_csv(rup_model))
-    if rup_model.endswith('.xml'):
-        [rup_node] = nrml.read(rup_model)
-        conv = sourceconverter.RuptureConverter(
-            oqparam.rupture_mesh_spacing, oqparam.complex_fault_mesh_spacing)
-        rup = conv.convert_node(rup_node)
-    else:
-        raise ValueError('Unrecognized ruptures model %s' % rup_model)
+    [rup_node] = nrml.read(oqparam.inputs['rupture_model'])
+    conv = sourceconverter.RuptureConverter(oqparam.rupture_mesh_spacing)
+    rup = conv.convert_node(rup_node)
     rup.tectonic_region_type = '*'  # there is not TRT for scenario ruptures
     rup.rup_id = oqparam.ses_seed
     return rup
@@ -702,17 +693,20 @@ def get_full_lt(oqparam):
 
 
 def save_source_info(csm, h5):
+    """
+    Creates source_info, source_wkt, trt_smrs, toms
+    """
     data = {}  # src_id -> row
     wkts = []
     lens = []
     for sg in csm.src_groups:
         for src in sg:
-            lens.append(len(src.et_ids))
+            lens.append(len(src.trt_smrs))
             row = [src.source_id, src.grp_id, src.code,
                    0, 0, 0, csm.full_lt.trti[src.tectonic_region_type], 0]
             wkts.append(src._wkt)
             data[src.id] = row
-    logging.info('There are %d groups and %d sources with len(et_ids)=%.2f',
+    logging.info('There are %d groups and %d sources with len(trt_smrs)=%.2f',
                  len(csm.src_groups), sum(len(sg) for sg in csm.src_groups),
                  numpy.mean(lens))
     csm.source_info = data  # src_id -> row
@@ -721,7 +715,20 @@ def save_source_info(csm, h5):
         # avoid hdf5 damned bug by creating source_info in advance
         hdf5.create(h5, 'source_info', source_info_dt, attrs=attrs)
         h5['source_wkt'] = numpy.array(wkts, hdf5.vstr)
-        h5['et_ids'] = csm.get_et_ids()
+        h5['trt_smrs'] = csm.get_trt_smrs()
+        h5['toms'] = numpy.array(
+            [get_tom_name(sg) for sg in csm.src_groups], hdf5.vstr)
+
+
+def get_tom_name(sg):
+    """
+    :param sg: a source group instance
+    :returns: name of the associated temporal occurrence model
+    """
+    if sg.temporal_occurrence_model:
+        return sg.temporal_occurrence_model.__class__.__name__
+    else:
+        return 'PoissonTOM'
 
 
 def _check_csm(csm, oqparam, h5):
@@ -795,13 +802,12 @@ def get_composite_source_model(oqparam, h5=None):
                 csm.full_lt = full_lt
             if h5:
                 # avoid errors with --reuse_hazard
-                h5['et_ids'] = csm.get_et_ids()
-                hdf5.create(h5, 'source_info', source_info_dt)
+                save_source_info(csm, h5)
             _check_csm(csm, oqparam, h5)
             return csm
 
     # read and process the composite source model from the input files
-    csm = get_csm(oqparam, full_lt,  h5)
+    csm = get_csm(oqparam, full_lt, h5)
     save_source_info(csm, h5)
     if oqparam.cachedir and not oqparam.is_ucerf():
         logging.info('Saving %s', fname)
@@ -819,19 +825,8 @@ def get_imts(oqparam):
     return list(map(imt.from_string, sorted(oqparam.imtls)))
 
 
-def get_amplification(oqparam):
-    """
-    :returns: a DataFrame (ampcode, level, PGA, SA() ...)
-    """
-    fname = oqparam.inputs['amplification']
-    df = hdf5.read_csv(fname, {'ampcode': site.ampcode_dt, None: F64},
-                       index='ampcode')
-    df.fname = fname
-    return df
-
-
-def _cons_coeffs(records, limit_states):
-    dtlist = [(lt, F32) for lt in records['loss_type']]
+def _cons_coeffs(records, loss_types, limit_states):
+    dtlist = [(lt, F32) for lt in loss_types]
     coeffs = numpy.zeros(len(limit_states), dtlist)
     for rec in records:
         coeffs[rec['loss_type']] = [rec[ds] for ds in limit_states]
@@ -846,18 +841,35 @@ def get_crmodel(oqparam):
         an :class:`openquake.commonlib.oqvalidation.OqParam` instance
     """
     risklist = get_risk_functions(oqparam)
+    if not oqparam.limit_states and risklist.limit_states:
+        oqparam.limit_states = risklist.limit_states
+    elif 'damage' in oqparam.calculation_mode and risklist.limit_states:
+        assert oqparam.limit_states == risklist.limit_states
+    loss_types = oqparam.loss_dt().names
     consdict = {}
     if 'consequence' in oqparam.inputs:
-        # build consdict of the form cname_by_tagname -> tag -> array
-        for by, fname in oqparam.inputs['consequence'].items():
+        # build consdict of the form consequence_by_tagname -> tag -> array
+        for by, fnames in oqparam.inputs['consequence'].items():
+            if isinstance(fnames, str):  # single file
+                fnames = [fnames]
             dtypedict = {
-                by: str, 'cname': str, 'loss_type': str, None: float}
-            dic = group_array(
-                hdf5.read_csv(fname, dtypedict).array, 'cname')
-            for cname, group in dic.items():
-                bytag = {tag: _cons_coeffs(grp, risklist.limit_states)
-                         for tag, grp in group_array(group, by).items()}
-                consdict['%s_by_%s' % (cname, by)] = bytag
+                by: str, 'consequence': str, 'loss_type': str, None: float}
+
+            # i.e. collapsed.csv, fatalities.csv, ...
+            array = numpy.concatenate([
+                hdf5.read_csv(fname, dtypedict).array for fname in fnames])
+
+            dic = group_array(array, 'consequence')
+            for consequence, group in dic.items():
+                if consequence not in scientific.KNOWN_CONSEQUENCES:
+                    raise InvalidFile('Unknown consequence %s in %s' %
+                                      (consequence, fnames))
+                bytag = {
+                    tag: _cons_coeffs(grp, loss_types, risklist.limit_states)
+                    for tag, grp in group_array(group, by).items()}
+                consdict['%s_by_%s' % (consequence, by)] = bytag
+    # for instance consdict['collapsed_by_taxonomy']['W_LFM-DUM_H3']
+    # is [(0.05,), (0.2 ,), (0.6 ,), (1.  ,)] for damage state and structural
     crm = riskmodels.CompositeRiskModel(oqparam, risklist, consdict)
     return crm
 
@@ -883,7 +895,8 @@ def get_exposure(oqparam):
     exposure = asset.Exposure.read(
         oqparam.inputs['exposure'], oqparam.calculation_mode,
         oqparam.region, oqparam.ignore_missing_costs,
-        by_country='country' in oqparam.aggregate_by)
+        by_country='country' in oqparam.aggregate_by,
+        errors='ignore' if oqparam.ignore_encoding_errors else None)
     exposure.mesh, exposure.assets_by_site = exposure.get_mesh_assets_by_site()
     if oqparam.cachedir:
         logging.info('Saving %s', fname)
@@ -909,8 +922,8 @@ def get_sitecol_assetcol(oqparam, haz_sitecol=None, cost_types=()):
     if oqparam.region_grid_spacing:
         haz_distance = oqparam.region_grid_spacing * 1.414
         if haz_distance != asset_hazard_distance:
-            logging.info('Using asset_hazard_distance=%d km instead of %d km',
-                         haz_distance, asset_hazard_distance)
+            logging.debug('Using asset_hazard_distance=%d km instead of %d km',
+                          haz_distance, asset_hazard_distance)
     else:
         haz_distance = asset_hazard_distance
 
@@ -923,8 +936,8 @@ def get_sitecol_assetcol(oqparam, haz_sitecol=None, cost_types=()):
         for sid, assets in zip(sitecol.sids, assets_by):
             assets_by_site[sid] = assets
             num_assets += len(assets)
-        logging.info(
-            'Associated %d assets to %d sites', num_assets, len(sitecol))
+        logging.info('Associated {:_d} assets to {:_d} sites'.format(
+            num_assets, len(sitecol)))
     else:
         # asset sites and hazard sites are the same
         sitecol = haz_sitecol
@@ -970,7 +983,7 @@ def taxonomy_mapping(oqparam, taxonomies):
         lst = [[(taxo, 1)] for taxo in taxonomies]
         return {lt: lst for lt in oqparam.loss_names}
     dic = oqparam.inputs['taxonomy_mapping']
-    if isinstance(dic, str):  # filename
+    if isinstance(dic, str):  # same file for all loss_types
         dic = {lt: dic for lt in oqparam.loss_names}
     return {lt: _taxonomy_mapping(dic[lt], taxonomies)
             for lt in oqparam.loss_names}
@@ -1127,6 +1140,19 @@ def reduce_source_model(smlt_file, source_ids, remove=True):
     return good, total
 
 
+def get_shapefiles(dirname):
+    """
+    :param dirname: directory containing the shapefiles
+    :returns: list of shapefiles
+    """
+    out = []
+    extensions = ('.shp', '.dbf', '.prj', '.shx')
+    for fname in os.listdir(dirname):
+        if fname.endswith(extensions):
+            out.append(os.path.join(dirname, fname))
+    return out
+
+
 def get_input_files(oqparam, hazard=False):
     """
     :param oqparam: an OqParam instance
@@ -1134,6 +1160,20 @@ def get_input_files(oqparam, hazard=False):
     :returns: input path names in a specific order
     """
     fnames = set()  # files entering in the checksum
+    uri = oqparam.shakemap_uri
+    if isinstance(uri, dict) and uri:
+        # local files
+        for key, val in uri.items():
+            if key == 'fname' or key.endswith('_url'):
+                val = val.replace('file://', '')
+                fname = os.path.join(oqparam.base_path, val)
+                if os.path.exists(fname):
+                    uri[key] = fname
+                    fnames.add(fname)
+        # additional separate shapefiles
+        if uri['kind'] == 'shapefile' and not uri['fname'].endswith('.zip'):
+            fnames.update(get_shapefiles(os.path.dirname(fname)))
+
     for key in oqparam.inputs:
         fname = oqparam.inputs[key]
         if hazard and key not in ('source_model_logic_tree',
@@ -1158,7 +1198,11 @@ def get_input_files(oqparam, hazard=False):
                 fnames.update(exp.datafiles)
             fnames.update(fname)
         elif isinstance(fname, dict):
-            fnames.update(fname.values())
+            for key, val in fname.items():
+                if isinstance(val, list):  # list of files
+                    fnames.update(val)
+                else:
+                    fnames.add(val)
         elif isinstance(fname, list):
             for f in fname:
                 if f == oqparam.input_dir:
