@@ -18,20 +18,22 @@
 
 import ast
 import sys
+import copy
 import operator
+import itertools
 from contextlib import contextmanager
 import numpy
 from scipy.spatial import cKDTree
 
 from openquake.baselib.python3compat import raise_
-from openquake.hazardlib import site
+from openquake.hazardlib import site, mfd
 from openquake.hazardlib.geo.utils import (
     KM_TO_DEGREES, angular_distance, fix_lon, get_bounding_box,
     get_longitudinal_extent, BBoxError, spherical_to_cartesian)
 
 U32 = numpy.uint32
 MAX_DISTANCE = 2000  # km, ultra big distance used if there is no filter
-et_id = operator.attrgetter('et_id')
+trt_smr = operator.attrgetter('trt_smr')
 
 
 @contextmanager
@@ -118,16 +120,16 @@ class MagDepDistance(dict):
         >>> md.interp(dict(default=[5.0, 5.1, 5.2])); md.ddic
         {'default': {'5.00': 50.0, '5.10': 50.0, '5.20': 50.0}}
         """
-        items_by_trt = floatdict(value.replace('?', '-1'))
+        items_by_trt = floatdict(value)
         self = cls()
         for trt, items in items_by_trt.items():
             if isinstance(items, list):
-                self[trt] = unique_sorted(items)
+                self[trt] = unique_sorted([tuple(it) for it in items])
                 for mag, dist in self[trt]:
                     if mag < 1 or mag > 10:
                         raise ValueError('Invalid magnitude %s' % mag)
             else:  # assume scalar distance
-                assert items == -1 or items >= 0, items
+                assert items >= 0, items
                 self[trt] = [(1., items), (10., items)]
         return self
 
@@ -164,12 +166,6 @@ class MagDepDistance(dict):
         """
         return {trt: self[trt][-1][1] for trt in self}
 
-    def suggested(self):
-        """
-        :returns: True if there is a ? for any TRT
-        """
-        return any(self[trt][-1][1] == -1 for trt in self)
-
     def get_bounding_box(self, lon, lat, trt=None, mag=None):
         """
         Build a bounding box around the given lon, lat by computing the
@@ -223,7 +219,7 @@ def split_source(src):
     if len(splits) > 1:
         for i, split in enumerate(splits):
             split.source_id = '%s:%s' % (src.source_id, i)
-            split.et_id = src.et_id
+            split.trt_smr = src.trt_smr
             split.grp_id = grp_id
             split.id = src.id
             if has_samples:
@@ -233,7 +229,7 @@ def split_source(src):
     elif splits:  # single source
         [s] = splits
         s.source_id = src.source_id
-        s.et_id = src.et_id
+        s.trt_smr = src.trt_smr
         s.grp_id = grp_id
         s.id = src.id
         if has_samples:
@@ -316,20 +312,47 @@ class SourceFilter(object):
         Returns the sites within the integration distance from the source,
         or None.
         """
-        source_indices = list(self.filter([source]))
-        if source_indices:
-            return self.sitecol.filtered(source_indices[0][1])
+        source_sites = list(self.filter([source]))
+        if source_sites:
+            return source_sites[0][1]
 
     def split(self, sources):
         """
         :yields: pairs (split, sites)
         """
-        for src, _indices in self.filter(sources):
+        for src, _sites in self.filter(sources):
             for s in split_source(src):
                 sites = self.get_close_sites(s)
                 if sites is not None:
                     yield s, sites
 
+    def split_less(self, sources):
+        """
+        :yields: pairs (split, sites)
+        """
+        for src, _sites in self.filter(sources):
+            if src.__class__.__name__.startswith(('Multi', 'Collapsed')):
+                # do not split
+                yield src, _sites
+            elif hasattr(src, 'get_annual_occurrence_rates'):
+                for mag, rate in src.get_annual_occurrence_rates():
+                    new = copy.copy(src)
+                    new.mfd = mfd.ArbitraryMFD([mag], [rate])
+                    new.num_ruptures = new.count_ruptures()
+                    sites = self.get_close_sites(new)
+                    if sites is not None:
+                        yield new, sites
+            else:  # nonparametric source
+                # data is a list of pairs (rup, pmf)
+                for mag, group in itertools.groupby(
+                        src.data, lambda pair: pair[0].mag):
+                    new = src.__class__(src.source_id, src.name,
+                                        src.tectonic_region_type, list(group))
+                    vars(new).update(vars(src))
+                    sites = self.get_close_sites(new)
+                    if sites is not None:
+                        yield new, sites
+
     # used in source and rupture prefiltering: it should not discard too much
     def close_sids(self, src_or_rec, trt=None, maxdist=None):
         """
@@ -374,7 +397,7 @@ class SourceFilter(object):
     def filter(self, sources):
         """
         :param sources: a sequence of sources
-        :yields: sources with indices
+        :yields: pairs (sources, sites)
         """
         if self.sitecol is None:  # nofilter
             for src in sources:
@@ -383,7 +406,7 @@ class SourceFilter(object):
         for src in sources:
             sids = self.close_sids(src)
             if len(sids):
-                yield src, sids
+                yield src, self.sitecol.filtered(sids)
 
     def __getitem__(self, slc):
         if slc.start is None and slc.stop is None:
