@@ -25,14 +25,14 @@ than 20 lines of code:
 
    import sys
    from openquake.commonlib import logs
-   from openquake.calculators.base import get_calc
+   from openquake.calculators.base import calculators
 
    def main(job_ini):
-       calc_id = logs.init()
-       calc = get_calc(job_ini, calc_id)
-       calc.run(individual_curves='true', shutdown=True)
-       print('The hazard curves are in %s::/hcurves-rlzs'
-             % calc.datastore.filename)
+       with logs.init('job', job_ini) as log:
+           calc = calculators(log.get_oqparam(), log.calc_id)
+           calc.run(individual_curves='true', shutdown=True)
+           print('The hazard curves are in %s::/hcurves-rlzs'
+                % calc.datastore.filename)
 
    if __name__ == '__main__':
        main(sys.argv[1])  # path to a job.ini file
@@ -52,7 +52,7 @@ from openquake.hazardlib.probability_map import (
 from openquake.hazardlib.gsim.base import ContextMaker, PmapMaker
 from openquake.hazardlib.calc.filters import SourceFilter
 from openquake.hazardlib.sourceconverter import SourceGroup
-from openquake.hazardlib.tom import FatedTOM
+from openquake.hazardlib.tom import PoissonTOM, FatedTOM
 
 
 def _cluster(imtls, tom, gsims, pmap):
@@ -78,7 +78,7 @@ def _cluster(imtls, tom, gsims, pmap):
     return pmap
 
 
-def classical(group, src_filter, gsims, param, monitor=Monitor()):
+def classical(group, sitecol, cmaker):
     """
     Compute the hazard curves for a set of sources belonging to the same
     tectonic region type for all the GSIMs associated to that TRT.
@@ -88,14 +88,9 @@ def classical(group, src_filter, gsims, param, monitor=Monitor()):
     :returns:
         a dictionary with keys pmap, calc_times, rup_data, extra
     """
-    if not hasattr(src_filter, 'sitecol'):  # do not filter
-        src_filter = SourceFilter(src_filter, {})
-
-    # Get the parameters assigned to the group
-    src_mutex = getattr(group, 'src_interdep', None) == 'mutex'
+    src_filter = SourceFilter(sitecol, cmaker.maximum_distance)
     cluster = getattr(group, 'cluster', None)
     trts = set()
-    maxradius = 0
     for src in group:
         if not src.num_ruptures:
             # src.num_ruptures may not be set, so it is set here
@@ -104,28 +99,21 @@ def classical(group, src_filter, gsims, param, monitor=Monitor()):
         if cluster:
             src.temporal_occurrence_model = FatedTOM(time_span=1)
         trts.add(src.tectonic_region_type)
-        if hasattr(src, 'radius'):  # for prefiltered point sources
-            maxradius = max(maxradius, src.radius)
-
-    param['maximum_distance'] = src_filter.integration_distance
     [trt] = trts  # there must be a single tectonic region type
-    cmaker = ContextMaker(trt, gsims, param, monitor)
-    pmap, rup_data, calc_times = PmapMaker(cmaker, src_filter, group).make()
-    extra = {}
-    extra['task_no'] = getattr(monitor, 'task_no', 0)
-    extra['trt'] = trt
-    extra['source_id'] = src.source_id
-    extra['grp_id'] = src.grp_id
-    extra['maxradius'] = maxradius
-    group_probability = getattr(group, 'grp_probability', None)
-    if src_mutex and group_probability:
-        pmap *= group_probability
-
+    assert trt == cmaker.trt, (trt, cmaker.trt)
+    cmaker.maximum_distance = src_filter.integration_distance
+    try:
+        cmaker.tom = group.temporal_occurrence_model
+    except AttributeError:  # got a list of sources, not a group
+        time_span = cmaker.investigation_time  # None for nonparametric
+        cmaker.tom = PoissonTOM(time_span) if time_span else None
+    if cluster:
+        cmaker.tom = FatedTOM(time_span=1)
+    dic = PmapMaker(cmaker, src_filter, group).make()
     if cluster:
         tom = getattr(group, 'temporal_occurrence_model')
-        pmap = _cluster(param['imtls'], tom, gsims, pmap)
-    return dict(pmap=pmap, calc_times=calc_times,
-                rup_data=rup_data, extra=extra)
+        dic['pmap'] = _cluster(cmaker.imtls, tom, cmaker.gsims, dic['pmap'])
+    return dic
 
 
 # not used in the engine, only in tests and possibly notebooks
@@ -171,53 +159,60 @@ def calc_hazard_curves(
         odic = groupby(groups, operator.attrgetter('tectonic_region_type'))
         groups = [SourceGroup(trt, odic[trt], 'src_group', 'indep', 'indep')
                   for trt in odic]
-    # ensure the sources have the right et_id
     idx = 0
+    span = None
     for i, grp in enumerate(groups):
         for src in grp:
-            if not hasattr(src, 'et_id'):
-                src.et_id = i  # fix et_id
+            tom = getattr(src, 'temporal_occurrence_model', None)
+            if tom:
+                span = tom.time_span
             src.grp_id = i
             src.id = idx
             idx += 1
     imtls = DictArray(imtls)
     shift_hypo = kwargs['shift_hypo'] if 'shift_hypo' in kwargs else False
     param = dict(imtls=imtls, truncation_level=truncation_level, reqv=reqv,
-                 cluster=grp.cluster, shift_hypo=shift_hypo)
+                 cluster=grp.cluster, shift_hypo=shift_hypo,
+                 investigation_time=kwargs.get('investigation_time', span))
     pmap = ProbabilityMap(imtls.size, 1)
     # Processing groups with homogeneous tectonic region
     mon = Monitor()
+    sitecol = getattr(srcfilter, 'sitecol', srcfilter)
+    if sitecol is not srcfilter:
+        param['maximum_distance'] = srcfilter.integration_distance
     for group in groups:
+        trt = group.trt
+        cmaker = ContextMaker(trt, [gsim_by_trt[trt]], param, mon)
         for src in group:
             if not src.nsites:  # not set
                 src.nsites = 1
-        gsim = gsim_by_trt[group[0].tectonic_region_type]
         if group.atomic:  # do not split
-            it = [classical(group, srcfilter, [gsim], param, mon)]
+            it = [classical(group, sitecol, cmaker)]
         else:  # split the group and apply `classical` in parallel
             it = apply(
-                classical, (group.sources, srcfilter, [gsim], param),
+                classical, (group.sources, sitecol, cmaker),
                 weight=operator.attrgetter('weight'))
         for dic in it:
             pmap |= dic['pmap']
-    sitecol = getattr(srcfilter, 'sitecol', srcfilter)
     return pmap.convert(imtls, len(sitecol.complete))
 
 
-# called in adv-manual/developing.rst
-def calc_hazard_curve(site1, src, gsims, oqparam):
+# called in adv-manual/developing.rst and in SingleSiteOptTestCase
+def calc_hazard_curve(site1, src, gsims, oqparam, monitor=Monitor()):
     """
     :param site1: site collection with a single site
     :param src: a seismic source object
     :param gsims: a list of GSIM objects
     :param oqparam: an object with attributes .maximum_distance, .imtls
+    :param monitor: a Monitor instance (optional)
     :returns: a ProbabilityCurve object
     """
     assert len(site1) == 1, site1
     trt = src.tectonic_region_type
-    cmaker = ContextMaker(trt, gsims, vars(oqparam))
+    cmaker = ContextMaker(trt, gsims, vars(oqparam), monitor)
+    cmaker.tom = src.temporal_occurrence_model
     srcfilter = SourceFilter(site1, oqparam.maximum_distance)
-    pmap, rup_data, calc_times = PmapMaker(cmaker, srcfilter, [src]).make()
+    pmap = PmapMaker(cmaker, srcfilter, [src]).make()['pmap']
     if not pmap:  # filtered away
         zero = numpy.zeros((oqparam.imtls.size, len(gsims)))
         return ProbabilityCurve(zero)
