@@ -28,17 +28,23 @@ import collections
 import numpy
 import pandas
 from scipy.interpolate import interp1d
-
+try:
+    import numba
+except ImportError:
+    numba = None
 from openquake.baselib import hdf5, parallel
 from openquake.baselib.general import (
-    AccumDict, DictArray, groupby, block_splitter)
+    AccumDict, DictArray, groupby, RecordBuilder)
 from openquake.baselib.performance import Monitor
 from openquake.hazardlib import imt as imt_module
-from openquake.hazardlib.tom import PoissonTOM
+from openquake.hazardlib.const import StdDev
+from openquake.hazardlib.tom import registry
+from openquake.hazardlib.site import site_param_dt
 from openquake.hazardlib.calc.filters import MagDepDistance
 from openquake.hazardlib.probability_map import ProbabilityMap
 from openquake.hazardlib.geo.surface import PlanarSurface
 
+STD_TYPES = (StdDev.TOTAL, StdDev.INTER_EVENT, StdDev.INTRA_EVENT)
 KNOWN_DISTANCES = frozenset(
     'rrup rx ry0 rjb rhypo repi rcdpp azimuth azimuth_cp rvolc closest_point'
     .split())
@@ -130,67 +136,61 @@ def get_num_distances(gsims):
     return len(dists)
 
 
-# used only in contexts_test.py
-def _make_pmap(ctxs, cmaker):
-    RuptureContext.temporal_occurrence_model = PoissonTOM(
-        cmaker.investigation_time)
-    # easy case of independent ruptures, useful for debugging
-    pmap = ProbabilityMap(cmaker.loglevels.size, len(cmaker.gsims))
-    for ctx, poes in cmaker.gen_ctx_poes(ctxs):
-        pnes = ctx.get_probability_no_exceedance(poes)  # (N, L, G)
-        for sid, pne in zip(ctx.sids, pnes):
-            pmap.setdefault(sid, 1.).array *= pne
-    return ~pmap
-
-
-def read_ctxs(dstore, slc=slice(None), req_site_params=None):
+def use_recarray(gsims):
     """
-     :returns: a list of contexts
+    :returns:
+        True if the `ctx` argument of gsim.compute is a recarray for all gsims
     """
-    sitecol = dstore['sitecol'].complete
-    site_params = {par: sitecol[par]
-                   for par in req_site_params or sitecol.array.dtype.names}
-    params = {n: dstore['rup/' + n][slc] for n in dstore['rup']}
-    ctxs = []
-    for u in range(len(params['mag'])):
-        ctx = RuptureContext()
-        for par, arr in params.items():
-            if par.endswith('_'):
-                par = par[:-1]
-            setattr(ctx, par, arr[u])
-        for par, arr in site_params.items():
-            setattr(ctx, par, arr[ctx.sids])
-        ctx.idx = {sid: idx for idx, sid in enumerate(ctx.sids)}
-        ctxs.append(ctx)
-    close_ctxs = [[] for sid in sitecol.sids]
-    for ctx in ctxs:
-        for sid in ctx.idx:
-            close_ctxs[sid].append(ctx)
-    return ctxs, close_ctxs
+    return all(gsim.compute.__annotations__.get("ctx") is numpy.recarray
+               for gsim in gsims)
 
 
 class ContextMaker(object):
     """
-    A class to manage the creation of contexts for distances, sites, rupture.
+    A class to manage the creation of contexts and to compute mean/stddevs
+    and possibly PoEs.
+
+    :param trt: a tectonic region type string
+    :param gsims: a list of GSIMs or a dictionary gsim -> rlz indices
+    :param param:
+       a dictionary of parameters like the maximum_distance, the IMTLs,
+       the investigation time, etc
+
+    NB: the trt can be different from the tectonic region type for which
+    the underlying GSIMs are defined. This is intentional.
     """
     REQUIRES = ['DISTANCES', 'SITES_PARAMETERS', 'RUPTURE_PARAMETERS']
+    rup_indep = True
+    tom = None
 
-    def __init__(self, trt, gsims, param=None, monitor=Monitor()):
-        param = param or {}  # empty in the gmpe-smtk
+    @property
+    def dtype(self):
+        """
+        :returns: dtype of the underlying ctx_builder
+        """
+        return self.ctx_builder.dtype
+
+    def __init__(self, trt, gsims, param, monitor=Monitor()):
+        param = param
         self.af = param.get('af', None)
         self.max_sites_disagg = param.get('max_sites_disagg', 10)
+        self.disagg_by_src = param.get('disagg_by_src')
         self.collapse_level = param.get('collapse_level', False)
         self.trt = trt
         self.gsims = gsims
-        self.single_site_opt = numpy.array(
-            [hasattr(gsim, 'get_mean_std1') for gsim in gsims])
         self.maximum_distance = (
             param.get('maximum_distance') or MagDepDistance({}))
+        self.minimum_distance = param.get('minimum_distance', 0)
         self.investigation_time = param.get('investigation_time')
+        if self.investigation_time:
+            self.tom = registry['PoissonTOM'](self.investigation_time)
+        self.ses_seed = param.get('ses_seed', 42)
+        self.ses_per_logic_tree_path = param.get('ses_per_logic_tree_path', 1)
         self.trunclevel = param.get('truncation_level')
         self.num_epsilon_bins = param.get('num_epsilon_bins', 1)
+        self.grp_id = param.get('grp_id', 0)
         self.effect = param.get('effect')
-        self.task_no = getattr(monitor, 'task_no', 0)
+        self.use_recarray = use_recarray(gsims)
         for req in self.REQUIRES:
             reqset = set()
             for gsim in gsims:
@@ -207,13 +207,29 @@ class ContextMaker(object):
         elif 'hazard_imtls' in param:
             self.imtls = DictArray(param['hazard_imtls'])
         else:
-            self.imtls = {}
-        self.imts = [imt_module.from_string(imt) for imt in self.imtls]
+            raise KeyError('Missing imtls in ContextMaker!')
+        try:
+            self.min_iml = param['min_iml']
+        except KeyError:
+            self.min_iml = [0. for imt in self.imtls]
         self.reqv = param.get('reqv')
         if self.reqv is not None:
             self.REQUIRES_DISTANCES.add('repi')
-        self.mon = monitor
-        self.ctx_mon = monitor('make_contexts', measuremem=False)
+        reqs = (sorted(self.REQUIRES_RUPTURE_PARAMETERS) +
+                sorted(self.REQUIRES_SITES_PARAMETERS) +
+                sorted(self.REQUIRES_DISTANCES))
+        dic = {}
+        for req in reqs:
+            if req in site_param_dt:
+                dt = site_param_dt[req]
+                if isinstance(dt, tuple):  # (string_, size)
+                    dic[req] = b''
+                else:
+                    dic[req] = dt(0)
+            else:
+                dic[req] = 0.
+        dic['sids'] = numpy.uint32(0)
+        self.ctx_builder = RecordBuilder(**dic)
         self.loglevels = DictArray(self.imtls) if self.imtls else {}
         self.shift_hypo = param.get('shift_hypo')
         with warnings.catch_warnings():
@@ -222,54 +238,53 @@ class ContextMaker(object):
             for imt, imls in self.imtls.items():
                 if imt != 'MMI':
                     self.loglevels[imt] = numpy.log(imls)
+        self.init_monitoring(monitor)
 
-        # instantiate monitors
+    def init_monitoring(self, monitor):
+        # instantiating child monitors, may be called in the workers
+        self.ctx_mon = monitor('make_contexts', measuremem=False)
         self.gmf_mon = monitor('computing mean_std', measuremem=False)
         self.poe_mon = monitor('get_poes', measuremem=False)
+        self.pne_mon = monitor('composing pnes', measuremem=False)
+        self.task_no = getattr(monitor, 'task_no', 0)
 
-    def multi(self, ctxs):
+    def read_ctxs(self, dstore, slc=None):
         """
-        :params ctxs: a list of contexts, all referring to a single point
-        :returns: a multiple RuptureContext
+        :param dstore: a DataStore instance
+        :param slice: a slice of contexts with the same grp_id
+        :returns: a list of contexts plus N lists of contexts for each site
         """
-        ctx = RuptureContext()
-        for par in self.REQUIRES_SITES_PARAMETERS:
-            setattr(ctx, par, getattr(ctxs[0], par))
-        for par in self.REQUIRES_RUPTURE_PARAMETERS:
-            vals = [getattr(ctx, par) for ctx in ctxs]
-            setattr(ctx, par, numpy.array(vals))
-        for par in self.REQUIRES_DISTANCES:
-            dists = [getattr(ctx, par)[0] for ctx in ctxs]
-            setattr(ctx, par, numpy.array(dists))
-        ctx.ctxs = ctxs
-        return ctx
+        sitecol = dstore['sitecol'].complete
+        if slc is None:
+            slc = dstore['rup/grp_id'][:] == self.grp_id
+        params = {n: dstore['rup/' + n][slc] for n in dstore['rup']}
+        ctxs = []
+        for u in range(len(params['mag'])):
+            ctx = RuptureContext()
+            for par, arr in params.items():
+                if par.endswith('_'):
+                    par = par[:-1]
+                setattr(ctx, par, arr[u])
+            for par in sitecol.array.dtype.names:
+                setattr(ctx, par, sitecol[par][ctx.sids])
+            ctxs.append(ctx)
+        return ctxs
 
-    def gen_ctx_poes(self, ctxs):
+    def recarray(self, ctxs):
         """
-        :param ctxs: a list of C context objects
-        :yields: C pairs (ctx, poes of shape (N, L, G))
+        :params ctxs: a list of contexts
+        :returns: a recarray
         """
-        nsites = numpy.array([len(ctx.sids) for ctx in ctxs])
-        C = len(ctxs)
-        N = nsites.sum()
-        poes = numpy.zeros((N, self.loglevels.size, len(self.gsims)))
-        if self.single_site_opt.any():
-            ctx = self.multi(ctxs)
-        for g, gsim in enumerate(self.gsims):
-            with self.gmf_mon:
-                # builds mean_std of shape (2, N, M)
-                if self.single_site_opt[g] and C > 1 and (nsites == 1).all():
-                    mean_std = gsim.get_mean_std1(ctx, self.imts)
-                else:
-                    mean_std = gsim.get_mean_std(ctxs, self.imts)
-            with self.poe_mon:
-                # builds poes of shape (N, L, G)
-                poes[:, :, g] = gsim.get_poes(
-                    mean_std, self.loglevels, self.trunclevel, self.af, ctxs)
-        s = 0
-        for ctx, n in zip(ctxs, nsites):
-            yield ctx, poes[s:s+n]
-            s += n
+        C = sum(len(ctx) for ctx in ctxs)
+        ra = self.ctx_builder.zeros(C).view(numpy.recarray)
+        start = 0
+        for ctx in ctxs:
+            slc = slice(start, start + len(ctx))
+            for par in self.ctx_builder.names:
+                getattr(ra, par)[slc] = getattr(ctx, par)
+            ra.sids[slc] = ctx.sids
+            start = slc.stop
+        return ra
 
     def get_ctx_params(self):
         """
@@ -282,17 +297,19 @@ class ContextMaker(object):
             params.add(dparam + '_')
         return params
 
-    def from_srcs(self, srcs, site1):  # used in disagg.disaggregation
+    def from_srcs(self, srcs, sitecol):  # used in disagg.disaggregation
         """
+        :param srcs: a list of Source objects
+        :param sitecol: a SiteCollection instance
         :returns: a list RuptureContexts
         """
         allctxs = []
         for i, src in enumerate(srcs):
             src.id = i
-            ctxs = []
+            rctxs = []
             for rup in src.iter_ruptures(shift_hypo=self.shift_hypo):
-                ctxs.append(self.make_rctx(rup))
-            allctxs.extend(self.gen_ctxs(ctxs, site1, src.id))
+                rctxs.append(self.make_rctx(rup))
+            allctxs.extend(self.get_ctxs(rctxs, sitecol, src.id))
         return allctxs
 
     def filter(self, sites, rup):
@@ -379,7 +396,7 @@ class ContextMaker(object):
                 dctx.rrup = numpy.sqrt(reqv**2 + rupture.hypocenter.depth**2)
         return self.make_rctx(rupture), sites, dctx
 
-    def gen_ctxs(self, ruptures, sites, src_id, mon=Monitor()):
+    def get_ctxs(self, ruptures, sites, src_id, mon=Monitor()):
         """
         :param ruptures:
             a list of ruptures generated by the same source
@@ -389,9 +406,10 @@ class ContextMaker(object):
             the ID of the source (for debugging purposes)
         :param mon:
             a Monitor object
-        :yields:
+        :returns:
             fat RuptureContexts
         """
+        ctxs = []
         fewsites = len(sites.complete) <= self.max_sites_disagg
         for rup in ruptures:
             with mon:
@@ -407,10 +425,12 @@ class ContextMaker(object):
                 for par in self.REQUIRES_DISTANCES | {'rrup'}:
                     setattr(ctx, par, getattr(dctx, par))
                 if fewsites:
+                    # get closest point on the surface
                     closest = rup.surface.get_closest_points(sites.complete)
                     ctx.clon = closest.lons[ctx.sids]
                     ctx.clat = closest.lats[ctx.sids]
-            yield ctx
+            ctxs.append(ctx)
+        return ctxs
 
     # this is used with pointsource_distance approximation for close distances,
     # when there are many ruptures affecting few sites
@@ -466,17 +486,124 @@ class ContextMaker(object):
             ctx.sids = sitecol1.sids
             ctx.mag = mag
             ctx.width = .01  # 10 meters to avoid warnings in abrahamson_2014
-            means = []
-            for gsim in self.gsims:
-                try:
-                    mean = gsim.get_mean_std([ctx], self.imts)[0, 0]
-                except ValueError:  # magnitude outside of supported range
-                    continue
-                means.append(mean.max())
-            if means:
-                gmv[m, d] = numpy.exp(max(means))
+            try:
+                maxmean = max(ms[0].max() for ms in self.get_mean_stds(
+                    [ctx], StdDev.TOTAL))
+                # shape NM
+            except ValueError:  # magnitude outside of supported range
+                continue
+            else:
+                gmv[m, d] = numpy.exp(maxmean)
         return gmv
 
+    def get_pmap(self, ctxs, probmap=None):
+        """
+        :param ctxs: a list of contexts
+        :param probmap: if not None, update it
+        :returns: a new ProbabilityMap if probmap is None
+        """
+        tom = self.tom
+        rup_indep = self.rup_indep
+        if probmap is None:  # create new pmap
+            pmap = ProbabilityMap(self.imtls.size, len(self.gsims))
+        else:  # update passed probmap
+            pmap = probmap
+        for ctx, poes in zip(ctxs, self.gen_poes(ctxs)):
+            # pnes and poes of shape (N, L, G)
+            with self.pne_mon:
+                pnes = get_probability_no_exceedance(ctx, poes, tom)
+                for sid, pne in zip(ctx.sids, pnes):
+                    probs = pmap.setdefault(sid, self.rup_indep).array
+                    if rup_indep:
+                        probs *= pne
+                    else:  # rup_mutex
+                        probs += (1. - pne) * ctx.weight
+        if probmap is None:  # return the new pmap
+            return ~pmap if rup_indep else pmap
+
+    # called by gen_poes and by the GmfComputer
+    def get_mean_stds(self, ctxs, stdtype=StdDev.ALL):
+        """
+        :param ctxs: a list of contexts
+        :param stdtype: a standard deviation type
+        :returns: a list of G arrays of shape (O, M, N) with mean and stddevs
+        """
+        if not hasattr(self, 'imts'):
+            self.imts = tuple(imt_module.from_string(im) for im in self.imtls)
+        ctxs = [ctx.roundup(self.minimum_distance) for ctx in ctxs]
+        N = sum(len(ctx.sids) for ctx in ctxs)
+        M = len(self.imtls)
+        out = []
+        if self.use_recarray:
+            ctxs = [self.recarray(ctxs)]
+        for g, gsim in enumerate(self.gsims):
+            if stdtype is None or self.trunclevel == 0:
+                stypes = ()
+            elif stdtype == StdDev.EVENT:
+                if gsim.DEFINED_FOR_STANDARD_DEVIATION_TYPES == {StdDev.TOTAL}:
+                    stypes = (StdDev.TOTAL,)
+                else:
+                    stypes = (StdDev.INTER_EVENT, StdDev.INTRA_EVENT)
+            elif stdtype == StdDev.ALL:
+                stypes = tuple(sdt for sdt in STD_TYPES if sdt in
+                               gsim.DEFINED_FOR_STANDARD_DEVIATION_TYPES)
+            else:
+                stypes = (stdtype,)
+            S = len(stypes)
+            arr = numpy.zeros((1 + S, M, N))
+            compute = gsim.__class__.__dict__.get('compute')
+            if compute:  # new api
+                outs = numpy.zeros((4, M, N))
+                start = 0
+                for ctx in ctxs:
+                    slc = slice(start, start + len(ctx))
+                    compute(gsim, ctx, self.imts, *outs[:, :, slc])
+                    start = slc.stop
+                arr[0] = outs[0]
+                for s, stype in enumerate(stypes, 1):
+                    if stype == StdDev.TOTAL:
+                        arr[s] = outs[1]
+                    elif stype == StdDev.INTER_EVENT:
+                        arr[s] = outs[2]
+                    elif stype == StdDev.INTRA_EVENT:
+                        arr[s] = outs[3]
+            else:  # legacy api
+                start = 0
+                for ctx in ctxs:
+                    stop = start + len(ctx.sids)
+                    for m, imt in enumerate(self.imts):
+                        mean, stds = gsim.get_mean_and_stddevs(
+                            ctx, ctx, ctx, imt, stypes)
+                        arr[0, m, start:stop] = mean
+                        for s in range(S):
+                            arr[1 + s, m, start:stop] = stds[s]
+                    start = stop
+            out.append(arr)
+        return out
+
+    def gen_poes(self, ctxs):
+        """
+        :param ctxs: a list of C context objects
+        :yields: poes of shape (N, L, G)
+        """
+        from openquake.hazardlib.site_amplification import get_poes_site
+        with self.gmf_mon:
+            mean_stdt = self.get_mean_stds(ctxs, StdDev.TOTAL)
+        s = 0
+        for ctx in ctxs:
+            with self.poe_mon:
+                n = len(ctx)
+                poes = numpy.zeros((n, self.loglevels.size, len(self.gsims)))
+                for g, gsim in enumerate(self.gsims):
+                    ms = mean_stdt[g][:, :, s:s+n]
+                    # builds poes of shape (n, L, G)
+                    if self.af:  # kernel amplification method
+                        poes[:, :, g] = get_poes_site(ms, self, ctx)
+                    else:  # regular case
+                        poes[:, :, g] = gsim.get_poes(ms, self, ctx)
+            yield poes
+            s += n
+
 
 # see contexts_tests.py for examples of collapse
 def combine_pmf(o1, o2):
@@ -486,7 +613,7 @@ def combine_pmf(o1, o2):
 
     :param o1: probability distribution of length n1
     :param o2: probability distribution of length n2
-    :returns: probability distribution of length n1 + n2
+    :returns: probability distribution of length n1 + n2 - 1
 
     >>> combine_pmf([.99, .01], [.98, .02])
     array([9.702e-01, 2.960e-02, 2.000e-04])
@@ -508,7 +635,7 @@ def _collapse(ctxs):
     for ctx in ctxs:
         if numpy.isnan(ctx.occurrence_rate):  # nonparametric
             nrups.append(ctx)
-        else:  # parametrix
+        else:  # parametric
             prups.append(ctx)
     if len(prups) > 1:
         ctx = copy.copy(prups[0])
@@ -549,12 +676,8 @@ class PmapMaker(object):
         self.N = len(self.srcfilter.sitecol.complete)
         self.group = group
         self.src_mutex = getattr(group, 'src_interdep', None) == 'mutex'
-        self.rup_indep = getattr(group, 'rup_interdep', None) != 'mutex'
+        self.cmaker.rup_indep = getattr(group, 'rup_interdep', None) != 'mutex'
         self.fewsites = self.N <= cmaker.max_sites_disagg
-        self.pne_mon = cmaker.mon('composing pnes', measuremem=False)
-        # NB: if maxsites is too big or too small the performance of
-        # get_poes can easily become 2-3 times worse!
-        self.maxsites = 512000 / len(self.gsims) / self.imtls.size
 
     def count_bytes(self, ctxs):
         # # usuful for debugging memory issues
@@ -569,80 +692,64 @@ class PmapMaker(object):
             nbytes += 8 * dparams * nsites
         return nbytes
 
-    def _update_pmap(self, ctxs, pmap=None):
-        # compute PoEs and update pmap
-        if pmap is None:  # for src_indep
-            pmap = self.pmap
-        rup_indep = self.rup_indep
-        # splitting in blocks makes sure that the maximum poes array
-        # generated has size N x L x G x 8 = 4 MB
-        for block in block_splitter(
-                ctxs, self.maxsites, lambda ctx: len(ctx.sids)):
-            for ctx, poes in self.cmaker.gen_ctx_poes(block):
-                with self.pne_mon:
-                    # pnes and poes of shape (N, L, G)
-                    pnes = ctx.get_probability_no_exceedance(poes)
-                    for sid, pne in zip(ctx.sids, pnes):
-                        probs = pmap.setdefault(sid, rup_indep).array
-                        if rup_indep:
-                            probs *= pne
-                        else:  # rup_mutex
-                            probs += (1. - pne) * ctx.weight
-
     def _ruptures(self, src, filtermag=None):
         return src.iter_ruptures(
             shift_hypo=self.shift_hypo, mag=filtermag)
 
-    def _gen_ctxs(self, rups, sites, srcid):
-        # yield context objects to save memory
-        ctxs = self.cmaker.gen_ctxs(rups, sites, srcid, self.ctx_mon)
+    def _get_ctxs(self, rups, sites, srcid):
+        ctxs = self.cmaker.get_ctxs(rups, sites, srcid, self.ctx_mon)
         if self.collapse_level > 1:
-            ctxs = self.cmaker.collapse_the_ctxs(list(ctxs))
+            ctxs = self.cmaker.collapse_the_ctxs(ctxs)
+        out = []
         for ctx in ctxs:
             self.numsites += len(ctx.sids)
             self.numctxs += 1
             if self.fewsites:  # keep the contexts in memory
                 self.rupdata.append(ctx)
-            yield ctx
+            out.append(ctx)
+        return out
 
     def _make_src_indep(self):
         # sources with the same ID
-        for src, sites in self.srcfilter.split(self.group):
+        pmap = ProbabilityMap(self.imtls.size, len(self.gsims))
+        # split the sources only if there is more than 1 site
+        filt = (self.srcfilter.split_less if self.N == 1
+                else self.srcfilter.split)
+        for src, sites in filt(self.group):
+            t0 = time.time()
             if self.fewsites:
                 sites = sites.complete
-            t0 = time.time()
             self.numctxs = 0
             self.numsites = 0
             rups = self._gen_rups(src, sites)
-            self._update_pmap(self._gen_ctxs(rups, sites, src.id))
+            self.cmaker.get_pmap(self._get_ctxs(rups, sites, src.id), pmap)
             dt = time.time() - t0
             self.calc_times[src.id] += numpy.array(
                 [self.numctxs, self.numsites, dt])
             timer.save(src, self.numctxs, self.numsites, dt,
                        self.cmaker.task_no)
-        return ~self.pmap if self.rup_indep else self.pmap
+        return ~pmap if self.cmaker.rup_indep else pmap
 
     def _make_src_mutex(self):
-        for src, indices in self.srcfilter.filter(self.group):
+        pmap = ProbabilityMap(self.imtls.size, len(self.gsims))
+        for src, sites in self.srcfilter.filter(self.group):
             t0 = time.time()
-            sites = self.srcfilter.sitecol.filtered(indices)
             self.numctxs = 0
             self.numsites = 0
             rups = self._ruptures(src)
-            L, G = self.cmaker.imtls.size, len(self.cmaker.gsims)
-            pmap = ProbabilityMap(L, G)
-            self._update_pmap(self._gen_ctxs(rups, sites, src.id), pmap)
-            p = pmap
-            if self.rup_indep:
+            pm = ProbabilityMap(self.cmaker.imtls.size, len(self.cmaker.gsims))
+            self.cmaker.get_pmap(self._get_ctxs(rups, sites, src.id), pm)
+            p = pm
+            if self.cmaker.rup_indep:
                 p = ~p
             p *= src.mutex_weight
-            self.pmap += p
+            pmap += p
             dt = time.time() - t0
             self.calc_times[src.id] += numpy.array(
                 [self.numctxs, self.numsites, dt])
             timer.save(src, self.numctxs, self.numsites, dt,
                        self.cmaker.task_no)
-        return self.pmap
+        return pmap
 
     def dictarray(self, ctxs):
         dic = {}  # par -> array
@@ -654,17 +761,20 @@ class PmapMaker(object):
 
     def make(self):
         self.rupdata = []
-        imtls = self.cmaker.imtls
-        L, G = imtls.size, len(self.gsims)
-        self.pmap = ProbabilityMap(L, G)
         # AccumDict of arrays with 3 elements nrups, nsites, calc_time
         self.calc_times = AccumDict(accum=numpy.zeros(3, numpy.float32))
         if self.src_mutex:
             pmap = self._make_src_mutex()
         else:
             pmap = self._make_src_indep()
-        rupdata = self.dictarray(self.rupdata)
-        return pmap, rupdata, self.calc_times
+        dic = {'pmap': pmap,
+               'rup_data': self.dictarray(self.rupdata),
+               'calc_times': self.calc_times,
+               'task_no': self.task_no,
+               'grp_id': self.group[0].grp_id}
+        if self.disagg_by_src:
+            dic['source_id'] = self.group[0].source_id
+        return dic
 
     def _gen_rups(self, src, sites):
         # yield ruptures, each one with a .sites attribute
@@ -754,6 +864,10 @@ class SitesContext(BaseContext):
             for slot in slots:
                 setattr(self, slot, getattr(sitecol, slot))
 
+    # used in the SMTK
+    def __len__(self):
+        return len(self.sids)
+
 
 class DistancesContext(BaseContext):
     """
@@ -804,6 +918,40 @@ def get_dists(ctx):
             if par in KNOWN_DISTANCES}
 
 
+def full_context(sites, rup, dctx=None):
+    """
+    :returns: a full RuptureContext with all the relevant attributes
+    """
+    self = RuptureContext()
+    for par, val in vars(rup).items():
+        setattr(self, par, val)
+    if hasattr(sites, 'array'):  # is a SiteCollection
+        for par in sites.array.dtype.names:
+            setattr(self, par, sites[par])
+    else:  # sites is a SitesContext
+        for par, val in vars(sites).items():
+            setattr(self, par, val)
+    if dctx:
+        for par, val in vars(dctx).items():
+            setattr(self, par, val)
+    return self
+
+
+def get_mean_stds(gsims, ctx, imts, stdtype=StdDev.ALL):
+    """
+    :param gsims: a list of G GSIMs
+    :param ctx: a RuptureContext or a recarray of size N
+    :param imts: a list of M IMTs
+    :param stdtype: a standard deviation type (TOTAL, EVENT, etc)
+    :returns:
+        an array of shape (G, O, M, N) obtained by applying the
+        given GSIMs, ctx amd imts
+    """
+    imtls = {imt.string: [0] for imt in imts}
+    cmaker = ContextMaker('*', gsims, {'imtls': imtls})
+    return numpy.array(cmaker.get_mean_stds([ctx], stdtype))
+
+
 # mock of a rupture used in the tests and in the SMTK
 class RuptureContext(BaseContext):
     """
@@ -820,22 +968,6 @@ class RuptureContext(BaseContext):
     _slots_ = (
         'mag', 'strike', 'dip', 'rake', 'ztor', 'hypo_lon', 'hypo_lat',
         'hypo_depth', 'width', 'hypo_loc')
-    temporal_occurrence_model = None  # to be set
-
-    @classmethod
-    def full(cls, rup, sites, dctx=None):
-        """
-        :returns: a full context with all the relevant attributes
-        """
-        self = cls()
-        for par, val in vars(rup).items():
-            setattr(self, par, val)
-        for par in sites.array.dtype.names:
-            setattr(self, par, sites[par])
-        if dctx:
-            for par, val in vars(dctx).items():
-                setattr(self, par, val)
-        return self
 
     def __init__(self, param_pairs=()):
         for param, value in param_pairs:
@@ -847,9 +979,14 @@ class RuptureContext(BaseContext):
         of magnitudes and it refers to a single site, returns the size of
         the array, otherwise returns 1.
         """
-        if isinstance(self.mag, numpy.ndarray) and len(self.vs30) == 1:
+        nsites = len(self.sids)
+        if nsites == 1 and isinstance(self.mag, numpy.ndarray):
             return len(self.mag)
-        return 1
+        return nsites
+
+    # used in acme_2019
+    def __len__(self):
+        return len(self.sids)
 
     def roundup(self, minimum_distance):
         """
@@ -871,47 +1008,53 @@ class RuptureContext(BaseContext):
                 setattr(ctx, dist, array)
         return ctx
 
-    def get_probability_no_exceedance(self, poes):
-        """
-        Compute and return the probability that in the time span for which the
-        rupture is defined, the rupture itself never generates a ground motion
-        value higher than a given level at a given site.
 
-        Such calculation is performed starting from the conditional probability
-        that an occurrence of the current rupture is producing a ground motion
-        value higher than the level of interest at the site of interest.
-        The actual formula used for such calculation depends on the temporal
-        occurrence model the rupture is associated with.
-        The calculation can be performed for multiple intensity measure levels
-        and multiple sites in a vectorized fashion.
+def get_probability_no_exceedance(rup, poes, tom):
+    """
+    Compute and return the probability that in the time span for which the
+    rupture is defined, the rupture itself never generates a ground motion
+    value higher than a given level at a given site.
 
-        :param poes:
-            2D numpy array containing conditional probabilities the the a
-            rupture occurrence causes a ground shaking value exceeding a
-            ground motion level at a site. First dimension represent sites,
-            second dimension intensity measure levels. ``poes`` can be obtained
-            calling the :func:`func <openquake.hazardlib.gsim.base.get_poes>`
-        """
-        if numpy.isnan(self.occurrence_rate):  # nonparametric rupture
-            # Uses the formula
-            #
-            #    ∑ p(k|T) * p(X<x|rup)^k
-            #
-            # where `p(k|T)` is the probability that the rupture occurs k times
-            # in the time span `T`, `p(X<x|rup)` is the probability that a
-            # rupture occurrence does not cause a ground motion exceedance, and
-            # thesummation `∑` is done over the number of occurrences `k`.
-            #
-            # `p(k|T)` is given by the attribute probs_occur and
-            # `p(X<x|rup)` is computed as ``1 - poes``.
-            prob_no_exceed = numpy.float64(
-                [v * (1 - poes) ** i for i, v in enumerate(self.probs_occur)]
-            ).sum(axis=0)
-            return numpy.clip(prob_no_exceed, 0., 1.)  # avoid numeric issues
+    Such calculation is performed starting from the conditional probability
+    that an occurrence of the current rupture is producing a ground motion
+    value higher than the level of interest at the site of interest.
+    The actual formula used for such calculation depends on the temporal
+    occurrence model the rupture is associated with.
+    The calculation can be performed for multiple intensity measure levels
+    and multiple sites in a vectorized fashion.
 
-        # parametric rupture
-        tom = self.temporal_occurrence_model
-        return tom.get_probability_no_exceedance(self.occurrence_rate, poes)
+    :param rup:
+        an object with attributes .occurrence_rate and possibly .probs_occur
+    :param poes:
+        2D numpy array containing conditional probabilities the the a
+        rupture occurrence causes a ground shaking value exceeding a
+        ground motion level at a site. First dimension represent sites,
+        second dimension intensity measure levels. ``poes`` can be obtained
+        calling the :func:`func <openquake.hazardlib.gsim.base.get_poes>`
+
+    :param tom:
+        temporal occurrence model instance, used only if the rupture
+        is parametric
+    """
+    if numpy.isnan(rup.occurrence_rate):  # nonparametric rupture
+        # Uses the formula
+        #
+        #    ∑ p(k|T) * p(X<x|rup)^k
+        #
+        # where `p(k|T)` is the probability that the rupture occurs k times
+        # in the time span `T`, `p(X<x|rup)` is the probability that a
+        # rupture occurrence does not cause a ground motion exceedance, and
+        # thesummation `∑` is done over the number of occurrences `k`.
+        #
+        # `p(k|T)` is given by the attribute probs_occur and
+        # `p(X<x|rup)` is computed as ``1 - poes``.
+        prob_no_exceed = numpy.float64(
+            [v * (1 - poes) ** i for i, v in enumerate(rup.probs_occur)]
+        ).sum(axis=0)
+        return numpy.clip(prob_no_exceed, 0., 1.)  # avoid numeric issues
+
+    # parametric rupture
+    return tom.get_probability_no_exceedance(rup.occurrence_rate, poes)
 
 
 class Effect(object):
@@ -983,7 +1126,7 @@ def get_effect_by_mag(mags, sitecol1, gsims_by_trt, maximum_distance, imtls):
     return dict(zip(mags, gmv))
 
 
-# used in calculators/classical.py
+# not used at the moment
 def get_effect(mags, sitecol1, gsims_by_trt, oq):
     """
     :params mags:
@@ -1061,3 +1204,93 @@ def ruptures_by_mag_dist(sources, srcfilter, gsims, params, monitor):
                 di = nbins - 1
             dic['%.2f' % rup.mag][di] += 1
     return {trt: AccumDict(dic)}
+
+
+def read_cmakers(dstore, full_lt=None):
+    """
+    :param dstore: a DataStore-like object
+    :param full_lt: a FullLogicTree instance, if given
+    :returns: a list of ContextMaker instance, one per source group
+    """
+    from openquake.hazardlib.site_amplification import AmplFunction
+    cmakers = []
+    oq = dstore['oqparam']
+    full_lt = full_lt or dstore['full_lt']
+    trt_smrs = dstore['trt_smrs'][:]
+    toms = dstore['toms'][:]
+    rlzs_by_gsim_list = full_lt.get_rlzs_by_gsim_list(trt_smrs)
+    trts = list(full_lt.gsim_lt.values)
+    num_eff_rlzs = len(full_lt.sm_rlzs)
+    start = 0
+    # some ugly magic on the pointsource_distance
+    if oq.pointsource_distance:
+        mags = dstore['source_mags']
+        psd = MagDepDistance.new(str(oq.pointsource_distance))
+        psd.interp({trt: mags[trt][:] for trt in mags})
+        oq.pointsource_distance = psd
+    for grp_id, rlzs_by_gsim in enumerate(rlzs_by_gsim_list):
+        trti = trt_smrs[grp_id][0] // num_eff_rlzs
+        trt = trts[trti]
+        if ('amplification' in oq.inputs and
+                oq.amplification_method == 'kernel'):
+            df = AmplFunction.read_df(oq.inputs['amplification'])
+            af = AmplFunction.from_dframe(df)
+        else:
+            af = None
+        cmaker = ContextMaker(
+            trt, rlzs_by_gsim,
+            {'truncation_level': oq.truncation_level,
+             'collapse_level': int(oq.collapse_level),
+             'num_epsilon_bins': oq.num_epsilon_bins,
+             'investigation_time': oq.investigation_time,
+             'maximum_distance': oq.maximum_distance,
+             'pointsource_distance': oq.pointsource_distance,
+             'minimum_distance': oq.minimum_distance,
+             'ses_seed': oq.ses_seed,
+             'ses_per_logic_tree_path': oq.ses_per_logic_tree_path,
+             'max_sites_disagg': oq.max_sites_disagg,
+             'disagg_by_src': oq.disagg_by_src,
+             'min_iml': oq.min_iml,
+             'imtls': oq.imtls,
+             'reqv': oq.get_reqv(),
+             'shift_hypo': oq.shift_hypo,
+             'af': af,
+             'grp_id': grp_id})
+        cmaker.tom = registry[toms[grp_id]](oq.investigation_time)
+        cmaker.trti = trti
+        cmaker.start = start
+        start += len(rlzs_by_gsim)
+        cmakers.append(cmaker)
+    return cmakers
+
+
+def read_cmaker(dstore, trt_smr):
+    """
+    :param dstore: a DataStore-like object
+    :returns: a ContextMaker instance
+    """
+    oq = dstore['oqparam']
+    full_lt = dstore['full_lt']
+    trts = list(full_lt.gsim_lt.values)
+    trt = trts[trt_smr // len(full_lt.sm_rlzs)]
+    rlzs_by_gsim = full_lt.get_rlzs_by_gsim()[trt_smr]
+    mags = dstore['source_mags']
+    md = MagDepDistance.new(str(oq.maximum_distance))
+    md.interp({trt: mags[trt][:] for trt in mags})
+    cmaker = ContextMaker(
+        trt, rlzs_by_gsim,
+        {'truncation_level': oq.truncation_level,
+         'collapse_level': int(oq.collapse_level),
+         'num_epsilon_bins': oq.num_epsilon_bins,
+         'investigation_time': oq.investigation_time,
+         'maximum_distance': md,
+         'minimum_distance': oq.minimum_distance,
+         'ses_seed': oq.ses_seed,
+         'ses_per_logic_tree_path': oq.ses_per_logic_tree_path,
+         'max_sites_disagg': oq.max_sites_disagg,
+         'disagg_by_src': oq.disagg_by_src,
+         'min_iml': oq.min_iml,
+         'imtls': oq.imtls,
+         'reqv': oq.get_reqv(),
+         'shift_hypo': oq.shift_hypo})
+    return cmaker
