@@ -35,15 +35,6 @@ def _get_finite_mesh(mesh):
     return Mesh(mesh.lons[ok], mesh.lats[ok], mesh.depths[ok])
 
 
-def _get_finite_top_rupture(mesh):
-    ok = numpy.isfinite(mesh.lons.flat)
-    if numpy.all(ok):
-        return mesh[0:1]
-    ok = numpy.isfinite(mesh.lons[0, :])
-    return RectangularMesh(mesh.lons[0, ok], mesh.lats[0, ok],
-                           mesh.depths[0, ok])
-
-
 def _find_turning_points(mesh, tol=1.0):
     """
     Identifies the turning points in a rectangular mesh based on the
@@ -183,16 +174,24 @@ class BaseSurface:
         top_edge = self.mesh[0:1]
         mean_strike = self.get_strike()
 
-        # Computing the distances between the sites and the two lines 
-        # perpendicular to the strike passing trough the two extremes 
+        # Manage the case of kite fault surfaces
+        ia = 0
+        ib = -1
+        if (self.__class__.__name__ == 'KiteSurface'):
+            idx = numpy.nonzero(numpy.isfinite(top_edge.lons[0, :]))[0]
+            ia = min(idx)
+            ib = max(idx)
+
+        # Computing the distances between the sites and the two lines
+        # perpendicular to the strike passing trough the two extremes
         # of the top of the rupture
-        dst1 = geodetic.distance_to_arc(top_edge.lons[0, 0],
-                                        top_edge.lats[0, 0],
+        dst1 = geodetic.distance_to_arc(top_edge.lons[0, ia],
+                                        top_edge.lats[0, ia],
                                         (mean_strike + 90.) % 360,
                                         mesh.lons, mesh.lats)
 
-        dst2 = geodetic.distance_to_arc(top_edge.lons[0, -1],
-                                        top_edge.lats[0, -1],
+        dst2 = geodetic.distance_to_arc(top_edge.lons[0, ib],
+                                        top_edge.lats[0, ib],
                                         (mean_strike + 90.) % 360,
                                         mesh.lons, mesh.lats)
 
@@ -227,42 +226,55 @@ class BaseSurface:
             Numpy array of distances in km.
         """
         top_edge = self.mesh[0:1]
-        xxx = _get_finite_top_rupture(mesh)
-
         dists = []
-        if top_edge.lons.shape[1] < 3:
 
+        ia = 0
+        ib = top_edge.lons.shape[1] - 2
+        if (self.__class__.__name__ == 'KiteSurface'):
+            idxs = numpy.nonzero(numpy.isfinite(top_edge.lons[0, :]))[0]
+            ia = min(idxs)
+            ib = sorted(idxs)[-2]
+
+        if top_edge.lons.shape[1] < 3:
             i = 0
+
+            if ((self.__class__.__name__ == 'KiteSurface') and
+                (numpy.isnan(top_edge.lons[0, i]) or
+                 numpy.isnan(top_edge.lons[0, i+1]))):
+                msg = 'Rx calculation. Top of rupture has less than two points'
+                raise ValueError(msg)
+
             p1 = Point(
                 top_edge.lons[0, i],
                 top_edge.lats[0, i],
-                top_edge.depths[0, i]
-            )
+                top_edge.depths[0, i])
             p2 = Point(
                 top_edge.lons[0, i + 1], top_edge.lats[0, i + 1],
-                top_edge.depths[0, i + 1]
-            )
+                top_edge.depths[0, i + 1])
             azimuth = p1.azimuth(p2)
             dists.append(
                 geodetic.distance_to_arc(
                     p1.longitude, p1.latitude, azimuth,
-                    mesh.lons, mesh.lats
-                )
-            )
+                    mesh.lons, mesh.lats))
 
         else:
 
             for i in range(top_edge.lons.shape[1] - 1):
+
+                if ((self.__class__.__name__ == 'KiteSurface') and
+                    (numpy.isnan(top_edge.lons[0, i]) or
+                     numpy.isnan(top_edge.lons[0, i+1]))):
+                    continue
+
                 p1 = Point(
                     top_edge.lons[0, i],
                     top_edge.lats[0, i],
-                    top_edge.depths[0, i]
-                )
+                    top_edge.depths[0, i])
                 p2 = Point(
                     top_edge.lons[0, i + 1],
                     top_edge.lats[0, i + 1],
-                    top_edge.depths[0, i + 1]
-                )
+                    top_edge.depths[0, i + 1])
+
                 # Swapping
                 if i == 0:
                     pt = p1
@@ -270,7 +282,7 @@ class BaseSurface:
                     p2 = pt
 
                 # Computing azimuth and distance
-                if i == 0 or i == top_edge.lons.shape[1] - 2:
+                if i == ia or i == ib:
                     azimuth = p1.azimuth(p2)
                     tmp = geodetic.distance_to_semi_arc(p1.longitude,
                                                         p1.latitude,
