@@ -32,6 +32,211 @@ from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
 from openquake.hazardlib import const
 from openquake.hazardlib.imt import PGA, SA
 
+# Period-Independent Coefficients (Table 2)
+CONSTS = {
+    'n': 1.18,
+    'c': 1.88,
+    'theta3': 0.1,
+    'theta4': 0.9,
+    'theta5': 0.0,
+    'theta9': 0.4,
+    'c4': 10.0,
+    'C1': 7.8
+}
+
+C1 = 7.2  # for Montalva2017
+
+# Total epistemic uncertainty factors from Abrahamson et al. (2018)
+BCHYDRO_SIGMA_MU = CoeffsTable(sa_damping=5, table="""
+    imt     SIGMA_MU_SINTER    SIGMA_MU_SSLAB
+    pga                 0.3              0.50
+    0.010               0.3              0.50
+    0.020               0.3              0.50
+    0.030               0.3              0.50
+    0.050               0.3              0.50
+    0.075               0.3              0.50
+    0.100               0.3              0.50
+    0.150               0.3              0.50
+    0.200               0.3              0.50
+    0.250               0.3              0.46
+    0.300               0.3              0.42
+    0.400               0.3              0.38
+    0.500               0.3              0.34
+    0.600               0.3              0.30
+    0.750               0.3              0.30
+    1.000               0.3              0.30
+    1.500               0.3              0.30
+    2.000               0.3              0.30
+    2.500               0.3              0.30
+    3.000               0.3              0.30
+    4.000               0.3              0.30
+    5.000               0.3              0.30
+    6.000               0.3              0.30
+    7.500               0.3              0.30
+    10.00               0.3              0.30
+    """)
+
+
+def get_stress_factor(imt, slab):
+    """
+    Returns the stress adjustment factor for the BC Hydro GMPE according to
+    Abrahamson et al. (2018)
+    """
+    if slab:
+        sigma_mu = BCHYDRO_SIGMA_MU[imt]["SIGMA_MU_SSLAB"]
+    else:
+        sigma_mu = BCHYDRO_SIGMA_MU[imt]["SIGMA_MU_SINTER"]
+    return sigma_mu / 1.65
+
+
+def _compute_magterm(C1, theta1, theta4, theta5, theta13, dc1, mag):
+    """
+    Computes the magnitude scaling term given by equation (2)
+    corrected by a local adjustment factor
+    """
+    base = theta1 + theta4 * dc1
+    dmag = C1 + dc1
+    f_mag = np.where(mag > dmag,
+                     theta5 * (mag - dmag) + theta13 * (10. - mag) ** 2.,
+                     theta4 * (mag - dmag) + theta13 * (10. - mag) ** 2.)
+    return base + f_mag
+
+
+# theta6_adj used in BCHydro
+def _compute_disterm(trt, C1, theta2, theta14, theta3, ctx, c4, theta9,
+                     theta6_adj, theta6, theta10):
+    if trt == const.TRT.SUBDUCTION_INTERFACE:
+        dists = ctx.rrup
+        assert theta10 == 0., theta10
+    elif trt == const.TRT.SUBDUCTION_INTRASLAB:
+        dists = ctx.rhypo
+    else:
+        raise NotImplementedError(trt)
+    return ((theta2 + theta14 + theta3 * (ctx.mag - C1)) * np.log(
+        dists + c4 * np.exp((ctx.mag - 6.) * theta9)) +
+            (theta6_adj + theta6) * dists) + theta10
+
+
+def _compute_forearc_backarc_term(trt, faba_model, C, ctx):
+    if trt == const.TRT.SUBDUCTION_INTERFACE:
+        dists = ctx.rrup
+        a, b = C['theta15'], C['theta16']
+        min_dist = 100.
+    elif trt == const.TRT.SUBDUCTION_INTRASLAB:
+        dists = ctx.rhypo
+        a, b = C['theta7'], C['theta8']
+        min_dist = 85.
+    else:
+        raise NotImplementedError(trt)
+    if faba_model is None:
+        f_faba = np.zeros_like(dists)
+        # Term only applies to backarc ctx (F_FABA = 0. for forearc)
+        fixed_dists = dists[ctx.backarc]
+        fixed_dists[fixed_dists < min_dist] = min_dist
+        f_faba[ctx.backarc] = a + b * np.log(fixed_dists / 40.)
+        return f_faba
+
+    # in BCHydro subclasses
+    fixed_dists = np.copy(dists)
+    fixed_dists[fixed_dists < min_dist] = min_dist
+    f_faba = a + b * np.log(fixed_dists / 40.)
+    return f_faba * faba_model(-ctx.xvf)
+
+
+def _compute_distance_term(kind, trt, theta6_adj, C, ctx):
+    """
+    Computes the distance scaling term, as contained within equation (1)
+    """
+    if kind.startswith("montalva"):
+        theta3 = C['theta3']
+    else:
+        theta3 = CONSTS['theta3']
+    if kind == "montalva17":
+        C1 = 7.2
+    else:
+        C1 = 7.8
+    if trt == const.TRT.SUBDUCTION_INTERFACE:
+        return _compute_disterm(
+            trt, C1, C['theta2'], 0., theta3, ctx, CONSTS['c4'],
+            CONSTS['theta9'], theta6_adj, C['theta6'], theta10=0.)
+    else:  # sslab
+        return _compute_disterm(
+            trt, C1, C['theta2'], C['theta14'], theta3, ctx,
+            CONSTS['c4'], CONSTS['theta9'], theta6_adj, C['theta6'],
+            C["theta10"])
+
+
+def _compute_focal_depth_term(trt, C, ctx):
+    """
+    Computes the hypocentral depth scaling term - as indicated by
+    equation (3)
+    For interface events F_EVENT = 0.. so no depth scaling is returned.
+    For SSlab events computes the hypocentral depth scaling term as
+    indicated by equation (3)
+    """
+    if trt == const.TRT.SUBDUCTION_INTERFACE:
+        return 0.
+    if ctx.hypo_depth > 120.0:
+        z_h = 120.0
+    else:
+        z_h = ctx.hypo_depth
+    return C['theta11'] * (z_h - 60.)
+
+
+def _compute_magnitude_term(kind, C, dc1, mag):
+    """
+    Computes the magnitude scaling term given by equation (2)
+    """
+    if kind == "base":
+        return _compute_magterm(
+            CONSTS['C1'], C['theta1'], CONSTS['theta4'],
+            CONSTS['theta5'], C['theta13'], dc1, mag)
+    elif kind == "montalva16":
+        return _compute_magterm(
+            CONSTS['C1'], C['theta1'], C['theta4'],
+            C['theta5'], C['theta13'], dc1, mag)
+    elif kind == "montalva17":
+        return _compute_magterm(C1, C['theta1'], C['theta4'],
+                                C['theta5'], 0., dc1, mag)
+
+
+def _compute_pga_rock(kind, trt, theta6_adj, faba_model, C, dc1, ctx):
+    """
+    Compute and return mean imt value for rock conditions
+    (vs30 = 1000 m/s)
+    """
+    mean = (_compute_magnitude_term(kind, C, dc1, ctx.mag) +
+            _compute_distance_term(kind, trt, theta6_adj, C, ctx) +
+            _compute_focal_depth_term(trt, C, ctx) +
+            _compute_forearc_backarc_term(trt, faba_model, C, ctx))
+    # Apply linear site term
+    site_response = ((C['theta12'] + C['b'] * CONSTS['n']) *
+                     np.log(1000. / C['vlin']))
+    return mean + site_response
+
+
+def _compute_site_response_term(C, ctx, pga1000):
+    """
+    Compute and return site response model term
+    This GMPE adopts the same site response scaling model of
+    Walling et al (2008) as implemented in the Abrahamson & Silva (2008)
+    GMPE. The functional form is retained here.
+    """
+    vs_star = ctx.vs30.copy()
+    vs_star[vs_star > 1000.0] = 1000.
+    arg = vs_star / C["vlin"]
+    site_resp_term = C["theta12"] * np.log(arg)
+    # Get linear scaling term
+    idx = ctx.vs30 >= C["vlin"]
+    site_resp_term[idx] += (C["b"] * CONSTS["n"] * np.log(arg[idx]))
+    # Get nonlinear scaling term
+    idx = np.logical_not(idx)
+    site_resp_term[idx] += (
+        -C["b"] * np.log(pga1000[idx] + CONSTS["c"]) +
+        C["b"] * np.log(pga1000[idx] + CONSTS["c"] *
+                        (arg[idx] ** CONSTS["n"])))
+    return site_resp_term
+
 
 class AbrahamsonEtAl2015SInter(GMPE):
     """
@@ -48,27 +253,20 @@ class AbrahamsonEtAl2015SInter(GMPE):
     model. The current class implements the 'central' model, whilst additional
     classes will implement the 'upper' and 'lower' alternatives.
     """
-
     #: Supported tectonic region type is subduction interface
-    DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTERFACE
+    DEFINED_FOR_TECTONIC_REGION_TYPE = trt = const.TRT.SUBDUCTION_INTERFACE
 
     #: Supported intensity measure types are spectral acceleration,
     #: and peak ground acceleration
-    DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([
-        PGA,
-        SA
-    ])
+    DEFINED_FOR_INTENSITY_MEASURE_TYPES = {PGA, SA}
 
     #: Supported intensity measure component is the geometric mean component
     DEFINED_FOR_INTENSITY_MEASURE_COMPONENT = const.IMC.AVERAGE_HORIZONTAL
 
     #: Supported standard deviation types are inter-event, intra-event
     #: and total, see table 3, pages 12 - 13
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
-        const.StdDev.TOTAL,
-        const.StdDev.INTER_EVENT,
-        const.StdDev.INTRA_EVENT
-    ])
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {
+        const.StdDev.TOTAL, const.StdDev.INTER_EVENT, const.StdDev.INTRA_EVENT}
 
     #: Site amplification is dependent upon Vs30
     #: For the Abrahamson et al (2013) GMPE a new term is introduced to
@@ -89,141 +287,58 @@ class AbrahamsonEtAl2015SInter(GMPE):
     #: Reference soil conditions (bottom of page 29)
     DEFINED_FOR_REFERENCE_VELOCITY = 1000
 
+    delta_c1 = None
+    kind = "base"
+    FABA_ALL_MODELS = {}  # overridden in BCHydro
+
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self.ergodic = kwargs.get('ergodic', True)
+        self.theta6_adj = kwargs.get("theta6_adjustment", 0.0)
+        self.sigma_mu_epsilon = kwargs.get("sigma_mu_epsilon", 0.0)
+        faba_type = kwargs.get("faba_taper_model", "Step")
+        if 'xvf' in self.REQUIRES_SITES_PARAMETERS:  # BCHydro subclasses
+            self.faba_model = self.FABA_ALL_MODELS[faba_type](**kwargs)
+        else:
+            self.faba_model = None
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+        <.base.GroundShakingIntensityModel.compute>`
         for spec of input and result values.
         """
-        # extract dictionaries of coefficients specific to required
-        # intensity measure type and for PGA
-        C = self.COEFFS[imt]
-        dc1 = self._get_delta_c1(imt)
         C_PGA = self.COEFFS[PGA()]
-        dc1_pga = self._get_delta_c1(PGA())
+        dc1_pga = self.delta_c1 or self.COEFFS_MAG_SCALE[PGA()]["dc1"]
         # compute median pga on rock (vs30=1000), needed for site response
         # term calculation
-        pga1000 = np.exp(
-            self._compute_pga_rock(C_PGA, dc1_pga, sites, rup, dists))
-        mean = (self._compute_magnitude_term(C, dc1, rup.mag) +
-                self._compute_distance_term(C, rup.mag, dists) +
-                self._compute_focal_depth_term(C, rup) +
-                self._compute_forearc_backarc_term(C, sites, dists) +
-                self._compute_site_response_term(C, sites, pga1000))
-        stddevs = self._get_stddevs(C, stddev_types, len(sites.vs30))
-        return mean, stddevs
+        pga1000 = np.exp(_compute_pga_rock(
+            self.kind, self.trt, self.theta6_adj, self.faba_model,
+            C_PGA, dc1_pga, ctx))
+        for m, imt in enumerate(imts):
+            C = self.COEFFS[imt]
+            dc1 = self.delta_c1 or self.COEFFS_MAG_SCALE[imt]["dc1"]
+            mean[m] = (
+                _compute_magnitude_term(
+                    self.kind, C, dc1, ctx.mag) +
+                _compute_distance_term(
+                    self.kind, self.trt, self.theta6_adj, C, ctx) +
+                _compute_focal_depth_term(
+                    self.trt, C, ctx) +
+                _compute_forearc_backarc_term(
+                    self.trt, self.faba_model, C, ctx) +
+                _compute_site_response_term(
+                    C, ctx, pga1000))
+            if self.sigma_mu_epsilon:
+                sigma_mu = get_stress_factor(
+                    imt, self.DEFINED_FOR_TECTONIC_REGION_TYPE ==
+                    const.TRT.SUBDUCTION_INTRASLAB)
+                mean[m] += sigma_mu * self.sigma_mu_epsilon
 
-    def _get_delta_c1(self, imt):
-        """
-        Returns the magnitude scaling parameter deltaC1 for capturing scaling
-        for large events.
-        """
-        return self.COEFFS_MAG_SCALE[imt]["dc1"]
-
-    def _compute_pga_rock(self, C, dc1, sites, rup, dists):
-        """
-        Compute and return mean imt value for rock conditions
-        (vs30 = 1000 m/s)
-        """
-        mean = (self._compute_magnitude_term(C, dc1, rup.mag) +
-                self._compute_distance_term(C, rup.mag, dists) +
-                self._compute_focal_depth_term(C, rup) +
-                self._compute_forearc_backarc_term(C, sites, dists))
-        # Apply linear site term
-        site_response = ((C['theta12'] + C['b'] * self.CONSTS['n']) *
-                         np.log(1000. / C['vlin']))
-        return mean + site_response
-
-    def _compute_magnitude_term(self, C, dc1, mag):
-        """
-        Computes the magnitude scaling term given by equation (2)
-        """
-        base = C['theta1'] + (self.CONSTS['theta4'] * dc1)
-        dmag = self.CONSTS["C1"] + dc1
-        if mag > dmag:
-            f_mag = (self.CONSTS['theta5'] * (mag - dmag)) +\
-                C['theta13'] * ((10. - mag) ** 2.)
-
-        else:
-            f_mag = (self.CONSTS['theta4'] * (mag - dmag)) +\
-                C['theta13'] * ((10. - mag) ** 2.)
-
-        return base + f_mag
-
-    def _compute_distance_term(self, C, mag, dists):
-        """
-        Computes the distance scaling term, as contained within equation (1)
-        """
-        return (C['theta2'] + self.CONSTS['theta3'] * (mag - 7.8)) *\
-            np.log(dists.rrup + self.CONSTS['c4'] * np.exp((mag - 6.) *
-                   self.CONSTS['theta9'])) + (C['theta6'] * dists.rrup)
-
-    def _compute_focal_depth_term(self, C, rup):
-        """
-        Computes the hypocentral depth scaling term - as indicated by
-        equation (3)
-        For interface events F_EVENT = 0.. so no depth scaling is returned
-        """
-        return 0.
-
-    def _compute_forearc_backarc_term(self, C, sites, dists):
-        """
-        Computes the forearc/backarc scaling term given by equation (4)
-        """
-        f_faba = np.zeros_like(dists.rrup)
-        # Term only applies to backarc sites (F_FABA = 0. for forearc)
-        max_dist = dists.rrup[sites.backarc]
-        max_dist[max_dist < 100.0] = 100.0
-        f_faba[sites.backarc] = C['theta15'] + \
-            (C['theta16'] * np.log(max_dist / 40.0))
-        return f_faba
-
-    def _compute_site_response_term(self, C, sites, pga1000):
-        """
-        Compute and return site response model term
-        This GMPE adopts the same site response scaling model of
-        Walling et al (2008) as implemented in the Abrahamson & Silva (2008)
-        GMPE. The functional form is retained here.
-        """
-        vs_star = sites.vs30.copy()
-        vs_star[vs_star > 1000.0] = 1000.
-        arg = vs_star / C["vlin"]
-        site_resp_term = C["theta12"] * np.log(arg)
-        # Get linear scaling term
-        idx = sites.vs30 >= C["vlin"]
-        site_resp_term[idx] += (C["b"] * self.CONSTS["n"] * np.log(arg[idx]))
-        # Get nonlinear scaling term
-        idx = np.logical_not(idx)
-        site_resp_term[idx] += (
-            -C["b"] * np.log(pga1000[idx] + self.CONSTS["c"]) +
-            C["b"] * np.log(pga1000[idx] + self.CONSTS["c"] *
-                            (arg[idx] ** self.CONSTS["n"])))
-        return site_resp_term
-
-    def _get_stddevs(self, C, stddev_types, num_sites):
-        """
-        Return standard deviations as defined in Table 3
-        """
-        stddevs = []
-        for stddev_type in stddev_types:
-            assert stddev_type in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev_type == const.StdDev.TOTAL:
-                sigma = C["sigma"] if self.ergodic else C["sigma_ss"]
-                stddevs.append(sigma + np.zeros(num_sites))
-            elif stddev_type == const.StdDev.INTER_EVENT:
-                stddevs.append(C['tau'] + np.zeros(num_sites))
-            elif stddev_type == const.StdDev.INTRA_EVENT:
-                if self.ergodic:
-                    phi = C["phi"]
-                else:
-                    # Get single station phi
-                    phi = np.sqrt(C["sigma_ss"] ** 2. - C["tau"] ** 2.)
-                stddevs.append(phi + np.zeros(num_sites))
-        return stddevs
+            sig[m] = C["sigma"] if self.ergodic else C["sigma_ss"]
+            tau[m] = C['tau']
+            phi[m] = C["phi"] if self.ergodic else np.sqrt(
+                C["sigma_ss"] ** 2. - C["tau"] ** 2.)
 
     # Period-dependent coefficients (Table 3)
     COEFFS = CoeffsTable(sa_damping=5, table="""\
@@ -265,18 +380,6 @@ class AbrahamsonEtAl2015SInter(GMPE):
     10.0  -0.2
     """)
 
-    CONSTS = {
-        # Period-Independent Coefficients (Table 2)
-        'n': 1.18,
-        'c': 1.88,
-        'theta3': 0.1,
-        'theta4': 0.9,
-        'theta5': 0.0,
-        'theta9': 0.4,
-        'c4': 10.0,
-        'C1': 7.8
-        }
-
 
 class AbrahamsonEtAl2015SInterHigh(AbrahamsonEtAl2015SInter):
     """
@@ -329,7 +432,7 @@ class AbrahamsonEtAl2015SSlab(AbrahamsonEtAl2015SInter):
     and the hypocentral depth is used to scale the ground motion by depth
     """
     #: Supported tectonic region type is subduction in-slab
-    DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTRASLAB
+    DEFINED_FOR_TECTONIC_REGION_TYPE = trt = const.TRT.SUBDUCTION_INTRASLAB
 
     #: Required distance measure is hypocentral for in-slab events
     REQUIRES_DISTANCES = {'rhypo'}
@@ -337,44 +440,7 @@ class AbrahamsonEtAl2015SSlab(AbrahamsonEtAl2015SInter):
     #: In-slab events require constraint of hypocentral depth and magnitude
     REQUIRES_RUPTURE_PARAMETERS = {'mag', 'hypo_depth'}
 
-    def _get_delta_c1(self, imt):
-        """
-        Returns the magnitude scaling parameter deltaC1 which is fixed at -0.3
-        for the central branch of the in-slab model
-        """
-        return -0.3
-
-    def _compute_focal_depth_term(self, C, rup):
-        """
-        Computes the hypocentral depth scaling term - as indicated by
-        equation (3)
-        """
-        if rup.hypo_depth > 120.0:
-            z_h = 120.0
-        else:
-            z_h = rup.hypo_depth
-        return C['theta11'] * (z_h - 60.)
-
-    def _compute_distance_term(self, C, mag, dists):
-        """
-        Computes the distance scaling term, as contained within equation (1b)
-        """
-        return ((C['theta2'] + C['theta14'] + self.CONSTS['theta3'] *
-                (mag - 7.8)) * np.log(dists.rhypo + self.CONSTS['c4'] *
-                np.exp((mag - 6.) * self.CONSTS['theta9'])) +
-                (C['theta6'] * dists.rhypo)) + C["theta10"]
-
-    def _compute_forearc_backarc_term(self, C, sites, dists):
-        """
-        Computes the forearc/backarc scaling term given by equation (4).
-        """
-        f_faba = np.zeros_like(dists.rhypo)
-        # Term only applies to backarc sites (F_FABA = 0. for forearc)
-        max_dist = dists.rhypo[sites.backarc]
-        max_dist[max_dist < 85.0] = 85.0
-        f_faba[sites.backarc] = C['theta7'] +\
-            (C['theta8'] * np.log(max_dist / 40.0))
-        return f_faba
+    delta_c1 = -0.3
 
 
 class AbrahamsonEtAl2015SSlabHigh(AbrahamsonEtAl2015SSlab):
@@ -383,13 +449,7 @@ class AbrahamsonEtAl2015SSlabHigh(AbrahamsonEtAl2015SSlab):
     values of the magnitude scaling for large slab earthquakes, as defined in
     table 8
     """
-
-    def _get_delta_c1(self, imt):
-        """
-        Returns them agnitude scaling parameter deltaC1 which is fixed at -0.1
-        for the upper branch of the in-slab model
-        """
-        return -0.1
+    delta_c1 = -0.1
 
 
 class AbrahamsonEtAl2015SSlabLow(AbrahamsonEtAl2015SSlab):
@@ -398,10 +458,4 @@ class AbrahamsonEtAl2015SSlabLow(AbrahamsonEtAl2015SSlab):
     values of the magnitude scaling for large slab earthquakes, as defined in
     table 8
     """
-
-    def _get_delta_c1(self, imt):
-        """
-        Returns them agnitude scaling parameter deltaC1 which is fixed at -0.5
-        for the lower branch of the in-slab model
-        """
-        return -0.5
+    delta_c1 = -0.5
