@@ -28,13 +28,141 @@ Module exports
 :class:`AtkinsonBoore2003SSlabJapanNSHMP2008`
 """
 import numpy as np
-# standard acceleration of gravity in m/s**2
 from scipy.constants import g
 import copy
 
 from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
 from openquake.hazardlib import const
 from openquake.hazardlib.imt import PGA, SA
+from openquake.baselib.general import CallableDict
+
+
+def _compute_mean(kind, C, g, mag, hypo_depth, rrup, vs30, pga_rock, imt):
+    """
+    Compute mean according to equation 1, page 1706.
+    """
+    hypo_depth = np.clip(hypo_depth, 0, 100.)
+    delta = 0.00724 * 10 ** (0.507 * mag)
+    R = np.sqrt(rrup ** 2 + delta ** 2)
+
+    s_amp = _compute_soil_amplification(kind, C, vs30, pga_rock, imt)
+
+    mean = (
+        # 1st term
+        C['c1'] + C['c2'] * mag +
+        # 2nd term
+        C['c3'] * hypo_depth +
+        # 3rd term
+        C['c4'] * R -
+        # 4th term
+        g * np.log10(R) +
+        # 5th, 6th and 7th terms
+        s_amp)
+
+    return mean
+
+
+_compute_site_class_dummy_variables = CallableDict()
+
+
+@_compute_site_class_dummy_variables.add('SInter')
+def _compute_site_class_dummy_variables_SInter(kind, vs30):
+    """
+    Compute site class dummy variables as explained in paragraph
+    'Functional Form', page 1706.
+    """
+    Sc = np.zeros_like(vs30)
+    Sd = np.zeros_like(vs30)
+    Se = np.zeros_like(vs30)
+
+    Sc[(vs30 > 360) & (vs30 <= 760)] = 1
+    Sd[(vs30 >= 180) & (vs30 <= 360)] = 1
+    Se[vs30 < 180] = 1
+
+    return Sc, Sd, Se
+
+
+@_compute_site_class_dummy_variables.add('SInter2008', 'SSlab2008')
+def _compute_site_class_dummy_variables_2008(kind, vs30):
+    """
+    Extend
+    :meth:`AtkinsonBoore2003SInter._compute_site_class_dummy_variables`
+    and includes dummy variable for B/C site conditions (vs30 > 760.)
+    """
+    Sbc = np.zeros_like(vs30)
+    Sc = np.zeros_like(vs30)
+    Sd = np.zeros_like(vs30)
+    Se = np.zeros_like(vs30)
+
+    Sbc[vs30 > 760.] = 1
+    Sc[(vs30 > 360) & (vs30 <= 760)] = 1
+    Sd[(vs30 >= 180) & (vs30 <= 360)] = 1
+    Se[vs30 < 180] = 1
+
+    return Sbc, Sc, Sd, Se
+
+
+_compute_soil_amplification = CallableDict()
+
+
+@_compute_soil_amplification.add('SInter')
+def _compute_soil_amplification_SInter(kind, C, vs30, pga_rock, imt):
+    """
+    Compute soil amplification (5th, 6th, and 7th terms in equation 1,
+    page 1706).
+    """
+    Sc, Sd, Se = _compute_site_class_dummy_variables(kind, vs30)
+    sl = _compute_soil_linear_factor(pga_rock, imt)
+
+    return C['c5'] * sl * Sc + C['c6'] * sl * Sd + C['c7'] * sl * Se
+
+
+@_compute_soil_amplification.add('SInter2008', 'SSlab2008')
+def _compute_soil_amplification_2008(kind, C, vs30, pga_rock, imt):
+    """
+    Compute soil amplification (5th, 6th, and 7th terms in equation 1,
+    page 1706) and add the B/C site condition as implemented by NSHMP.
+    """
+    Sbc, Sc, Sd, Se = _compute_site_class_dummy_variables(kind, vs30)
+    sl = _compute_soil_linear_factor(pga_rock, imt)
+
+    return (
+        C['c5'] * sl * Sbc * 0.5 +
+        C['c5'] * sl * Sc +
+        C['c6'] * sl * Sd +
+        C['c7'] * sl * Se)
+
+
+def _compute_soil_linear_factor(pga_rock, imt):
+    """
+    Compute soil linear factor as explained in paragraph 'Functional
+    Form', page 1706.
+    """
+    if imt.period >= 1:
+        return np.ones_like(pga_rock)
+    else:
+        sl = np.zeros_like(pga_rock)
+
+        pga_between_100_500 = (pga_rock > 100) & (pga_rock < 500)
+        pga_greater_equal_500 = pga_rock >= 500
+
+        is_SA_between_05_1 = 0.5 < imt.period < 1
+
+        is_SA_less_equal_05 = imt.period <= 0.5
+
+        if is_SA_between_05_1:
+            sl[pga_between_100_500] = (1 - (1. / imt.period - 1) *
+                                       (pga_rock[pga_between_100_500] -
+                                       100) / 400)
+            sl[pga_greater_equal_500] = 1 - (1. / imt.period - 1)
+
+        if is_SA_less_equal_05 or imt.period == 0:
+            sl[pga_between_100_500] = (1 - (pga_rock[pga_between_100_500] -
+                                       100) / 400)
+
+        sl[pga_rock <= 100] = 1
+
+        return sl
 
 
 class AtkinsonBoore2003SInter(GMPE):
@@ -72,11 +200,9 @@ class AtkinsonBoore2003SInter(GMPE):
     #: Supported standard deviation types are inter-event, intra-event
     #: and total, see table 1, page 1715
     DEFINED_FOR_STANDARD_DEVIATION_TYPES = {
-        const.StdDev.TOTAL,
-        const.StdDev.INTER_EVENT,
-        const.StdDev.INTRA_EVENT}
+        const.StdDev.TOTAL, const.StdDev.INTER_EVENT, const.StdDev.INTRA_EVENT}
 
-    DEFINED_FOR_REFERENCE_VELOCITY = 600
+    DEFINED_FOR_REFERENCE_VELOCITY = 800
 
     #: Required site parameters is Vs30, used to distinguish between NEHRP
     #: soil classes, see paragraph 'Functional Form', page 1706
@@ -90,159 +216,61 @@ class AtkinsonBoore2003SInter(GMPE):
     #: 1, page 1706
     REQUIRES_DISTANCES = {'rrup'}
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+    kind = 'SInter'
+
+    def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+        <.base.GroundShakingIntensityModel.compute>`
         for spec of input and result values.
         """
-        # extracting dictionary of coefficients specific to required
-        # intensity measure type.
-        C = self.COEFFS_SINTER[imt]
-
         # cap magnitude values at 8.5, see page 1709
-        mag = rup.mag
-        if mag > 8.5:
-            mag = 8.5
+        mag = np.clip(ctx.mag, 0, 8.5)
+        if self.kind == 'SInter2008':
+            ctx = copy.copy(ctx)
+            ctx.hypo_depth = 20.
 
         # compute PGA on rock (needed for site amplification calculation)
         G = 10 ** (1.2 - 0.18 * mag)
-        pga_rock = self._compute_mean(self.COEFFS_SINTER[PGA()], G, mag,
-                                      rup.hypo_depth, dists.rrup, sites.vs30,
-                                      # by passing pga_rock > 500 the soil
-                                      # amplification is 0
-                                      np.zeros_like(sites.vs30) + 600,
-                                      PGA())
-        pga_rock = 10 ** (pga_rock)
+        pga_rock = _compute_mean(self.kind, self.COEFFS_SINTER[PGA()], G, mag,
+                                 ctx.hypo_depth, ctx.rrup, ctx.vs30,
+                                 # by passing pga_rock > 500 the soil
+                                 # amplification is 0
+                                 np.zeros_like(ctx.vs30) + 600,
+                                 PGA())
+        pga_rock = 10 ** pga_rock
+        for m, imt in enumerate(imts):
 
-        # periods 0.4 s (2.5 Hz) and 0.2 s (5 Hz) need a special case because
-        # of the erratum. SA for 0.4s and 0.2s is computed and a weighted sum
-        # is returned
-        if imt.period in (0.2, 0.4):
-            C04 = self.COEFFS_SINTER[SA(period=0.4, damping=5.0)]
-            C02 = self.COEFFS_SINTER[SA(period=0.2, damping=5.0)]
-            mean04 = self._compute_mean(C04, G, mag, rup.hypo_depth,
-                                        dists.rrup, sites.vs30, pga_rock, imt)
-            mean02 = self._compute_mean(C02, G, mag, rup.hypo_depth,
-                                        dists.rrup, sites.vs30, pga_rock, imt)
+            C = self.COEFFS_SINTER[imt]
+            # periods 0.4 s (2.5 Hz) and 0.2 s (5 Hz) need a special case
+            # because of the erratum. SA for 0.4s and 0.2s is computed and a
+            # weighted sum is returned
+            if imt.period in (0.2, 0.4):
+                C04 = self.COEFFS_SINTER[SA(period=0.4, damping=5.0)]
+                C02 = self.COEFFS_SINTER[SA(period=0.2, damping=5.0)]
+                mean04 = _compute_mean(self.kind, C04, G, mag, ctx.hypo_depth,
+                                       ctx.rrup, ctx.vs30, pga_rock, imt)
+                mean02 = _compute_mean(self.kind, C02, G, mag, ctx.hypo_depth,
+                                       ctx.rrup, ctx.vs30, pga_rock, imt)
 
-            if imt.period == 0.2:
-                mean = 0.333 * mean02 + 0.667 * mean04
+                if imt.period == 0.2:
+                    mean[m] = 0.333 * mean02 + 0.667 * mean04
+                else:
+                    mean[m] = 0.333 * mean04 + 0.667 * mean02
             else:
-                mean = 0.333 * mean04 + 0.667 * mean02
-        else:
-            mean = self._compute_mean(C, G, mag, rup.hypo_depth, dists.rrup,
-                                      sites.vs30, pga_rock, imt)
+                mean[m] = _compute_mean(self.kind, C, G, mag, ctx.hypo_depth,
+                                        ctx.rrup, ctx.vs30, pga_rock, imt)
 
-        # convert from log10 to ln and units from cm/s**2 to g
-        mean = np.log((10 ** mean) * 1e-2 / g)
+            # convert from log10 to ln and units from cm/s**2 to g
+            mean[m] = np.log((10 ** mean[m]) * 1e-2 / g)
 
-        if imt.period == 4.0:
-            mean /= 0.550
+            if imt.period == 4.0:
+                mean[m] /= 0.550
 
-        stddevs = self._get_stddevs(C, stddev_types, sites.vs30.shape[0])
-
-        return mean, stddevs
-
-    def _compute_mean(self, C, g, mag, hypo_depth, rrup, vs30, pga_rock, imt):
-        """
-        Compute mean according to equation 1, page 1706.
-        """
-        if hypo_depth > 100:
-            hypo_depth = 100
-        delta = 0.00724 * 10 ** (0.507 * mag)
-        R = np.sqrt(rrup ** 2 + delta ** 2)
-
-        s_amp = self._compute_soil_amplification(C, vs30, pga_rock, imt)
-
-        mean = (
-            # 1st term
-            C['c1'] + C['c2'] * mag +
-            # 2nd term
-            C['c3'] * hypo_depth +
-            # 3rd term
-            C['c4'] * R -
-            # 4th term
-            g * np.log10(R) +
-            # 5th, 6th and 7th terms
-            s_amp
-        )
-
-        return mean
-
-    @classmethod
-    def _compute_soil_amplification(cls, C, vs30, pga_rock, imt):
-        """
-        Compute soil amplification (5th, 6th, and 7th terms in equation 1,
-        page 1706).
-        """
-        Sc, Sd, Se = cls._compute_site_class_dummy_variables(vs30)
-        sl = cls._compute_soil_linear_factor(pga_rock, imt)
-
-        return C['c5'] * sl * Sc + C['c6'] * sl * Sd + C['c7'] * sl * Se
-
-    @classmethod
-    def _compute_site_class_dummy_variables(cls, vs30):
-        """
-        Compute site class dummy variables as explained in paragraph
-        'Functional Form', page 1706.
-        """
-        Sc = np.zeros_like(vs30)
-        Sd = np.zeros_like(vs30)
-        Se = np.zeros_like(vs30)
-
-        Sc[(vs30 > 360) & (vs30 <= 760)] = 1
-        Sd[(vs30 >= 180) & (vs30 <= 360)] = 1
-        Se[vs30 < 180] = 1
-
-        return Sc, Sd, Se
-
-    @classmethod
-    def _compute_soil_linear_factor(cls, pga_rock, imt):
-        """
-        Compute soil linear factor as explained in paragraph 'Functional
-        Form', page 1706.
-        """
-        if imt.period >= 1:
-            return np.ones_like(pga_rock)
-        else:
-            sl = np.zeros_like(pga_rock)
-
-            pga_between_100_500 = (pga_rock > 100) & (pga_rock < 500)
-            pga_greater_equal_500 = pga_rock >= 500
-
-            is_SA_between_05_1 = 0.5 < imt.period < 1
-
-            is_SA_less_equal_05 = imt.period <= 0.5
-
-            if is_SA_between_05_1:
-                sl[pga_between_100_500] = (1 - (1. / imt.period - 1) *
-                                           (pga_rock[pga_between_100_500] -
-                                           100) / 400)
-                sl[pga_greater_equal_500] = 1 - (1. / imt.period - 1)
-
-            if is_SA_less_equal_05 or imt.period == 0:
-                sl[pga_between_100_500] = (1 - (pga_rock[pga_between_100_500] -
-                                           100) / 400)
-
-            sl[pga_rock <= 100] = 1
-
-            return sl
-
-    def _get_stddevs(self, C, stddev_types, num_sites):
-        """
-        Return standard deviations as defined in table 1, pag 1715.
-        """
-        stddevs = []
-        for stddev_type in stddev_types:
-            assert stddev_type in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev_type == const.StdDev.TOTAL:
-                stddevs.append(np.log(10 ** C['sigma']) + np.zeros(num_sites))
-            elif stddev_type == const.StdDev.INTRA_EVENT:
-                stddevs.append(np.log(10 ** C['s1']) + np.zeros(num_sites))
-            elif stddev_type == const.StdDev.INTER_EVENT:
-                stddevs.append(np.log(10 ** C['s2']) + np.zeros(num_sites))
-        return stddevs
+            sig[m] = np.log(10 ** C['sigma'])
+            if 's2' in C.dtype.names:  # in the Gupta subclass
+                tau[m] = np.log(10 ** C['s2'])
+                phi[m] = np.log(10 ** C['s1'])
 
     COEFFS_SINTER = CoeffsTable(sa_damping=5, table="""\
     IMT      c1          c2          c3           c4          c5          c6          c7          sigma       s1          s2
@@ -275,43 +303,42 @@ class AtkinsonBoore2003SSlab(AtkinsonBoore2003SInter):
     #: Supported tectonic region type is subduction interface
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTRASLAB
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+    def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+        <.base.GroundShakingIntensityModel.compute>`
         for spec of input and result values.
         """
-        # extracting dictionary of coefficients specific to required
-        # intensity measure type.
-        C = self.COEFFS_SSLAB[imt]
-
         # cap magnitude values at 8.0, see page 1709
-        mag = rup.mag
-        if mag >= 8.0:
-            mag = 8.0
+        mag = np.clip(ctx.mag, 0, 8.0)
 
         # compute PGA on rock (needed for site amplification calculation)
         G = 10 ** (0.301 - 0.01 * mag)
-        pga_rock = self._compute_mean(self.COEFFS_SSLAB[PGA()], G, mag,
-                                      rup.hypo_depth, dists.rrup, sites.vs30,
-                                      # by passing pga_rock > 500 the soil
-                                      # amplification is 0
-                                      np.zeros_like(sites.vs30) + 600,
-                                      PGA())
-        pga_rock = 10 ** (pga_rock)
+        pga_rock = _compute_mean(self.kind, self.COEFFS_SSLAB[PGA()], G, mag,
+                                 ctx.hypo_depth, ctx.rrup, ctx.vs30,
+                                 # by passing pga_rock > 500 the soil
+                                 # amplification is 0
+                                 np.zeros_like(ctx.vs30) + 600,
+                                 PGA())
+        pga_rock = 10 ** pga_rock
+        for m, imt in enumerate(imts):
 
-        # compute actual mean and convert from log10 to ln and units from
-        # cm/s**2 to g
-        mean = self._compute_mean(C, G, mag, rup.hypo_depth, dists.rrup,
-                                  sites.vs30, pga_rock, imt)
-        mean = np.log((10 ** mean) * 1e-2 / g)
+            C = self.COEFFS_SSLAB[imt]
 
-        if imt.period == 4.0:
-            mean /= 0.550
+            # compute actual mean and convert from log10 to ln and units from
+            # cm/s**2 to g
+            mean[m] = _compute_mean(
+                self.kind, C, G, mag, ctx.hypo_depth, ctx.rrup,
+                ctx.vs30, pga_rock, imt)
+            mean[m] = np.log((10 ** mean[m]) * 1e-2 / g)
 
-        stddevs = self._get_stddevs(C, stddev_types, sites.vs30.shape[0])
+            if imt.period == 4.0:
+                mean[m] /= 0.550
 
-        return mean, stddevs
+            sig[m] = np.log(10 ** C['sigma'])
+            if 's2' in C.dtype.names:  # in the Gupta subclass
+                tau[m] = np.log(10 ** C['s2'])
+                phi[m] = np.log(10 ** C['s1'])
 
     COEFFS_SSLAB = CoeffsTable(sa_damping=5, table="""\
     IMT      c1         c2         c3         c4         c5          c6         c7         sigma      s1        s2
@@ -341,59 +368,7 @@ class AtkinsonBoore2003SInterNSHMP2008(AtkinsonBoore2003SInter):
     in ``hazSUBXnga.f`` Fortran code available at:
     http://earthquake.usgs.gov/hazards/products/conterminous/2008/software/
     """
-
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
-        """
-        See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
-        for spec of input and result values.
-
-        Call super class method with hypocentral depth fixed at 20 km
-        """
-        # fix hypocentral depth to 20 km. Create new rupture context to avoid
-        # changing the original one
-        new_rup = copy.copy(rup)
-        new_rup.hypo_depth = 20.
-
-        mean, stddevs = super().get_mean_and_stddevs(
-            sites, new_rup, dists, imt, stddev_types)
-
-        return mean, stddevs
-
-    @classmethod
-    def _compute_soil_amplification(cls, C, vs30, pga_rock, imt):
-        """
-        Compute soil amplification (5th, 6th, and 7th terms in equation 1,
-        page 1706) and add the B/C site condition as implemented by NSHMP.
-        """
-        Sbc, Sc, Sd, Se = cls._compute_site_class_dummy_variables(vs30)
-        sl = cls._compute_soil_linear_factor(pga_rock, imt)
-
-        return (
-            C['c5'] * sl * Sbc * 0.5 +
-            C['c5'] * sl * Sc +
-            C['c6'] * sl * Sd +
-            C['c7'] * sl * Se
-        )
-
-    @classmethod
-    def _compute_site_class_dummy_variables(cls, vs30):
-        """
-        Extend
-        :meth:`AtkinsonBoore2003SInter._compute_site_class_dummy_variables`
-        and includes dummy variable for B/C site conditions (vs30 > 760.)
-        """
-        Sbc = np.zeros_like(vs30)
-        Sc = np.zeros_like(vs30)
-        Sd = np.zeros_like(vs30)
-        Se = np.zeros_like(vs30)
-
-        Sbc[vs30 > 760.] = 1
-        Sc[(vs30 > 360) & (vs30 <= 760)] = 1
-        Sd[(vs30 >= 180) & (vs30 <= 360)] = 1
-        Se[vs30 < 180] = 1
-
-        return Sbc, Sc, Sd, Se
+    kind = 'SInter2008'
 
 
 class AtkinsonBoore2003SSlabNSHMP2008(AtkinsonBoore2003SSlab):
@@ -409,30 +384,7 @@ class AtkinsonBoore2003SSlabNSHMP2008(AtkinsonBoore2003SSlab):
     in ``hazgridXnga2.f`` Fortran code available at:
     http://earthquake.usgs.gov/hazards/products/conterminous/2008/software/
     """
-    @classmethod
-    def _compute_soil_amplification(cls, C, vs30, pga_rock, imt):
-        """
-        Compute soil amplification (5th, 6th, and 7th terms in equation 1,
-        page 1706) and add the B/C site condition as implemented by NSHMP.
-
-        Call
-        :meth:`AtkinsonBoore2003SInterNSHMP2008._compute_soil_amplification`
-        """
-        return AtkinsonBoore2003SInterNSHMP2008._compute_soil_amplification(
-            C, vs30, pga_rock, imt)
-
-    @classmethod
-    def _compute_site_class_dummy_variables(cls, vs30):
-        """
-        Extend
-        :meth:`AtkinsonBoore2003SInter._compute_site_class_dummy_variables`
-        and includes dummy variable for B/C site conditions (vs30 > 760.)
-
-        Call
-        meth:`AtkinsonBoore2003SInter._compute_site_class_dummy_variables`
-        """
-        return AtkinsonBoore2003SInterNSHMP2008. \
-            _compute_site_class_dummy_variables(vs30)
+    kind = 'SSlab2008'
 
 
 class AtkinsonBoore2003SSlabCascadia(AtkinsonBoore2003SSlab):
@@ -457,13 +409,12 @@ class AtkinsonBoore2003SSlabCascadia(AtkinsonBoore2003SSlab):
     """)
 
 
-class AtkinsonBoore2003SSlabCascadiaNSHMP2008(AtkinsonBoore2003SSlabCascadia,
-                                              AtkinsonBoore2003SSlabNSHMP2008):
+class AtkinsonBoore2003SSlabCascadiaNSHMP2008(AtkinsonBoore2003SSlabNSHMP2008):
     """
     Combines :class:`AtkinsonBoore2003SSlabNSHMP2008` for NSHMP site
     amplification with :class:`AtkinsonBoore2003SSlabCascadia` for Cascadia.
     """
-    pass
+    COEFFS_SSLAB = AtkinsonBoore2003SSlabCascadia.COEFFS_SSLAB
 
 
 class AtkinsonBoore2003SSlabJapan(AtkinsonBoore2003SSlab):
@@ -485,8 +436,7 @@ class AtkinsonBoore2003SSlabJapan(AtkinsonBoore2003SSlab):
     """)
 
 
-class AtkinsonBoore2003SSlabJapanNSHMP2008(AtkinsonBoore2003SSlabJapan,
-                                           AtkinsonBoore2003SSlabNSHMP2008):
+class AtkinsonBoore2003SSlabJapanNSHMP2008(AtkinsonBoore2003SSlabNSHMP2008):
     """
     Combines :class:`AtkinsonBoore2003SSlabNSHMP2008` for NSHMP site
     amplification with :class:`AtkinsonBoore2003SSlabJapan` for Japan.
@@ -495,4 +445,4 @@ class AtkinsonBoore2003SSlabJapanNSHMP2008(AtkinsonBoore2003SSlabJapan,
     and 2 of Table 3 to test vector for
     AtkinsonBoore2003SSlabCascadiaNSHMP2008.
     """
-    pass
+    COEFFS_SSLAB = AtkinsonBoore2003SSlabJapan.COEFFS_SSLAB
