@@ -25,10 +25,383 @@ Module exports :class:`ChiouYoungs2014`
                :class:`ChiouYoungs2014NearFaultEffect`
 """
 import numpy as np
+
+from openquake.baselib.general import CallableDict
 from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
 from openquake.hazardlib import const
 from openquake.hazardlib.imt import PGA, PGV, SA
 
+CONSTANTS = {"c2": 1.06, "c4": -2.1, "c4a": -0.5, "crb": 50.0,
+             "c8a": 0.2695, "c11": 0.0, "phi6": 300.0, "phi6jp": 800.0}
+
+
+def _get_centered_cdpp(clsname, ctx):
+    """
+    Returns the centred dpp term (zero by default)
+    """
+    if clsname.endswith("NearFaultEffect"):
+        return ctx.rcdpp
+    return np.zeros(ctx.rrup.shape)
+
+
+def _get_centered_z1pt0(clsname, ctx):
+    """
+    Get z1pt0 centered on the Vs30- dependent average z1pt0(m)
+    California and non-Japan regions
+    """
+    if clsname.endswith("Japan"):
+        mean_z1pt0 = (-5.23 / 2.) * np.log(((ctx.vs30 ** 2.) + 412.39 ** 2.)
+                                           / (1360 ** 2. + 412.39 ** 2.))
+        return ctx.z1pt0 - np.exp(mean_z1pt0)
+
+    #: California and non-Japan regions
+    mean_z1pt0 = (-7.15 / 4.) * np.log(((ctx.vs30) ** 4. + 570.94 ** 4.)
+                                       / (1360 ** 4. + 570.94 ** 4.))
+    return ctx.z1pt0 - np.exp(mean_z1pt0)
+
+
+def _get_centered_ztor(ctx):
+    """
+    Get ztor centered on the M- dependent avarage ztor(km)
+    by different fault types.
+    """
+    if 30 <= ctx.rake <= 150:
+        # Reverse and reverse-oblique faulting
+        mean_ztor = max(2.704 - 1.226 * max(ctx.mag - 5.849, 0.0), 0.) ** 2
+    else:
+        # Strike-slip and normal faulting
+        mean_ztor = max(2.673 - 1.136 * max(ctx.mag - 4.970, 0.0), 0.) ** 2
+    return ctx.ztor - mean_ztor
+
+
+def _get_ln_y_ref(ctx, C):
+    """
+    Get an intensity on a reference soil.
+    Implements eq. 13a.
+    """
+    # Reverse faulting flag
+    Frv = 1. if 30 <= ctx.rake <= 150 else 0.
+    # Normal faulting flag
+    Fnm = 1. if -120 <= ctx.rake <= -60 else 0.
+    # A part in eq. 11
+    mag_test1 = np.cosh(2. * max(ctx.mag - 4.5, 0))
+    # Centered DPP
+    centered_dpp = 0
+    # Centered Ztor
+    centered_ztor = 0
+
+    dist_taper = np.fmax(1 - (np.fmax(ctx.rrup - 40,
+                              np.zeros_like(ctx)) / 30.),
+                         np.zeros_like(ctx))
+    dist_taper = dist_taper.astype(np.float64)
+    ln_y_ref = (
+        # first part of eq. 11
+        C['c1']
+        + (C['c1a'] + C['c1c'] / mag_test1) * Frv
+        + (C['c1b'] + C['c1d'] / mag_test1) * Fnm
+        + (C['c7'] + C['c7b'] / mag_test1) * centered_ztor
+        + (C['c11'] + C['c11b'] / mag_test1) *
+        np.cos(np.radians(ctx.dip)) ** 2
+        # second part
+        + C['c2'] * (ctx.mag - 6)
+        + ((C['c2'] - C['c3']) / C['cn'])
+        * np.log(1 + np.exp(C['cn'] * (C['cm'] - ctx.mag)))
+        # third part
+        + C['c4']
+        * np.log(ctx.rrup + C['c5']
+                 * np.cosh(C['c6'] * max(ctx.mag - C['chm'], 0)))
+        + (C['c4a'] - C['c4'])
+        * np.log(np.sqrt(ctx.rrup ** 2 + C['crb'] ** 2))
+        # forth part
+        + (C['cg1'] + C['cg2'] / (np.cosh(max(ctx.mag - C['cg3'], 0))))
+        * ctx.rrup
+        # fifth part
+        + C['c8'] * dist_taper
+        * min(max(ctx.mag - 5.5, 0) / 0.8, 1.0)
+        * np.exp(-1 * C['c8a'] * (ctx.mag - C['c8b']) ** 2) * centered_dpp
+        # sixth part
+        # + C['c9'] * Fhw * np.cos(math.radians(ctx.dip)) *
+        # (C['c9a'] + (1 - C['c9a']) * np.tanh(ctx.rx / C['c9b']))
+        # * (1 - np.sqrt(ctx.rjb ** 2 + ctx.ztor ** 2)
+        #   / (ctx.rrup + 1.0))
+    )
+    return ln_y_ref
+
+
+def _get_mean(ctx, C, ln_y_ref, exp1, exp2):
+    """
+    Add site effects to an intensity. Implements eq. 13b.
+    """
+    eta = epsilon = 0.
+    ln_y = (
+        # first line of eq. 12
+        ln_y_ref + eta
+        # second line
+        + C['phi1'] * np.log(ctx.vs30 / 1130).clip(-np.inf, 0)
+        # third line
+        + C['phi2'] * (exp1 - exp2)
+        * np.log((np.exp(ln_y_ref) * np.exp(eta) + C['phi4']) / C['phi4'])
+        # fourth line - removed
+        # fifth line
+        + epsilon)
+    return ln_y
+
+
+def get_basin_depth_term(clsname, C, centered_z1pt0):
+    """
+    Returns the basin depth scaling
+    """
+    if clsname.endswith("Japan"):
+        return C["phi5jp"] * (1.0 - np.exp(-centered_z1pt0 /
+                                           CONSTANTS["phi6jp"]))
+    return C["phi5"] * (1.0 - np.exp(-centered_z1pt0 /
+                                     CONSTANTS["phi6"]))
+
+
+def get_directivity(clsname, C, ctx):
+    """
+    Returns the directivity term.
+
+    The directivity prediction parameter is centered on the average
+    directivity prediction parameter. Here we set the centered_dpp
+    equal to zero, since the near fault directivity effect prediction is
+    off by default in our calculation.
+    """
+    cdpp = _get_centered_cdpp(clsname, ctx)
+    if not np.any(cdpp > 0.0):
+        # No directivity term
+        return 0.0
+    f_dir = np.exp(-C["c8a"] * ((ctx.mag - C["c8b"]) ** 2.)) * cdpp
+    f_dir *= min((max(ctx.mag - 5.5, 0.0) / 0.8), 1.)
+    rrup_max = ctx.rrup - 40.
+    rrup_max[rrup_max < 0.0] = 0.0
+    rrup_max = 1.0 - (rrup_max / 30.)
+    rrup_max[rrup_max < 0.0] = 0.0
+    return C["c8"] * rrup_max * f_dir
+
+
+get_far_field_distance_scaling = CallableDict()
+
+
+@get_far_field_distance_scaling.add("CAL")
+def get_far_field_distance_scaling_1(region, C, mag, rrup):
+    """
+    Returns the far-field distance scaling term - both magnitude and
+    distance - for California and other regions
+    """
+    # Get the attenuation distance scaling
+    f_r = (CONSTANTS["c4a"] - CONSTANTS["c4"]) * np.log(
+        np.sqrt(rrup ** 2. + CONSTANTS["crb"] ** 2.))
+    # Get the magnitude dependent term
+    f_rm = C["cg1"] + (C["cg2"] / np.cosh(max(mag - C["cg3"], 0.0)))
+    return f_r + f_rm * rrup
+
+
+@get_far_field_distance_scaling.add("JPN")
+def get_far_field_distance_scaling_2(region, C, mag, rrup):
+    """
+    Returns the far-field distance scaling term - both magnitude and
+    distance - for Japan
+    """
+    # Get the attenuation distance scaling
+    f_r = (CONSTANTS["c4a"] - CONSTANTS["c4"]) * np.log(
+        np.sqrt(rrup ** 2. + CONSTANTS["crb"] ** 2.))
+
+    # Get the magnitude dependent term
+    f_rm = (C["cg1"] +
+            (C["cg2"] / np.cosh(max(mag - C["cg3"], 0.0)))) * rrup
+    if (mag > 6.0) and (mag < 6.9):
+        # Apply adjustment factor for Japan
+        f_rm *= C["gjpit"]
+    return f_r + f_rm
+
+
+@get_far_field_distance_scaling.add("ITA")
+def get_far_field_distance_scaling_3(region, C, mag, rrup):
+    """
+    Returns the far-field distance scaling term - both magnitude and
+    distance - for Italy
+    """
+    # Get the attenuation distance scaling
+    f_r = (CONSTANTS["c4a"] - CONSTANTS["c4"]) * np.log(
+        np.sqrt(rrup ** 2. + CONSTANTS["crb"] ** 2.))
+
+    # Get the magnitude dependent term
+    f_rm = (C["cg1"] +
+            (C["cg2"] / np.cosh(max(mag - C["cg3"], 0.0)))) * rrup
+    if (mag > 6.0) and (mag < 6.9):
+        # Apply adjustment factor for Italy
+        f_rm *= C["gjpit"]
+    return f_r + f_rm
+
+
+@get_far_field_distance_scaling.add("WEN")
+def get_far_field_distance_scaling_4(region, C, mag, rrup):
+    """
+    Returns the far-field distance scaling term - both magnitude and
+    distance - for Wenchuan
+    """
+    # Get the attenuation distance scaling
+    f_r = (CONSTANTS["c4a"] - CONSTANTS["c4"]) * np.log(
+        np.sqrt(rrup ** 2. + CONSTANTS["crb"] ** 2.))
+
+    # Get the magnitude dependent term
+    f_rm = (C["cg1"] +
+            (C["cg2"] / np.cosh(max(mag - C["cg3"], 0.0)))) * rrup
+    # Apply adjustment factor for Wenchuan
+    return f_r + (f_rm * C["gwn"])
+
+
+def get_geometric_spreading(C, mag, rrup):
+    """
+    Returns the near-field geometric spreading term
+    """
+    # Get the near-field magnitude scaling
+    return CONSTANTS["c4"] * np.log(
+        rrup + C["c5"] * np.cosh(C["c6"] * max(mag - C["chm"], 0.0)))
+
+
+def get_hanging_wall_term(C, ctx):
+    """
+    Returns the hanging wall term
+    """
+    fhw = np.zeros(ctx.rrup.shape)
+    idx = ctx.rx >= 0.0
+    if np.any(idx):
+        fdist = 1.0 - (np.sqrt(ctx.rjb[idx] ** 2. + ctx.ztor ** 2.) /
+                       (ctx.rrup[idx] + 1.0))
+        fdist *= (C["c9a"] + (1.0 - C["c9a"]) * np.tanh(ctx.rx[idx] /
+                                                        C["c9b"]))
+        fhw[idx] += (C["c9"] * np.cos(np.radians(ctx.dip)) * fdist)
+    return fhw
+
+
+def get_linear_site_term(clsname, C, ctx):
+    """
+    Returns the linear site scaling term
+    """
+    if clsname.endswith("Japan"):
+        return C["phi1jp"] * np.log(ctx.vs30 / 1130).clip(-np.inf, 0.0)
+    return C["phi1"] * np.log(ctx.vs30 / 1130).clip(-np.inf, 0.0)
+
+
+def get_region(clsname):
+    if clsname.endswith("Italy"):
+        return "ITA"
+    elif clsname.endswith("Japan"):
+        return "JPN"
+    elif clsname.endswith("Wenchuan"):
+        return "WEN"
+    else:
+        return "CAL"
+
+
+def get_ln_y_ref(clsname, C, ctx):
+    """
+    Returns the ground motion on the reference rock, described fully by
+    Equation 11
+    """
+    region = get_region(clsname)
+    delta_ztor = _get_centered_ztor(ctx)
+    return (get_stress_scaling(C) +
+            get_magnitude_scaling(C, ctx.mag) +
+            get_source_scaling_terms(C, ctx, delta_ztor) +
+            get_hanging_wall_term(C, ctx) +
+            get_geometric_spreading(C, ctx.mag, ctx.rrup) +
+            get_far_field_distance_scaling(region, C, ctx.mag, ctx.rrup) +
+            get_directivity(clsname, C, ctx))
+
+
+def get_magnitude_scaling(C, mag):
+    """
+    Returns the magnitude scaling
+    """
+    f_m = np.log(1.0 + np.exp(C["cn"] * (C["cm"] - mag)))
+    f_m = CONSTANTS["c2"] * (mag - 6.0) +\
+        ((CONSTANTS["c2"] - C["c3"]) / C["cn"]) * f_m
+    return f_m
+
+
+def get_nonlinear_site_term(C, ctx, y_ref):
+    """
+    Returns the nonlinear site term and the Vs-scaling factor (to be
+    used in the standard deviation model
+    """
+    vs = ctx.vs30.clip(-np.inf, 1130.0)
+    f_nl_scaling = C["phi2"] * (np.exp(C["phi3"] * (vs - 360.)) -
+                                np.exp(C["phi3"] * (1130. - 360.)))
+    f_nl = np.log((y_ref + C["phi4"]) / C["phi4"]) * f_nl_scaling
+    return f_nl, f_nl_scaling
+
+
+def get_phi(C, mag, ctx, nl0):
+    """
+    Returns the within-event variability described in equation 13, line 3
+    """
+    phi = C["sig3"] * np.ones(ctx.vs30.shape)
+    phi[ctx.vs30measured] = 0.7
+    phi = np.sqrt(phi + ((1.0 + nl0) ** 2.))
+    mdep = C["sig1"] + (((C["sig2"] - C["sig1"]) / 1.5) *
+                        (min(max(mag, 5.0), 6.5) - 5.0))
+    return mdep * phi
+
+
+def get_source_scaling_terms(C, ctx, delta_ztor):
+    """
+    Returns additional source scaling parameters related to style of
+    faulting, dip and top of rupture depth
+    """
+    f_src = 0.0
+    coshm = np.cosh(2.0 * max(ctx.mag - 4.5, 0.0))
+    # Style of faulting term
+    if 30 <= ctx.rake <= 150:
+        # reverse faulting flag
+        f_src += (C["c1a"] + (C["c1c"] / coshm))
+    elif -120 <= ctx.rake <= -60:
+        # normal faulting flag
+        f_src += (C["c1b"] + (C["c1d"] / coshm))
+    # Top of rupture term
+    f_src += ((C["c7"] + (C["c7b"] / coshm)) * delta_ztor)
+    # Dip term
+    f_src += ((CONSTANTS["c11"] + (C["c11b"] / coshm)) *
+              np.cos(np.radians(ctx.dip)) ** 2.0)
+    return f_src
+
+
+def get_stddevs(clsname, C, ctx, mag, y_ref, f_nl_scaling):
+    """
+    Returns the standard deviation model described in equation 13
+    """
+    if clsname == 'ChiouYoungs2014PEER':
+        # the standard deviation, which is fixed at 0.65 for every site
+        return [0.65 * np.ones_like(ctx.vs30), 0, 0]
+
+    # Determines the nonlinear term described in equation 13, line 4
+    nl0 = f_nl_scaling * (y_ref / (y_ref + C["phi4"]))
+    # Get between and within-event variability
+    tau = get_tau(C, mag)
+    phi_nl0 = get_phi(C, mag, ctx, nl0)
+    # Get total standard deviation propagating the uncertainty in the
+    # nonlinear amplification term
+    sigma = np.sqrt(((1.0 + nl0) ** 2.) * (tau ** 2.) + phi_nl0 ** 2.)
+    return [sigma, np.abs((1 + nl0) * tau), phi_nl0]
+
+
+def get_stress_scaling(C):
+    """
+    Returns the stress drop scaling factor
+    """
+    return C["c1"]
+
+
+def get_tau(C, mag):
+    """
+    Returns the between-event variability described in equation 13, line 2
+    """
+    # eq. 13 to calculate inter-event standard error
+    mag_test = min(max(mag, 5.0), 6.5) - 5.0
+    return C['tau1'] + ((C['tau2'] - C['tau1']) / 1.5) * mag_test
+
 
 class ChiouYoungs2014(GMPE):
     """
@@ -38,8 +411,9 @@ class ChiouYoungs2014(GMPE):
     NGA Model for the Average Horizontal Component of Peak Ground Motion and
     Response Spectra, Earthquake Spectra, 30(3), 1117 - 1153,
     DOI: 10.1193/072813EQS219M
-
     """
+    adapted = False  # overridden in acme_2019
+
     #: Supported tectonic region type is active shallow crust
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.ACTIVE_SHALLOW_CRUST
 
@@ -70,239 +444,35 @@ class ChiouYoungs2014(GMPE):
     #: Reference shear wave velocity
     DEFINED_FOR_REFERENCE_VELOCITY = 1130
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+        <.base.GroundShakingIntensityModel.compute>`
         for spec of input and result values.
         """
-        C = self.COEFFS[imt]
-        # Get ground motion on reference rock
-        ln_y_ref = self.get_ln_y_ref(C, rup, dists)
-        y_ref = np.exp(ln_y_ref)
-        # Get the site amplification
-        # Get basin depth
-        dz1pt0 = self._get_centered_z1pt0(sites)
-        # In the case that Z1.0 = 0.0 then no deep soil correction is applied
-        dz1pt0[sites.z1pt0 <= 0.0] = 0.0
-        f_z1pt0 = self.get_basin_depth_term(C, dz1pt0)
-        # Get linear amplification term
-        f_lin = self.get_linear_site_term(C, sites)
-        # Get nonlinear amplification term
-        f_nl, f_nl_scaling = self.get_nonlinear_site_term(C, sites, y_ref)
-        # Add on the site amplification
-        mean = ln_y_ref + (f_lin + f_nl + f_z1pt0)
-        # Get standard deviations
-        stddevs = self.get_stddevs(C, sites, rup.mag, y_ref, f_nl_scaling,
-                                   stddev_types)
-        return mean, stddevs
-
-    def get_ln_y_ref(self, C, rup, dists):
-        """
-        Returns the ground motion on the reference rock, described fully by
-        Equation 11
-        """
-        delta_ztor = self._get_centered_ztor(rup)
-        return (self.get_stress_scaling(C) +
-                self.get_magnitude_scaling(C, rup.mag) +
-                self.get_source_scaling_terms(C, rup, delta_ztor) +
-                self.get_hanging_wall_term(C, rup, dists) +
-                self.get_geometric_spreading(C, rup.mag, dists.rrup) +
-                self.get_far_field_distance_scaling(C, rup.mag, dists.rrup) +
-                self.get_directivity(C, rup, dists))
-
-    def get_stddevs(self, C, sites, mag, y_ref, f_nl_scaling, stddev_types):
-        """
-        Returns the standard deviation model described in equation 13
-        """
-        # Determines the nonlinear term described in equation 13, line 4
-        nl0 = f_nl_scaling * (y_ref / (y_ref + C["phi4"]))
-        # Get between and within-event variability
-        tau = self.get_tau(C, mag)
-        phi_nl0 = self.get_phi(C, mag, sites, nl0)
-        # Get total standard deviation propagating the uncertainty in the
-        # nonlinear amplification term
-        sigma = np.sqrt(((1.0 + nl0) ** 2.) * (tau ** 2.) + phi_nl0 ** 2.)
-        stddevs = []
-        for stddev_type in stddev_types:
-            assert stddev_type in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev_type == const.StdDev.TOTAL:
-                # Equation 13, line 1
-                stddevs.append(sigma)
-            elif stddev_type == const.StdDev.INTRA_EVENT:
-                stddevs.append(phi_nl0)
-            elif stddev_type == const.StdDev.INTER_EVENT:
-                # This is implied by equation 13, line 1
-                stddevs.append(np.abs((1 + nl0) * tau))
-        return stddevs
-
-    def get_tau(self, C, mag):
-        """
-        Returns the between-event variability described in equation 13, line 2
-        """
-        # eq. 13 to calculate inter-event standard error
-        mag_test = min(max(mag, 5.0), 6.5) - 5.0
-        return C['tau1'] + ((C['tau2'] - C['tau1']) / 1.5) * mag_test
-
-    def get_phi(self, C, mag, sites, nl0):
-        """
-        Returns the within-event variability described in equation 13, line 3
-        """
-        phi = C["sig3"] * np.ones(sites.vs30.shape)
-        phi[sites.vs30measured] = 0.7
-        phi = np.sqrt(phi + ((1.0 + nl0) ** 2.))
-        mdep = C["sig1"] + (((C["sig2"] - C["sig1"]) / 1.5) *
-                            (min(max(mag, 5.0), 6.5) - 5.0))
-        return mdep * phi
-
-    def get_stress_scaling(self, C):
-        """
-        Returns the stress drop scaling factor
-        """
-        return C["c1"]
-
-    def get_magnitude_scaling(self, C, mag):
-        """
-        Returns the magnitude scaling
-        """
-        f_m = np.log(1.0 + np.exp(C["cn"] * (C["cm"] - mag)))
-        f_m = self.CONSTANTS["c2"] * (mag - 6.0) +\
-            ((self.CONSTANTS["c2"] - C["c3"]) / C["cn"]) * f_m
-        return f_m
-
-    def get_geometric_spreading(self, C, mag, rrup):
-        """
-        Returns the near-field geometric spreading term
-        """
-        # Get the near-field magnitude scaling
-        return self.CONSTANTS["c4"] * np.log(
-            rrup + C["c5"] * np.cosh(C["c6"] * max(mag - C["chm"], 0.0)))
-
-    def get_far_field_distance_scaling(self, C, mag, rrup):
-        """
-        Returns the far-field distance scaling term - both magnitude and
-        distance
-        """
-        # Get the attenuation distance scaling
-        f_r = (self.CONSTANTS["c4a"] - self.CONSTANTS["c4"]) * np.log(
-            np.sqrt(rrup ** 2. + self.CONSTANTS["crb"] ** 2.))
-        # Get the magnitude dependent term
-        f_rm = C["cg1"] + (C["cg2"] / np.cosh(max(mag - C["cg3"], 0.0)))
-        return f_r + f_rm * rrup
-
-    def get_source_scaling_terms(self, C, rup, delta_ztor):
-        """
-        Returns additional source scaling parameters related to style of
-        faulting, dip and top of rupture depth
-        """
-        f_src = 0.0
-        coshm = np.cosh(2.0 * max(rup.mag - 4.5, 0.0))
-        # Style of faulting term
-        if 30 <= rup.rake <= 150:
-            # reverse faulting flag
-            f_src += (C["c1a"] + (C["c1c"] / coshm))
-        elif -120 <= rup.rake <= -60:
-            # normal faulting flag
-            f_src += (C["c1b"] + (C["c1d"] / coshm))
-        # Top of rupture term
-        f_src += ((C["c7"] + (C["c7b"] / coshm)) * delta_ztor)
-        # Dip term
-        f_src += ((self.CONSTANTS["c11"] + (C["c11b"] / coshm)) *
-                  np.cos(np.radians(rup.dip)) ** 2.0)
-        return f_src
-
-    def get_hanging_wall_term(self, C, rup, dists):
-        """
-        Returns the hanging wall term
-        """
-        fhw = np.zeros(dists.rrup.shape)
-        idx = dists.rx >= 0.0
-        if np.any(idx):
-            fdist = 1.0 - (np.sqrt(dists.rjb[idx] ** 2. + rup.ztor ** 2.) /
-                           (dists.rrup[idx] + 1.0))
-            fdist *= (C["c9a"] + (1.0 - C["c9a"]) * np.tanh(dists.rx[idx] /
-                                                            C["c9b"]))
-            fhw[idx] += (C["c9"] * np.cos(np.radians(rup.dip)) * fdist)
-        return fhw
-
-    def get_directivity(self, C, rup, dists):
-        """
-        Returns the directivity term.
-
-        The directivity prediction parameter is centered on the average
-        directivity prediction parameter. Here we set the centered_dpp
-        equal to zero, since the near fault directivity effect prediction is
-        off by default in our calculation.
-        """
-        cdpp = self._get_centered_cdpp(dists)
-        if not np.any(cdpp > 0.0):
-            # No directivity term
-            return 0.0
-        f_dir = np.exp(-C["c8a"] * ((rup.mag - C["c8b"]) ** 2.)) * cdpp
-        f_dir *= min((max(rup.mag - 5.5, 0.0) / 0.8), 1.)
-        rrup_max = dists.rrup - 40.
-        rrup_max[rrup_max < 0.0] = 0.0
-        rrup_max = 1.0 - (rrup_max / 30.)
-        rrup_max[rrup_max < 0.0] = 0.0
-        return C["c8"] * rrup_max * f_dir
-
-    def get_linear_site_term(self, C, sites):
-        """
-        Returns the linear site scaling term
-        """
-        return C["phi1"] * np.log(sites.vs30 / 1130).clip(-np.inf, 0.0)
-
-    def get_basin_depth_term(self, C, centered_z1pt0):
-        """
-        Returns the basin depth scaling
-        """
-        return C["phi5"] * (1.0 - np.exp(-centered_z1pt0 /
-                                         self.CONSTANTS["phi6"]))
-
-    def get_nonlinear_site_term(self, C, sites, y_ref):
-        """
-        Returns the nonlinear site term and the Vs-scaling factor (to be
-        used in the standard deviation model
-        """
-        vs = sites.vs30.clip(-np.inf, 1130.0)
-        f_nl_scaling = C["phi2"] * (np.exp(C["phi3"] * (vs - 360.)) -
-                                    np.exp(C["phi3"] * (1130. - 360.)))
-        f_nl = np.log((y_ref + C["phi4"]) / C["phi4"]) * f_nl_scaling
-        return f_nl, f_nl_scaling
-
-    def _get_centered_z1pt0(self, sites):
-        """
-        Get z1pt0 centered on the Vs30- dependent average z1pt0(m)
-        California and non-Japan regions
-
-        """
-        #: California and non-Japan regions
-        mean_z1pt0 = (-7.15 / 4.) * np.log(((sites.vs30) ** 4. + 570.94 ** 4.)
-                                           / (1360 ** 4. + 570.94 ** 4.))
-        return sites.z1pt0 - np.exp(mean_z1pt0)
-
-    def _get_centered_ztor(self, rup):
-        """
-        Get ztor centered on the M- dependent avarage ztor(km)
-        by different fault types.
-        """
-        if 30 <= rup.rake <= 150:
-            # Reverse and reverse-oblique faulting
-            mean_ztor = max(2.704 - 1.226 * max(rup.mag - 5.849, 0.0), 0.) ** 2
-        else:
-            # Strike-slip and normal faulting
-            mean_ztor = max(2.673 - 1.136 * max(rup.mag - 4.970, 0.0), 0.) ** 2
-        return rup.ztor - mean_ztor
-
-    def _get_centered_cdpp(self, dists):
-        """
-        Returns the centred dpp term - in this case this is turned off by
-        default
-        """
-        return np.zeros(dists.rrup.shape)
+        name = self.__class__.__name__
+        for m, imt in enumerate(imts):
+            C = self.COEFFS[imt]
+            # Get ground motion on reference rock
+            ln_y_ref = get_ln_y_ref(name, C, ctx)
+            y_ref = np.exp(ln_y_ref)
+            # Get the site amplification
+            # Get basin depth
+            dz1pt0 = _get_centered_z1pt0(name, ctx)
+            # for Z1.0 = 0.0 no deep soil correction is applied
+            dz1pt0[ctx.z1pt0 <= 0.0] = 0.0
+            f_z1pt0 = get_basin_depth_term(name, C, dz1pt0)
+            # Get linear amplification term
+            f_lin = get_linear_site_term(name, C, ctx)
+            # Get nonlinear amplification term
+            f_nl, f_nl_scaling = get_nonlinear_site_term(C, ctx, y_ref)
+            # Add on the site amplification
+            mean[m] = ln_y_ref + (f_lin + f_nl + f_z1pt0)
+            # Get standard deviations
+            sig[m], tau[m], phi[m] = get_stddevs(
+                name, C, ctx, ctx.mag, y_ref, f_nl_scaling)
 
     #: Coefficient tables are constructed from values in tables 1 - 5
-
     COEFFS = CoeffsTable(sa_damping=5, table="""\
 IMT     c1      c1a     c1b     c1c     c1d     cn      cm    c2      c3    c4     c4a  crb   c5      chm     c6      c7      c7b     c8     c8a    c8b       c9     c9a    c9b     c11      c11b        cg1        cg2       cg3     phi1       phi2      phi3     phi4     phi5   phi6  gjpit  gwn      phi1jp  phi5jp   phi6jp     tau1    tau2    sig1    sig2    sig3    sig2jp
 pga   -1.5065  0.165  -0.255  -0.165  0.255  16.0875  4.9993  1.06  1.9636  -2.1  -0.5  50  6.4551  3.0956  0.4908  0.0352   0.0462  0.     0.2695  0.4833  0.9228  0.1202  6.8607  0.      -0.4536    -0.007146  -0.006758  4.2542  -0.521   -0.1417   -0.00701   0.102151  0.     300  1.5817  0.7594  -0.6846  0.459    800.        0.4     0.26    0.4912  0.3762  0.8     0.4528
@@ -333,9 +503,6 @@ pgv    2.3549  0.165  -0.0626 -0.165  0.0626  3.3024  5.423   1.06  2.3152  -2.1
 10    -5.3461  0.0124 -0.1    -0.0124 0.1    1.5265   6.7415  1.06  2.9396  -2.1  -0.5  50  7.5818  3.838   0.45    0.0003  -0.0253  0.2154 0.2695  7.77    0.      0.1     6.5     0.      -0.1       -0.00095   -0.0037    3.623   -0.7053   0.       -0.001361  0.000515  0.33   300  1.7679  0.      -0.4068  1.464    800.        0.4612  0.3474  0.4426  0.4426  0.7     0.3755
 """)
 
-    CONSTANTS = {"c2": 1.06, "c4": -2.1, "c4a": -0.5, "crb": 50.0,
-                 "c8a": 0.2695, "c11": 0.0, "phi6": 300.0, "phi6jp": 800.0}
-
 
 class ChiouYoungs2014Japan(ChiouYoungs2014):
     """
@@ -343,67 +510,12 @@ class ChiouYoungs2014Japan(ChiouYoungs2014):
     Japan far-field distance attuation scaling and site model
     """
 
-    def get_far_field_distance_scaling(self, C, mag, rrup):
-        """
-        Returns the far-field distance scaling term - both magnitude and
-        distance
-        """
-        # Get the attenuation distance scaling
-        f_r = (self.CONSTANTS["c4a"] - self.CONSTANTS["c4"]) * np.log(
-            np.sqrt(rrup ** 2. + self.CONSTANTS["crb"] ** 2.))
-
-        # Get the magnitude dependent term
-        f_rm = (C["cg1"] +
-                (C["cg2"] / np.cosh(max(mag - C["cg3"], 0.0)))) * rrup
-        if (mag > 6.0) and (mag < 6.9):
-            # Apply adjustment factor for Japan
-            f_rm *= C["gjpit"]
-        return f_r + f_rm
-
-    def get_linear_site_term(self, C, sites):
-        """
-        Returns the linear site scaling term
-        """
-        return C["phi1jp"] * np.log(sites.vs30 / 1130).clip(-np.inf, 0.0)
-
-    def get_basin_depth_term(self, C, centered_z1pt0):
-        """
-        Returns the basin depth scaling
-        """
-        return C["phi5jp"] * (1.0 - np.exp(-centered_z1pt0 /
-                                           self.CONSTANTS["phi6jp"]))
-
-    def _get_centered_z1pt0(self, sites):
-        """
-        Get z1pt0 centered on the Vs30- dependent average z1pt0(m) for Japan
-        """
-        #: Japan
-        mean_z1pt0 = (-5.23 / 2.) * np.log(((sites.vs30 ** 2.) + 412.39 ** 2.)
-                                           / (1360 ** 2. + 412.39 ** 2.))
-        return sites.z1pt0 - np.exp(mean_z1pt0)
-
 
 class ChiouYoungs2014Italy(ChiouYoungs2014):
     """
     Adaption of the Chiou & Youngs (2014) GMPE for the the Italy far-field
     attenuation scaling, but assuming the California site amplification model
     """
-    def get_far_field_distance_scaling(self, C, mag, rrup):
-        """
-        Returns the far-field distance scaling term - both magnitude and
-        distance
-        """
-        # Get the attenuation distance scaling
-        f_r = (self.CONSTANTS["c4a"] - self.CONSTANTS["c4"]) * np.log(
-            np.sqrt(rrup ** 2. + self.CONSTANTS["crb"] ** 2.))
-
-        # Get the magnitude dependent term
-        f_rm = (C["cg1"] +
-                (C["cg2"] / np.cosh(max(mag - C["cg3"], 0.0)))) * rrup
-        if (mag > 6.0) and (mag < 6.9):
-            # Apply adjustment factor for Italy
-            f_rm *= C["gjpit"]
-        return f_r + f_rm
 
 
 class ChiouYoungs2014Wenchuan(ChiouYoungs2014):
@@ -414,20 +526,6 @@ class ChiouYoungs2014Wenchuan(ChiouYoungs2014):
     is calibrated only for the M7.9 Wenchuan earthquake, so application to
     other scenarios is at the user's own risk
     """
-    def get_far_field_distance_scaling(self, C, mag, rrup):
-        """
-        Returns the far-field distance scaling term - both magnitude and
-        distance
-        """
-        # Get the attenuation distance scaling
-        f_r = (self.CONSTANTS["c4a"] - self.CONSTANTS["c4"]) * np.log(
-            np.sqrt(rrup ** 2. + self.CONSTANTS["crb"] ** 2.))
-
-        # Get the magnitude dependent term
-        f_rm = (C["cg1"] +
-                (C["cg2"] / np.cosh(max(mag - C["cg3"], 0.0)))) * rrup
-        # Apply adjustment factor for Wenchuan
-        return f_r + (f_rm * C["gwn"])
 
 
 class ChiouYoungs2014PEER(ChiouYoungs2014):
@@ -436,25 +534,9 @@ class ChiouYoungs2014PEER(ChiouYoungs2014):
     tests. In this version the total standard deviation is fixed at 0.65
     """
     #: Only the total standars deviation is defined
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
-        const.StdDev.TOTAL,
-    ])
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {const.StdDev.TOTAL}
     #: The PEER tests requires only PGA
-    DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([
-        PGA,
-    ])
-
-    def get_stddevs(self, C, sites, mag, y_ref, f_nl_scaling, stddev_types):
-        """
-        Returns the standard deviation, which is fixed at 0.65 for every site
-        """
-        ret = []
-        for stddev_type in stddev_types:
-            assert stddev_type in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev_type == const.StdDev.TOTAL:
-                # Fix total sigma to 0.65
-                ret.append(0.65 * np.ones_like(sites.vs30))
-        return ret
+    DEFINED_FOR_INTENSITY_MEASURE_TYPES = {PGA}
 
 
 class ChiouYoungs2014NearFaultEffect(ChiouYoungs2014):
@@ -462,15 +544,33 @@ class ChiouYoungs2014NearFaultEffect(ChiouYoungs2014):
     This implements the Chiou & Youngs (2014) GMPE include the near fault
     effect prediction. In this version, we add the distance measure, rcdpp
     for directivity prediction.
-
     """
     #: Required distance measures are RRup, Rjb, Rx, and Rcdpp
-    REQUIRES_DISTANCES = set(('rrup', 'rjb', 'rx', 'rcdpp'))
+    REQUIRES_DISTANCES = {'rrup', 'rjb', 'rx', 'rcdpp'}
 
-    def _get_centered_cdpp(self, dists):
-        """
-        Get directivity prediction parameter centered on the avgerage
-        directivity prediction parameter.
 
+class ChiouYoungs2014ACME2019(ChiouYoungs2014):
+    """
+    Implements a modified version of the CY2014 GMM. Main changes:
+    - Hanging wall term excluded
+    - Centered Ztor = 0
+    - Centered Dpp = 0
+    """
+    adapted = True
+
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
-        return dists.rcdpp
+        See :meth:`superclass method
+        <.base.GroundShakingIntensityModel.compute>`
+        for spec of input and result values.
+        """
+        for m, imt in enumerate(imts):
+            C = self.COEFFS[imt]
+            # intensity on a reference soil is used for both mean
+            # and stddev calculations.
+            ln_y_ref = _get_ln_y_ref(self.__class__.__name__, ctx, C)
+            # exp1 and exp2 are parts of eq. 12 and eq. 13,
+            # calculate it once for both.
+            exp1 = np.exp(C['phi3'] * (ctx.vs30.clip(-np.inf, 1130) - 360))
+            exp2 = np.exp(C['phi3'] * (1130 - 360))
+            mean[m] = _get_mean(ctx, C, ln_y_ref, exp1, exp2)
