@@ -32,6 +32,33 @@ from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
 
 LOG10 = np.log(10)
 
+#: "coefficient e_1 = 0.5 was selected for all periods in the present
+#: study." (p. 881)
+CONSTS = {'e': 0.5}
+
+
+def _compute_mag_dist_terms(ctx, coeffs):
+    """
+    Compute equation (5) and implcitly equation (6):
+
+    ``log(pre) = c + a*M + b*X - log(X + d*10^(e*M)) + epsilon``
+    """
+
+    log_pre = coeffs['c'] + coeffs['a']*ctx.mag + coeffs['b']*ctx.rrup \
+        - np.log10(ctx.rrup + coeffs['d']*10**(coeffs['e']*ctx.mag))
+
+    return log_pre
+
+
+def _compute_site_amplification(ctx, coeffs):
+    """
+    Compute equation (8):
+
+    ``G = p*log(VS30) + q``
+    """
+
+    return coeffs['p']*np.log10(ctx.vs30) + coeffs['q']
+
 
 class Kanno2006Shallow(GMPE):
     # pylint: disable=too-few-public-methods
@@ -75,6 +102,8 @@ class Kanno2006Shallow(GMPE):
     #: standard deviation is tabulated.
     DEFINED_FOR_STANDARD_DEVIATION_TYPES = {const.StdDev.TOTAL}
 
+    DEFINED_FOR_REFERENCE_VELOCITY = 800
+
     #: "Coefficients p and q were derived by regression analysis on the
     #: residuals averaged at intervals of every 100 m/sec in AVS30." (p. 884)
     REQUIRES_SITES_PARAMETERS = {'vs30'}
@@ -88,11 +117,11 @@ class Kanno2006Shallow(GMPE):
     #: earthquakes for which the fault model is not available." (p. 880)
     REQUIRES_DISTANCES = {'rrup'}
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         # pylint: disable=too-many-arguments
         """
         See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+        <.base.GroundShakingIntensityModel.compute>`
         for specification of input and result values.
 
         Implements the following equations:
@@ -124,68 +153,33 @@ class Kanno2006Shallow(GMPE):
 
         Note finally that "log represents log_10 in the present study"
         (p. 880).
-
         """
+        for m, imt in enumerate(imts):
+            # merge coefficients
+            coeffs = CONSTS.copy()
+            cb = self.COEFFS_BASE[imt]
+            cs = self.COEFFS_SITE[imt]
+            for n in cb.dtype.names:
+                coeffs[n] = cb[n]
+            for n in cs.dtype.names:
+                coeffs[n] = cs[n]
 
-        # obtain coefficients for required intensity measure type (IMT)
-        coeffs = self.COEFFS_BASE[imt].copy()
-        coeffs.update(self.COEFFS_SITE[imt])
+            # compute bedrock motion, equation (5)
+            log_mean = _compute_mag_dist_terms(ctx, coeffs)
 
-        # obtain IMT-independent coefficients
-        coeffs.update(self.CONSTS)
+            # make site corrections, equation (9)
+            log_mean += _compute_site_amplification(ctx, coeffs)
 
-        # compute bedrock motion, equation (5)
-        log_mean = self._compute_mag_dist_terms(rup, dists, coeffs)
+            # retrieve standard deviations
+            log_stddev = coeffs['epsilon']
 
-        # make site corrections, equation (9)
-        log_mean += self._compute_site_amplification(sites, coeffs)
+            # convert from common to natural logarithm
+            mean[m] = log_mean * LOG10
+            sig[m] = log_stddev * LOG10
 
-        # retrieve standard deviations
-        log_stddevs = self._get_stddevs(coeffs, sites.vs30.size, stddev_types)
-
-        # convert from common to natural logarithm
-        ln_mean = log_mean*LOG10
-        ln_stddevs = np.array(log_stddevs)*LOG10
-
-        # convert accelerations from cm/s^2 to g
-        if not imt.name == "PGV":
-            ln_mean -= np.log(100*g)
-
-        return ln_mean, ln_stddevs
-
-    @classmethod
-    def _compute_mag_dist_terms(cls, rup, dists, coeffs):
-        """
-        Compute equation (5) and implcitly equation (6):
-
-        ``log(pre) = c + a*M + b*X - log(X + d*10^(e*M)) + epsilon``
-        """
-
-        log_pre = coeffs['c'] + coeffs['a']*rup.mag + coeffs['b']*dists.rrup \
-            - np.log10(dists.rrup + coeffs['d']*10**(coeffs['e']*rup.mag))
-
-        return log_pre
-
-    @classmethod
-    def _compute_site_amplification(cls, sites, coeffs):
-        """
-        Compute equation (8):
-
-        ``G = p*log(VS30) + q``
-        """
-
-        return coeffs['p']*np.log10(sites.vs30) + coeffs['q']
-
-    def _get_stddevs(self, coeffs, num_sites, stddev_types):
-        """
-        Only total error is reported so this is a simple lookup.
-        """
-        stddevs = []
-        for stddev_type in stddev_types:
-            assert stddev_type in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            stddevs.append(np.zeros(num_sites) + coeffs['epsilon'])
-
-        return stddevs
+            # convert accelerations from cm/s^2 to g
+            if not imt.string == "PGV":
+                mean[m] -= np.log(100*g)
 
     #: Coefficients obtained from author via personal communcation with
     #: slightly more precision than Table 3, p. 884.
@@ -277,12 +271,6 @@ class Kanno2006Shallow(GMPE):
       pgv -0.7057  1.7650
     """)
 
-    #: "coefficient e_1 = 0.5 was selected for all periods in the present
-    #: study." (p. 881)
-    CONSTS = {
-        'e': 0.5,
-    }
-
 
 class Kanno2006Deep(Kanno2006Shallow):
     # pylint: disable=too-few-public-methods
@@ -384,11 +372,11 @@ class Kanno2006Deep(Kanno2006Shallow):
 #    #: so results cannot be verified without a dataset from the authors.
 #    non_verified = True
 #
-#    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+#    def compute(self, ctx, imts, mean, sig, tau, phi):
 #        # pylint: disable=too-many-arguments
 #        """
 #        See :meth:`superclass method
-#        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+#        <.base.GroundShakingIntensityModel.compute>`
 #        for specification of input and result values.
 #
 #        Implements equation (10) on p. 883 which is a depth correction for
@@ -412,29 +400,29 @@ class Kanno2006Deep(Kanno2006Shallow):
 #        """
 #        # compute mean and standard deviations as per parent class
 #        parent = super()
-#        ln_mean, [ln_stddevs] = parent.get_mean_and_stddevs(
-#            sites, rup, dists, imt, stddev_types)
+#        ln_mean, [ln_stddevs] = parent.compute(
+#            ctx, imts, mean, sig, tau, phi)
 #
 #        # compute site corrections, equation (9)
 #        coeffs = self.COEFFS_NORTHEAST[imt].copy()
-#        log_amp = self._compute_depth_correction(sites, rup, coeffs)
+#        log_amp = self._compute_depth_correction(ctx, coeffs)
 #
 #        # convert correction factor from common to natural logarithm
 #        ln_mean += log_amp*np.log(10.0)
 #
 #        return ln_mean, [ln_stddevs]
 #
-#    def _compute_depth_correction(self, sites, rup, coeffs):
+#    def _compute_depth_correction(self, ctx, coeffs):
 #        """
 #        Compute equation (10):
 #
 #        ``A = log(obs/pre) = (alpha*R_tr + beta)*(D - 30)``
 #        """
 #
-#        r_trench = _get_min_distance_to_sub_trench(sites.lon, sites.lat)
+#        r_trench = _get_min_distance_to_sub_trench(ctx.lon, ctx.lat)
 #
 #        log_amp = (coeffs['alpha']*r_trench + coeffs['beta']) * \
-#            (rup.hypo_depth - self.REF_DEPTH_KM)
+#            (ctx.hypo_depth - self.REF_DEPTH_KM)
 #
 #        return log_amp
 #
