@@ -33,8 +33,8 @@ logic tree file is as in this example::
               </logicTreeBranch>
 """
 import inspect
-import numpy
-from openquake.hazardlib import const
+import numpy as np
+from openquake.hazardlib import const, contexts
 from openquake.hazardlib.gsim.base import GMPE, registry
 
 
@@ -97,20 +97,22 @@ class AvgGMPE(GMPE):
         # gsims are all the same, then the AvgGMPE should use the same
         if all(d == def_for_stddevs[0] for d in def_for_stddevs[1:]):
             self.DEFINED_FOR_STANDARD_DEVIATION_TYPES = def_for_stddevs[0]
-        self.weights = numpy.array(weights)
+        self.weights = np.array(weights)
 
-    def get_mean_and_stddevs(self, sctx, rctx, dctx, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         Call the underlying GMPEs and return the weighted mean and stddev
         """
-        means = []
-        std2 = [[] for _ in stddev_types]
-        for g, gsim in enumerate(self.gsims):
-            mean, stddevs = gsim.get_mean_and_stddevs(
-                sctx, rctx, dctx, imt, stddev_types)
-            means.append(mean)
-            for s, stddev in enumerate(stddevs):
-                std2[s].append(stddev**2)
-        mean = self.weights @ means
-        stddevs = [numpy.sqrt(self.weights @ s) for s in std2]
-        return mean, stddevs
+        outs = contexts.get_mean_stds(self.gsims, ctx, imts, const.StdDev.ALL)
+        # shape (G, O, M, N)
+        G = len(outs)
+        M, N = outs[0].shape[1:]
+        data = np.zeros((G, 4, M, N))
+        for i, out in enumerate(outs):
+            data[i, 0] = out[0]
+            for s in range(len(out) - 1):
+                data[i, 1 + s] = out[1 + s] ** 2
+        mean[:] = np.average(data[:, 0], 0, self.weights)
+        sig[:] = np.sqrt(np.average(data[:, 1], 0, self.weights))
+        tau[:] = np.sqrt(np.average(data[:, 2], 0, self.weights))
+        phi[:] = np.sqrt(np.average(data[:, 3], 0, self.weights))
