@@ -22,7 +22,40 @@ Module :mod:`openquake.hazardlib.mgmpe.split_sigma_gmpe` implements
 
 import numpy as np
 from openquake.hazardlib.gsim.base import GMPE, registry
-from openquake.hazardlib import const
+from openquake.hazardlib import const, contexts
+
+
+def _get_stddvs(between_absolute, within_absolute, total):
+    """
+    This computes the between and within event std given the total std.
+
+    :param total:
+        A 1D :class:`numpy.ndarray` with the total standard deviation
+        values.
+    :param stds_types:
+        A list with the type of std requested.
+    :return:
+        A list of 3 arrays
+    """
+    # Compute missing standard deviations
+    new_stds = {'total': total}
+    if np.isscalar(between_absolute):
+        between = np.ones_like(total) * between_absolute
+        if np.any(total - between < 0):
+            raise ValueError('Between event std larger than total')
+        within = np.sqrt(total**2 - between**2)
+        new_stds['between'] = between
+        new_stds['within'] = within
+    elif np.isscalar(within_absolute):
+        within = np.ones_like(total) * within_absolute
+        if np.any(total - within < 0):
+            raise ValueError('Within event std larger than total')
+        between = np.sqrt(total**2 - within**2)
+        new_stds['between'] = between
+        new_stds['within'] = within
+    else:
+        pass
+    return [new_stds['total'], new_stds['between'], new_stds['within']]
 
 
 class SplitSigmaGMPE(GMPE):
@@ -58,64 +91,22 @@ class SplitSigmaGMPE(GMPE):
         self.between_absolute = between_absolute
         self.within_absolute = within_absolute
 
-        # Set the supported stds
-        self.DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
+        # Set the supported stds; essential!
+        self.DEFINED_FOR_STANDARD_DEVIATION_TYPES = {
             const.StdDev.TOTAL,
             const.StdDev.INTER_EVENT,
-            const.StdDev.INTRA_EVENT
-        ])
+            const.StdDev.INTRA_EVENT}
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stds_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+        <.base.GroundShakingIntensityModel.compute>`
         for spec of input and result values.
         """
         # compute mean and standard deviation
-        stds_total_type = [const.StdDev.TOTAL]
-        mean, std_total = self.gmpe.get_mean_and_stddevs(
-            sites, rup, dists, imt, stds_total_type)
-        stddvs = self._get_stddvs(np.array(std_total[0]), stds_types)
-        return mean, stddvs
-
-    def _get_stddvs(self, total, stds_types):
-        """
-        This computes the between and within event std given the total std.
-
-        :param total:
-            A 1D :class:`numpy.ndarray` with the total standard deviation
-            values.
-        :param stds_types:
-            A list with the type of std requested.
-        :return:
-            A list of 1D :class:`numpy.ndarray`
-        """
-        # Compute missing standard deviations
-        new_stds = {'total': total}
-        if np.isscalar(self.between_absolute):
-            between = np.ones_like(total) * self.between_absolute
-            if np.any(total - between < 0):
-                raise ValueError('Between event std larger than total')
-            within = np.sqrt(total**2 - between**2)
-            new_stds['between'] = between
-            new_stds['within'] = within
-        elif np.isscalar(self.within_absolute):
-            within = np.ones_like(total) * self.within_absolute
-            if np.any(total - within < 0):
-                raise ValueError('Within event std larger than total')
-            between = np.sqrt(total**2 - within**2)
-            new_stds['between'] = between
-            new_stds['within'] = within
-        else:
-            pass
-        stds = []
-        for key in stds_types:
-            if key == const.StdDev.TOTAL:
-                stds.append(new_stds['total'])
-            elif key == const.StdDev.INTER_EVENT:
-                stds.append(new_stds['between'])
-            elif key == const.StdDev.INTRA_EVENT:
-                stds.append(new_stds['within'])
-            else:
-                raise ValueError('Unsupported std type')
-        return stds
+        [out] = contexts.get_mean_stds(
+            [self.gmpe], ctx, imts, const.StdDev.ALL)
+        for m, imt in enumerate(imts):
+            mean[m] = out[0, m]
+            sig[m], tau[m], phi[m] = _get_stddvs(
+                self.between_absolute, self.within_absolute, out[1, m])
