@@ -22,10 +22,65 @@ Module exports class:`MorikawaFujiwara2013`
 
 import numpy as np
 
+from openquake.baselib.general import CallableDict
 from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
 from openquake.hazardlib import const
 from openquake.hazardlib.imt import PGA, PGV, SA, JMA
 
+CONSTS = {
+    "D0": 300.,
+    "e": 0.5,
+    "Mw01": 8.2,
+    "Mw1": 16.0}
+
+
+def _get_basin_term(C, z1pt4):
+    d0 = CONSTS["D0"]
+    tmp = np.ones_like(z1pt4) * C['Dlmin']
+    return C['pd'] * np.log10(np.maximum(tmp, z1pt4)/d0)
+
+
+def _get_intensity_correction_term(C, region, xvf, focal_depth):
+    if region == 'NE':
+        gamma = C['gNE']
+    elif region == 'SW':
+        gamma = C['gEW']
+    elif region is None:
+        gamma = 0.
+    else:
+        raise ValueError('Unsupported region')
+    return (gamma * np.minimum(xvf, 75.0) *
+            np.maximum(focal_depth-30., 0.))
+
+
+_get_magnitude_term = CallableDict()
+
+
+@_get_magnitude_term.add(const.TRT.ACTIVE_SHALLOW_CRUST)
+def _get_magnitude_term_1(trt, region, C, rrup, mw1prime, mw1, rhypo):
+    return (C['a'] * (mw1prime - mw1)**2 + C['b1'] * rrup + C['c1'] -
+            np.log10(rrup + C['d'] * 10.**(CONSTS['e'] * mw1prime)))
+
+
+@_get_magnitude_term.add(const.TRT.SUBDUCTION_INTERFACE)
+def _get_magnitude_term_2(trt, region, C, rrup, mw1prime, mw1, rhypo):
+    return (C['a'] * (mw1prime - mw1)**2 + C['b2'] * rrup + C['c2'] -
+            np.log10(rrup + C['d'] * 10.**(CONSTS['e']*mw1prime)))
+
+
+@_get_magnitude_term.add(const.TRT.SUBDUCTION_INTRASLAB)
+def _get_magnitude_term_3(trt, region, C, rrup, mw1prime, mw1, rhypo):
+    tmp = (C['a'] * (mw1prime - mw1)**2 + C['b3'] * rrup + C['c3'] -
+           np.log10(rrup + C['d'] * 10.**(CONSTS['e']*mw1prime)))
+    if region == "SW":
+        tmp[rhypo < 80] += C['PH']
+    return tmp
+
+
+def _get_shallow_amplification_term(C, vs30):
+    tmp = np.ones_like(vs30) * C['Vsmax']
+    return C['ps'] * np.log10(np.minimum(tmp, vs30)/C['V0'])
+
 
 class MorikawaFujiwara2013Crustal(GMPE):
     """
@@ -39,12 +94,7 @@ class MorikawaFujiwara2013Crustal(GMPE):
 
     #: Supported intensity measure types are spectral acceleration,
     #: peak ground velocity and peak ground acceleration
-    DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([
-        PGA,
-        PGV,
-        SA,
-        JMA
-    ])
+    DEFINED_FOR_INTENSITY_MEASURE_TYPES = {PGA, PGV, SA, JMA}
 
     #: Supported intensity measure component is orientation-independent
     #: measure :attr:`~openquake.hazardlib.const.IMC.RotD50`
@@ -52,9 +102,7 @@ class MorikawaFujiwara2013Crustal(GMPE):
 
     #: Supported standard deviation types are inter-event, intra-event
     #: and total, see equation 2, pag 106.
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
-        const.StdDev.TOTAL,
-    ])
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {const.StdDev.TOTAL}
 
     #: Required site parameters are:
     #: - Vs30 - time averaged shear-wave velocity [m/s]
@@ -68,66 +116,32 @@ class MorikawaFujiwara2013Crustal(GMPE):
     #: Required distance measure is Rrup [km]
     REQUIRES_DISTANCES = {'rrup'}
 
-    def _set_params(self):
-        self.region = None
-        self.model = 'model1'
+    region = None
+    model = 'model1'
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
-        self._set_params()
-        C = self.COEFFS[imt]
+    def compute(self, ctx: np.recarray, imts, mean, sig, tau, phi):
+        trt = self.DEFINED_FOR_TECTONIC_REGION_TYPE
+        mw01 = CONSTS["Mw01"]
+        mw1 = CONSTS["Mw1"]
+        mw1prime = np.array(ctx.mag)
+        mw1prime[ctx.mag >= mw01] = mw01
+        for m, imt in enumerate(imts):
+            C = self.COEFFS[imt]
+            if self.model == 'model1':
+                mag_term = _get_magnitude_term(
+                    trt, self.region, C, ctx.rrup, mw1prime, mw1,
+                    ctx.hypo_depth)
+            else:
+                msg = "Model not supported"
+                raise ValueError(msg)
 
-        mw01 = self.CONSTS["Mw01"]
-        mw1 = self.CONSTS["Mw1"]
-        mw1prime = np.min([rup.mag, mw01])
-
-        if self.model == 'model1':
-            mag_term = self._get_magnitude_term(C, dists.rrup, mw1prime, mw1,
-                                                rup.hypo_depth)
-        else:
-            msg = "Model not supported"
-            raise ValueError(msg)
-
-        mean = (mag_term + self._get_basin_term(C, sites.z1pt4) +
-                self._get_shallow_amplification_term(C, sites.vs30) +
-                self._get_intensity_correction_term(C, self.region, sites.xvf,
-                rup.hypo_depth))
-
-        stddevs = self._compute_stddevs(C, len(sites.vs30), stddev_types)
-        mean = np.log(10**mean/980.665)
-        return mean, stddevs
-
-    def _compute_stddevs(self, C, num_sites, stddev_types):
-        """ Return total standard deviation (converted to base e) """
-        stddevs = []
-        for _ in stddev_types:
-            stddevs.append(np.zeros(num_sites) + C['sigma'] * np.log(10))
-        return stddevs
-
-    def _get_magnitude_term(self, C, rrup, mw1prime, mw1, rhypo):
-        return (C['a']*(mw1prime - mw1)**2 + C['b1'] * rrup + C['c1'] -
-                np.log10(rrup + C['d'] * 10.**(self.CONSTS['e']*mw1prime)))
-
-    def _get_basin_term(self, C, z1pt4):
-        d0 = self.CONSTS["D0"]
-        tmp = np.ones_like(z1pt4) * C['Dlmin']
-        return C['pd'] * np.log10(np.maximum(tmp, z1pt4)/d0)
-
-    def _get_shallow_amplification_term(self, C, vs30):
-        tmp = np.ones_like(vs30) * C['Vsmax']
-        return C['ps'] * np.log10(np.minimum(tmp, vs30)/C['V0'])
-
-    def _get_intensity_correction_term(self, C, region, xvf, focal_depth):
-        if region == 'NE':
-            gamma = C['gNE']
-        elif region == 'SW':
-            gamma = C['gEW']
-        elif region is None:
-            gamma = 0.
-        else:
-            raise ValueError('Unsupported region')
-        return (gamma * np.minimum(xvf, 75.0) *
-                np.maximum(focal_depth-30., 0.))
+            mean[m] = (mag_term + _get_basin_term(C, ctx.z1pt4) +
+                       _get_shallow_amplification_term(C, ctx.vs30) +
+                       _get_intensity_correction_term(
+                           C, self.region, ctx.xvf, ctx.hypo_depth))
 
+            mean[m] = np.log(10**mean[m] / 980.665)
+            sig[m] = C['sigma'] * np.log(10)
 
     COEFFS = CoeffsTable(sa_damping=5, table="""\
   IMT       a        b1        b2        b3      c1      c2      c3         d        pd  Dlmin        ps   Vsmax   V0       gNE       gEW      PH   sigma
@@ -183,83 +197,44 @@ class MorikawaFujiwara2013Crustal(GMPE):
    10 -0.0427 -0.000100 -0.001179 -0.001498  5.9820  5.7595  5.9869  0.003828  0.348396  155.0 -0.298398   650.0  350  0.000015 -0.000012 -0.1665  0.3007
 """)
 
-    CONSTS = {
-        "D0": 300.,
-        "e": 0.5,
-        "Mw01": 8.2,
-        "Mw1": 16.0}
-
 
 class MorikawaFujiwara2013SubInterface(MorikawaFujiwara2013Crustal):
-
     #: Supported tectonic region type is active shallow crust
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTERFACE
-
-    def _set_params(self):
-        self.region = None
-        self.model = 'model1'
-
-    def _get_magnitude_term(self, C, rrup, mw1prime, mw1, rhypo):
-        return (C['a']*(mw1prime - mw1)**2 + C['b2'] * rrup + C['c2'] -
-                np.log10(rrup + C['d'] * 10.**(self.CONSTS['e']*mw1prime)))
+    region = None
+    model = 'model1'
 
 
 class MorikawaFujiwara2013SubInterfaceNE(MorikawaFujiwara2013SubInterface):
-
     #: Supported tectonic region type is active shallow crust
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTERFACE
-
-    def _set_params(self):
-        self.region = 'NE'
-        self.model = 'model1'
+    region = 'NE'
+    model = 'model1'
 
 
 class MorikawaFujiwara2013SubInterfaceSW(MorikawaFujiwara2013SubInterface):
-
     #: Supported tectonic region type is active shallow crust
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTERFACE
-
-    def _set_params(self):
-        self.region = 'SW'
-        self.model = 'model1'
+    region = 'SW'
+    model = 'model1'
 
 
 class MorikawaFujiwara2013SubSlab(MorikawaFujiwara2013Crustal):
-
     #: Supported tectonic region type is active shallow crust
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTRASLAB
-
-    def _set_params(self):
-        self.region = None
-        self.model = 'model1'
-
-    def _get_magnitude_term(self, C, rrup, mw1prime, mw1, rhypo):
-        return (C['a']*(mw1prime - mw1)**2 + C['b3'] * rrup + C['c3'] -
-                np.log10(rrup + C['d'] * 10.**(self.CONSTS['e']*mw1prime)))
+    region = None
+    model = 'model1'
 
 
 class MorikawaFujiwara2013SubSlabNE(MorikawaFujiwara2013SubSlab):
-
     #: Supported tectonic region type is active shallow crust
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTRASLAB
-
-    def _set_params(self):
-        self.region = 'NE'
-        self.model = 'model1'
+    region = 'NE'
+    model = 'model1'
 
 
 class MorikawaFujiwara2013SubSlabSW(MorikawaFujiwara2013SubSlabNE):
-
     #: Supported tectonic region type is active shallow crust
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTRASLAB
-
-    def _set_params(self):
-        self.region = 'SW'
-        self.model = 'model1'
-
-    def _get_magnitude_term(self, C, rrup, mw1prime, mw1, rhypo):
-        tmp = (C['a']*(mw1prime - mw1)**2 + C['b3'] * rrup + C['c3'] -
-               np.log10(rrup + C['d'] * 10.**(self.CONSTS['e']*mw1prime)))
-        if rhypo < 80:
-            tmp += C['PH']
-        return tmp
+    region = 'SW'
+    model = 'model1'
