@@ -154,6 +154,49 @@ REQUIRES_DISTANCES
 '''.split()
 
 
+def get_alatik_youngs_sigma_mu(mag, rake, imt):
+    """
+    Implements the statistical uncertainty model of Al Atik & Youngs (2014)
+    given in equations 9 to 11 in the manuscript.
+    """
+    if str(imt) == "PGA":
+        period = 0.01
+    elif str(imt).startswith("SA"):
+        period = imt.period
+    else:
+        raise ValueError("Al Atik & Youngs (2014) Model not supported "
+                         "for %s" % str(imt))
+    if mag >= 7.0:
+        sigma_mu = 0.056 * (mag - 7.0) + 0.083
+    else:
+        sigma_mu = 0.083
+    if period >= 1.0:
+        sigma_mu += (0.0171 * np.log(period))
+    if rake >= -135. and rake <= -45.:
+        # Normal faulting case
+        sigma_mu += 0.038
+    return sigma_mu
+
+
+def get_stddevs(branch, phi_ds2s, homoskedastic_sigma, imt, mag):
+    """
+    Returns the standard deviations as described in Figure 10 and
+    section 4 of Tromans et al. (2019).
+    """
+    if homoskedastic_sigma:
+        # Homoskedastic sigma branch
+        tau = HOMOSKEDASTIC_TAU[branch](imt)
+        phi = HOMOSKEDASTIC_PHI[branch](imt)
+    else:
+        # Heteroskedastic sigma branch
+        tau = HETEROSKEDASTIC_TAU[branch](imt, mag)
+        phi = HETEROSKEDASTIC_PHI[branch](imt, mag)
+    # Add on the delta phi_d2s
+    if phi_ds2s:
+        phi = np.sqrt(phi ** 2 + DELTA_PHI_S2S[imt]["dfs2s"] ** 2)
+    return [np.sqrt(tau ** 2 + phi ** 2), tau, phi]
+
+
 class TromansEtAl2019(GMPE):
     """
     Implements a modifiable GMPE to apply the standard deviation model and
@@ -203,11 +246,8 @@ class TromansEtAl2019(GMPE):
 
     #: Supported standard deviation types are inter-event, intra-event
     #: and total
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
-        const.StdDev.TOTAL,
-        const.StdDev.INTER_EVENT,
-        const.StdDev.INTRA_EVENT
-    ])
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {
+        const.StdDev.TOTAL, const.StdDev.INTER_EVENT, const.StdDev.INTRA_EVENT}
 
     #: Required site parameter is not set
     REQUIRES_SITES_PARAMETERS = set()
@@ -219,7 +259,7 @@ class TromansEtAl2019(GMPE):
     #: Required distance measure will be set by the GMPE
     REQUIRES_DISTANCES = set()
 
-    def __init__(self, gmpe_name, branch="central",
+    def __init__(self, gmpe_name, branch="central", sigma_mu_epsilon=0.0,
                  homoskedastic_sigma=False,  scaling_factor=None,
                  vskappa=None, phi_ds2s=True, **kwargs):
         super().__init__(gmpe_name=gmpe_name, branch=branch,
@@ -244,58 +284,37 @@ class TromansEtAl2019(GMPE):
             for key in vskappa:
                 in_vskappa[from_string(key)] = {"vskappa":
                                                 np.log(vskappa[key])}
-            self.vskappa = CoeffsTable(sa_damping=5, table=in_vskappa)
+            self.vskappa = CoeffsTable.fromdict(in_vskappa)
         else:
             self.vskappa = None
         self.branch = branch
+        self.sigma_mu_epsilon = sigma_mu_epsilon
         self.homoskedastic_sigma = homoskedastic_sigma
         self.phi_ds2s = phi_ds2s
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         Returns the mean and standard deviations applying, where specified,
         scalar adjustment and vs-kappa adjustment to the mean from the
-        original GMPE
+        original GMPE.
         """
-        # Retrieve the mean values from the GMPE - cannot avoid calling
-        # the stddevs part, but no stddevs will be returned.
-        mean = self.gmpe.get_mean_and_stddevs(sites, rup, dists, imt, [])[0]
+        # Retrieve the mean values from the GMPE
+        self.gmpe.compute(ctx, imts, mean, sig, tau, phi)
         # Apply scaling factor
         if self.scaling_factor:
             mean += self.scaling_factor
-        # Apply vs-kappa correction
-        if self.vskappa:
-            mean += self.vskappa[imt]["vskappa"]
-        # Get stddevs
-        stddevs = self.get_stddevs(stddev_types, imt, rup.mag, mean.shape)
-        return mean, stddevs
-
-    def get_stddevs(self, stddev_types, imt, mag, nsites):
-        """
-        Returns the standard deviations as described in Figure 10 and
-        section 4 of Tromans et al. (2019).
-        """
-        if self.homoskedastic_sigma:
-            # Homoskedastic sigma branch
-            tau = np.zeros(nsites) + HOMOSKEDASTIC_TAU[self.branch](imt)
-            phi = np.zeros(nsites) + HOMOSKEDASTIC_PHI[self.branch](imt)
-        else:
-            # Heteroskedastic sigma branch
-            tau = np.zeros(nsites) + HETEROSKEDASTIC_TAU[self.branch](imt, mag)
-            phi = np.zeros(nsites) + HETEROSKEDASTIC_PHI[self.branch](imt, mag)
-        # Add on the delta phi_d2s
-        if self.phi_ds2s:
-            phi = np.sqrt(phi ** 2. + DELTA_PHI_S2S[imt]["dfs2s"] ** 2.)
-        stddevs = []
-        for stddev_type in stddev_types:
-            assert stddev_type in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev_type == const.StdDev.TOTAL:
-                stddevs.append(np.sqrt(tau ** 2. + phi ** 2.))
-            elif stddev_type == const.StdDev.INTRA_EVENT:
-                stddevs.append(phi)
-            elif stddev_type == const.StdDev.INTER_EVENT:
-                stddevs.append(tau)
-        return stddevs
+        for m, imt in enumerate(imts):
+            # Apply sigma_mu epsilon factor
+            if self.sigma_mu_epsilon:
+                mean += (self.sigma_mu_epsilon *
+                         get_alatik_youngs_sigma_mu(ctx.mag, ctx.rake, imt))
+            # Apply vs-kappa correction
+            if self.vskappa:
+                mean += self.vskappa[imt]["vskappa"]
+            # Get stddevs
+            sig[m], tau[m], phi[m] = get_stddevs(
+                self.branch, self.phi_ds2s, self.homoskedastic_sigma,
+                imt, ctx.mag)
 
 
 class TromansEtAl2019SigmaMu(TromansEtAl2019):
@@ -310,59 +329,3 @@ class TromansEtAl2019SigmaMu(TromansEtAl2019):
     #: Required rupture parameters are magnitude and style of faulting, others
     #: will be taken from the GMPE
     REQUIRES_RUPTURE_PARAMETERS = {'mag', 'rake'}
-
-    def __init__(self, gmpe_name, branch="central", sigma_mu_epsilon=0.0,
-                 homoskedastic_sigma=False,  scaling_factor=None,
-                 vskappa=None, **kwargs):
-        super().__init__(gmpe_name=gmpe_name, branch=branch,
-                         homoskedastic_sigma=homoskedastic_sigma,
-                         scaling_factor=scaling_factor, vskappa=vskappa,
-                         **kwargs)
-        self.sigma_mu_epsilon = sigma_mu_epsilon
-
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
-        """
-        Returns the mean and standard deviations applying, where specified,
-        scalar adjustment and vs-kappa adjustment to the mean from the
-        original GMPE
-        """
-        # Retrieve the mean values from the GMPE - cannot avoid calling
-        # the stddevs part, but no stddevs will be returned.
-        mean = self.gmpe.get_mean_and_stddevs(sites, rup, dists, imt, [])[0]
-        # Apply scaling factor
-        if self.scaling_factor:
-            mean += self.scaling_factor
-        # Apply sigma_mu epsilon factor
-        if self.sigma_mu_epsilon:
-            mean += (self.sigma_mu_epsilon *
-                     self.get_alatik_youngs_sigma_mu(rup.mag, rup.rake, imt))
-        # Apply vs-kappa correction
-        if self.vskappa:
-            mean += self.vskappa[imt]["vskappa"]
-        # Get stddevs
-        stddevs = self.get_stddevs(stddev_types, imt, rup.mag, mean.shape)
-        return mean, stddevs
-
-    @staticmethod
-    def get_alatik_youngs_sigma_mu(mag, rake, imt):
-        """
-        Implements the statistical uncertainty model of Al Atik & Youngs (2014)
-        given in equations 9 to 11 in the manuscript.
-        """
-        if str(imt) == "PGA":
-            period = 0.01
-        elif str(imt).startswith("SA"):
-            period = imt.period
-        else:
-            raise ValueError("Al Atik & Youngs (2014) Model not supported "
-                             "for %s" % str(imt))
-        if mag >= 7.0:
-            sigma_mu = 0.056 * (mag - 7.0) + 0.083
-        else:
-            sigma_mu = 0.083
-        if period >= 1.0:
-            sigma_mu += (0.0171 * np.log(period))
-        if rake >= -135. and rake <= -45.:
-            # Normal faulting case
-            sigma_mu += 0.038
-        return sigma_mu
