@@ -22,9 +22,10 @@ import os
 import numpy as np
 from scipy.interpolate import interp1d
 from openquake.hazardlib import const
-from openquake.hazardlib.imt import PGA, SA
-from openquake.hazardlib.gsim.base import CoeffsTable, gsim_aliases
-from openquake.hazardlib.gsim.nga_east import ITPL, NGAEastGMPE
+from openquake.hazardlib.gsim.base import CoeffsTable, add_alias
+from openquake.hazardlib.gsim.nga_east import (
+    ITPL, NGAEastGMPE, get_mean_amp, get_site_amplification_sigma)
+from openquake.hazardlib.gsim.gmpe_table import _get_mean
 
 # Coefficients for EPRI sigma model taken from Table 5.5 of Goulet et al.
 # (2017)
@@ -148,6 +149,72 @@ def get_stewart_2019_phis2s(imt, vs30):
     return phis2s
 
 
+@_get_mean.add("usgs")
+def _get_mean(kind, distance_type, data, dctx, ctx):
+    """
+    Returns the mean intensity measure level from the tables applying
+    log-log interpolation of the IML with distance (contrast with the
+    linear interpolation applied in usual GMPE tables)
+    :param data:
+        The intensity measure level vector for the given magnitude and IMT
+    :param key:
+        The distance type
+    :param distances:
+        The distance vector for the given magnitude and IMT
+    """
+    # For values outside of the interpolation range use -999. to ensure
+    # value is identifiable and outside of potential real values
+    # For extremely short distance (rrup = 0) use an arbitrarily small
+    # distance measure (1.0E-5 used by US NSHMP code)
+    ctx[ctx < 1.0E-5] = 1.0E-5
+    interpolator_mean = interp1d(np.log10(ctx), np.log(data),
+                                 bounds_error=False,
+                                 fill_value=-999.)
+    mean = np.exp(interpolator_mean(np.log10(getattr(dctx, distance_type))))
+    # For those distances less than or equal to the shortest distance
+    # extrapolate the shortest distance value
+    mean[getattr(dctx, distance_type) <= ctx[0]] = data[0]
+    # For those distances significantly greater than the furthest distance
+    # set to 1E-20.
+    mean[getattr(dctx, distance_type) > (ctx[-1] + 1.0E-3)] = 1E-20
+    # If any distance is between the final distance and a margin of 0.001
+    # km then assign to smallest distance
+    mean[mean < -1.] = data[-1]
+    return mean
+
+
+def _get_stddevs(sigma_model, ctx, imt):
+    """
+    Returns the standard deviations according to the choice of aleatory
+    uncertainty model. Note that for compatibility with the US NSHMP
+    code a weighted sum of the two aleatory uncertainty models is used,
+    with the EPRI model assigned a weight of 0.8 and the PANEL model 0.2.
+    """
+    if sigma_model in ("EPRI", "COLLAPSED"):
+        # EPRI recommended aleatory uncertainty model
+        tau_epri, phi_epri = get_epri_tau_phi(imt, ctx.mag)
+    if sigma_model in ("PANEL", "COLLAPSED"):
+        # Panel recommended model
+        tau_panel, phi0_panel = get_panel_tau_phi(imt, ctx.mag)
+        phis2s = get_stewart_2019_phis2s(imt, ctx.vs30)
+        phi_panel = np.sqrt(phi0_panel ** 2. + phis2s ** 2.)
+    if sigma_model == "EPRI":
+        tau = tau_epri
+        phi = phi_epri
+        sigma = np.sqrt(tau ** 2. + phi ** 2.)
+    elif sigma_model == "PANEL":
+        tau = tau_panel
+        phi = phi_panel
+        sigma = np.sqrt(tau ** 2. + phi ** 2.)
+    else:
+        # Get the weighted sum of the two models
+        sigma_epri = np.sqrt(tau_epri ** 2. + phi_epri ** 2.)
+        sigma_panel = np.sqrt(tau_panel ** 2. + phi_panel ** 2.)
+        sigma = 0.8 * sigma_epri + 0.2 * sigma_panel
+        tau, phi = 0., 0.
+    return [sigma, tau, phi]
+
+
 class NGAEastUSGSGMPE(NGAEastGMPE):
     """
     For the "core" NGA East set the table is provided in the code in a
@@ -159,6 +226,7 @@ class NGAEastUSGSGMPE(NGAEastGMPE):
                                                 const.StdDev.INTRA_EVENT))
     gmpe_table = ""
     PATH = os.path.join(os.path.dirname(__file__), "usgs_nga_east_tables")
+    kind = "usgs"
 
     def __init__(self, **kwargs):
         self.sigma_model = kwargs.get("sigma_model", "COLLAPSED")
@@ -173,132 +241,34 @@ class NGAEastUSGSGMPE(NGAEastGMPE):
                 set((const.StdDev.TOTAL,))
         super().__init__(**kwargs)
 
-    def _setup_standard_deviations(self, fle):
-        # Not needed here
-        pass
-
-    def get_mean_and_stddevs(self, sctx, rctx, dctx, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         Returns the mean and standard deviations
         """
-        # Get the PGA on the reference rock condition
-        if PGA in self.DEFINED_FOR_INTENSITY_MEASURE_TYPES:
-            rock_imt = PGA()
-        else:
-            rock_imt = SA(0.01)
-        pga_r = self.get_hard_rock_mean(rctx, dctx, rock_imt, stddev_types)
+        for m, imt in enumerate(imts):
+            imean, site_amp, pga_r = get_mean_amp(self, ctx, imt)
 
-        # Get the desired spectral acceleration on rock
-        if not str(imt) == "PGA":
-            # Calculate the ground motion at required spectral period for
-            # the reference rock
-            imean = self.get_hard_rock_mean(rctx, dctx, imt, stddev_types)
-        else:
-            # Avoid re-calculating PGA if that was already done!
-            imean = np.copy(pga_r)
+            # Get the coefficients for the IMT
+            C_LIN = self.COEFFS_LINEAR[imt]
+            C_F760 = self.COEFFS_F760[imt]
+            C_NL = self.COEFFS_NONLINEAR[imt]
 
-        # Get the coefficients for the IMT
-        C_LIN = self.LINEAR_COEFFS[imt]
-        C_F760 = self.F760[imt]
-        C_NL = self.NONLINEAR_COEFFS[imt]
-
-        site_amp = self.get_site_amplification(imt, np.exp(pga_r), sctx)
-
-        # Get collapsed amplification model for -sigma, 0, +sigma with weights
-        # of 0.185, 0.63, 0.185 respectively
-        if self.epistemic_site:
-            f_rk = np.log((np.exp(pga_r) + C_NL["f3"]) / C_NL["f3"])
-            site_amp_sigma = self.get_site_amplification_sigma(
-                sctx, f_rk, C_LIN, C_F760, C_NL)
-            mean = np.log(
-                0.185 * (np.exp(imean + (site_amp - site_amp_sigma))) +
-                0.63 * (np.exp(imean + site_amp)) +
-                0.185 * (np.exp(imean + (site_amp + site_amp_sigma))))
-        else:
-            mean = imean + site_amp
-
-        # Get standard deviation model
-        nsites = getattr(dctx, self.distance_type).shape
-        stddevs = self.get_stddevs(rctx.mag, sctx.vs30, imt,
-                                   stddev_types, nsites)
-        return mean, stddevs
-
-    def _get_mean(self, data, dctx, dists):
-        """
-        Returns the mean intensity measure level from the tables applying
-        log-log interpolation of the IML with distance (contrast with the
-        linear interpolation applied in usual GMPE tables)
-        :param data:
-            The intensity measure level vector for the given magnitude and IMT
-        :param key:
-            The distance type
-        :param distances:
-            The distance vector for the given magnitude and IMT
-        """
-        # For values outside of the interpolation range use -999. to ensure
-        # value is identifiable and outside of potential real values
-        # For extremely short distance (rrup = 0) use an arbitrarily small
-        # distance measure (1.0E-5 used by US NSHMP code)
-        dists[dists < 1.0E-5] = 1.0E-5
-        interpolator_mean = interp1d(np.log10(dists), np.log(data),
-                                     bounds_error=False,
-                                     fill_value=-999.)
-        mean = np.exp(interpolator_mean(np.log10(getattr(dctx,
-                                                         self.distance_type))))
-        # For those distances less than or equal to the shortest distance
-        # extrapolate the shortest distance value
-        mean[getattr(dctx, self.distance_type) <= dists[0]] = data[0]
-        # For those distances significantly greater than the furthest distance
-        # set to 1E-20.
-        mean[getattr(dctx, self.distance_type) > (dists[-1] + 1.0E-3)] = 1E-20
-        # If any distance is between the final distance and a margin of 0.001
-        # km then assign to smallest distance
-        mean[mean < -1.] = data[-1]
-        return mean
-
-    def get_stddevs(self, mag, vs30, imt, stddev_types, num_sites):
-        """
-        Returns the standard deviations according to the choice of aleatory
-        uncertainty model. Note that for compatibility with the US NSHMP
-        code a weighted sum of the two aleatory uncertainty models is used,
-        with the EPRI model assigned a weight of 0.8 and the PANEL model 0.2.
-        """
-        if self.sigma_model in ("EPRI", "COLLAPSED"):
-            # EPRI recommended aleatory uncertainty model
-            tau_epri, phi_epri = get_epri_tau_phi(imt, mag)
-            tau_epri += np.zeros(num_sites)
-            phi_epri += np.zeros(num_sites)
-        if self.sigma_model in ("PANEL", "COLLAPSED"):
-            # Panel recommended model
-            tau_panel, phi0_panel = get_panel_tau_phi(imt, mag)
-            tau_panel += np.zeros(num_sites)
-            phis2s = get_stewart_2019_phis2s(imt, vs30)
-            phi_panel = np.sqrt(phi0_panel ** 2. + phis2s ** 2.)
-
-        if self.sigma_model == "EPRI":
-            tau = tau_epri
-            phi = phi_epri
-            sigma = np.sqrt(tau ** 2. + phi ** 2.)
-        elif self.sigma_model == "PANEL":
-            tau = tau_panel
-            phi = phi_panel
-            sigma = np.sqrt(tau ** 2. + phi ** 2.)
-        else:
-            # Get the weighted sum of the two models
-            sigma_epri = np.sqrt(tau_epri ** 2. + phi_epri ** 2.)
-            sigma_panel = np.sqrt(tau_panel ** 2. + phi_panel ** 2.)
-            sigma = 0.8 * sigma_epri + 0.2 * sigma_panel
-        stddevs = []
-        for stddev_type in stddev_types:
-            assert stddev_type in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev_type == const.StdDev.TOTAL:
-                stddevs.append(sigma)
-            elif stddev_type == const.StdDev.INTRA_EVENT:
-                stddevs.append(phi)
-            elif stddev_type == const.StdDev.INTER_EVENT:
-                stddevs.append(tau)
-        return stddevs
+            # Get collapsed amplification model for -sigma, 0, +sigma
+            # with weights of 0.185, 0.63, 0.185 respectively
+            if self.epistemic_site:
+                f_rk = np.log((np.exp(pga_r) + C_NL["f3"]) / C_NL["f3"])
+                site_amp_sigma = get_site_amplification_sigma(
+                    self, ctx, f_rk, C_LIN, C_F760, C_NL)
+                mean[m] = np.log(
+                    0.185 * (np.exp(imean - site_amp_sigma)) +
+                    0.63 * np.exp(imean) +
+                    0.185 * np.exp(imean + site_amp_sigma))
+            else:
+                mean[m] = imean
 
+            # Get standard deviation model
+            sig[m], tau[m], phi[m] = _get_stddevs(
+                self.sigma_model, ctx, imt)
 
 lines = '''\
 NGAEastUSGSSeedSP15 SP15
@@ -341,5 +311,4 @@ NGAEastUSGSSammons16 usgs_16
 NGAEastUSGSSammons17 usgs_17'''.splitlines()
 for line in lines:
     alias, key = line.split()
-    gsim_aliases[alias] = (f'[NGAEastUSGSGMPE]\n'
-                           f'gmpe_table="nga_east_{key}.hdf5"')
+    add_alias(alias, NGAEastUSGSGMPE, gmpe_table=f"nga_east_{key}.hdf5")
