@@ -30,6 +30,12 @@ from openquake.hazardlib.geo.mesh import Mesh
 
 U32LIMIT = 2 ** 32
 ampcode_dt = (numpy.string_, 4)
+param = dict(
+    vs30measured='reference_vs30_type',
+    vs30='reference_vs30_value',
+    z1pt0='reference_depth_to_1pt0km_per_sec',
+    z2pt5='reference_depth_to_2pt5km_per_sec',
+    backarc='reference_backarc')
 
 
 class Site(object):
@@ -56,6 +62,7 @@ class Site(object):
 
         :class:`Sites <Site>` are pickleable
     """
+
     def __init__(self, location, vs30=numpy.nan,
                  z1pt0=numpy.nan, z2pt5=numpy.nan, **extras):
         if not numpy.isnan(vs30) and vs30 <= 0:
@@ -124,6 +131,8 @@ site_param_dt = {
     'z1pt4': numpy.float64,
     'backarc': numpy.bool,
     'xvf': numpy.float64,
+    'soiltype': numpy.uint32,
+    'bas': numpy.bool,
 
     # Parameters for site amplification
     'ampcode': ampcode_dt,
@@ -187,7 +196,7 @@ class SiteCollection(object):
                     if item[0] not in ('lon', 'lat'))
 
     @classmethod
-    def from_shakemap(cls, shakemap_array):
+    def from_usgs_shakemap(cls, shakemap_array):
         """
         Build a site collection from a shakemap array
         """
@@ -246,10 +255,12 @@ class SiteCollection(object):
             self._set('vs30', sitemodel.reference_vs30_value)
             self._set('vs30measured',
                       sitemodel.reference_vs30_type == 'measured')
-            self._set('z1pt0', sitemodel.reference_depth_to_1pt0km_per_sec)
-            self._set('z2pt5', sitemodel.reference_depth_to_2pt5km_per_sec)
-            self._set('siteclass', sitemodel.reference_siteclass)
-            self._set('backarc', sitemodel.reference_backarc)
+            if 'z1pt0' in req_site_params:
+                self._set('z1pt0', sitemodel.reference_depth_to_1pt0km_per_sec)
+            if 'z2pt5' in req_site_params:
+                self._set('z2pt5', sitemodel.reference_depth_to_2pt5km_per_sec)
+            if 'backarc' in req_site_params:
+                self._set('backarc', sitemodel.reference_backarc)
         else:
             for name in sitemodel.dtype.names:
                 if name not in ('lon', 'lat'):
@@ -381,10 +392,19 @@ class SiteCollection(object):
         return not_equal(self.array, other.array)
 
     def __toh5__(self):
-        return self.array, {}
+        names = self.array.dtype.names
+        cols = ' '.join(names)
+        return {n: self.array[n] for n in names}, {'__pdcolumns__': cols}
 
-    def __fromh5__(self, array, attrs):
-        self.array = array
+    def __fromh5__(self, dic, attrs):
+        if isinstance(dic, dict):  # engine >= 3.11
+            params = attrs['__pdcolumns__'].split()
+            dtype = numpy.dtype([(p, site_param_dt[p]) for p in params])
+            self.array = numpy.zeros(len(dic['sids']), dtype)
+            for p in dic:
+                self.array[p] = dic[p][()]
+        else:  # old engine, dic is actually a structured array
+            self.array = dic
         self.complete = self
 
     @property
@@ -474,7 +494,7 @@ class SiteCollection(object):
         for name in ok:
             self._set(name, site_model[name])
         for name in set(self.array.dtype.names) - set(site_model.dtype.names):
-            if name in ('vs30measured', 'backarc'):
+            if name == 'vs30measured':
                 self._set(name, 0)  # default
                 # NB: by default reference_vs30_type == 'measured' is 1
                 # but vs30measured is 0 (the opposite!!)
