@@ -20,12 +20,25 @@ seismic sources.
 import abc
 import zlib
 import numpy
+from openquake.baselib import general
+from openquake.hazardlib import mfd
 from openquake.hazardlib.geo import Point
 from openquake.hazardlib.source.rupture import ParametricProbabilisticRupture
 
 EPS = .01  # used for src.nsites outside the maximum_distance
 
 
+def get_code2cls():
+    """
+    :returns: a dictionary source code -> source class
+    """
+    dic = {}
+    for cls in general.gen_subclasses(BaseSeismicSource):
+        if hasattr(cls, 'code'):
+            dic[cls.code] = cls
+    return dic
+
+
 class BaseSeismicSource(metaclass=abc.ABCMeta):
     """
     Base class representing a seismic source, that is a structure generating
@@ -39,7 +52,7 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
     :param tectonic_region_type:
         Source's tectonic regime. See :class:`openquake.hazardlib.const.TRT`.
     """
-    et_id = 0  # set by the engine
+    trt_smr = 0  # set by the engine
     nsites = 0  # set when filtering the source
     ngsims = 1
     min_mag = 0  # set in get_oqparams and CompositeSourceModel.filter
@@ -61,17 +74,19 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
         if not self.num_ruptures:
             self.num_ruptures = self.count_ruptures()
         w = self.num_ruptures * self.ngsims * (.1 if self.nsites == EPS else 1)
-        if not hasattr(self, 'nodal_plane_distribution'):  # not pointlike
-            w *= 10  # increase weight of non point sources
+        if hasattr(self, 'data'):  # nonparametric rupture
+            w *= 20  # increase weight 20 times
+        elif not hasattr(self, 'nodal_plane_distribution'):  # not pointlike
+            w *= 5  # increase weight of non point sources
         return w
 
     @property
-    def et_ids(self):
+    def trt_smrs(self):
         """
-        :returns: a list of source group IDs (usually of 1 element)
+        :returns: a list of integers (usually of 1 element)
         """
-        et_id = self.et_id
-        return [et_id] if isinstance(et_id, int) else et_id
+        trt_smr = self.trt_smr
+        return [trt_smr] if isinstance(trt_smr, int) else trt_smr
 
     def serial(self, ses_seed):
         """
@@ -83,7 +98,7 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
         self.source_id = source_id
         self.name = name
         self.tectonic_region_type = tectonic_region_type
-        self.et_id = -1  # set by the engine
+        self.trt_smr = -1  # set by the engine
         self.num_ruptures = 0  # set by the engine
         self.seed = None  # set by the engine
         self.min_mag = 0  # set by the SourceConverter
@@ -108,15 +123,15 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
     def sample_ruptures(self, eff_num_ses, ses_seed):
         """
         :param eff_num_ses: number of stochastic event sets * number of samples
-        :yields: triples (rupture, et_id, num_occurrences)
+        :yields: triples (rupture, trt_smr, num_occurrences)
         """
         seed = self.serial(ses_seed)
         numpy.random.seed(seed)
-        for et_id in self.et_ids:
+        for trt_smr in self.trt_smrs:
             for rup, num_occ in self._sample_ruptures(eff_num_ses):
                 rup.rup_id = seed
                 seed += 1
-                yield rup, et_id, num_occ
+                yield rup, trt_smr, num_occ
 
     def _sample_ruptures(self, eff_num_ses):
         tom = getattr(self, 'temporal_occurrence_model', None)
@@ -144,6 +159,8 @@ class BaseSeismicSource(metaclass=abc.ABCMeta):
         elif hasattr(self, 'source_file'):
             # unbound UCERFSource
             mags.add(numpy.nan)
+        elif hasattr(self, 'mags'):  # MultiFaultSource
+            mags.update(mag for mag in self.mags if mag >= self.min_mag)
         else:  # nonparametric
             for rup, pmf in self.data:
                 if rup.mag >= self.min_mag:
@@ -280,6 +297,7 @@ class ParametricSeismicSource(BaseSeismicSource, metaclass=abc.ABCMeta):
         If either rupture aspect ratio or rupture mesh spacing is not positive
         (if not None).
     """
+
     def __init__(self, source_id, name, tectonic_region_type, mfd,
                  rupture_mesh_spacing, magnitude_scaling_relationship,
                  rupture_aspect_ratio, temporal_occurrence_model):
@@ -335,7 +353,9 @@ class ParametricSeismicSource(BaseSeismicSource, metaclass=abc.ABCMeta):
 
     def get_one_rupture(self, ses_seed, rupture_mutex=False):
         """
-        Yields one random rupture from a source
+        Yields one random rupture from a source. IMPORTANT: this method
+        does not take into account the frequency of occurrence of the
+        ruptures
         """
         # The Mutex case is admitted only for non-parametric ruptures
         msg = 'Mutually exclusive ruptures are admitted only in case of'
@@ -354,3 +374,74 @@ class ParametricSeismicSource(BaseSeismicSource, metaclass=abc.ABCMeta):
                     rup.rup_id = self.rup_id
                 rup.idx = idx
                 return rup
+
+    def modify_set_msr(self, new_msr):
+        """
+        Updates the MSR originally assigned to the source
+
+        :param new_msr:
+            An instance of the :class:`openquake.hazardlib.scalerel.BaseMSR`
+        """
+        self.magnitude_scaling_relationship = new_msr
+
+    def modify_set_slip_rate(self, slip_rate: float):
+        """
+        Updates the slip rate assigned to the source
+
+        :param slip_rate:
+            The value of slip rate [mm/yr]
+        """
+        self.slip_rate = slip_rate
+
+    def modify_set_mmax_truncatedGR(self, mmax: float):
+        """
+        Updates the mmax assigned. This works on for parametric MFDs.s
+
+        :param mmax:
+            The value of the new maximum magnitude
+        """
+        # Check that the current src has a TruncatedGRMFD MFD
+        msg = 'This modification works only when the source MFD is a '
+        msg += 'TruncatedGRMFD'
+        assert self.mfd.__class__.__name__ == 'TruncatedGRMFD', msg
+        self.mfd.max_mag
+
+    def modify_recompute_mmax(self, epsilon: float = 0):
+        """
+        Updates the value of mmax using the msr and the area of the fault
+
+        :param epsilon:
+            Number of standard deviations to be added or substracted
+        """
+        msr = self.magnitude_scaling_relationship
+        area = self.get_fault_surface_area() * 1e6  # area in m^2
+        mag = msr.get_median_mag(area=area, rake=self.rake)
+        std = msr.get_std_dev_mag(area=area, rake=self.rake)
+        self.mfd.max_mag = mag + epsilon * std
+
+    def modify_adjust_mfd_from_slip(self, slip_rate: float, rigidity: float,
+                                    recompute_mmax: float = None):
+        """
+        :slip_rate:
+            A float defining slip rate [in mm]
+        :rigidity:
+            A float defining material rigidity [in GPa]
+        :rigidity:
+            A float defining material rigidity [in GPa]
+        """
+        # Check that the current src has a TruncatedGRMFD MFD
+        msg = 'This modification works only when the source MFD is a '
+        msg += 'TruncatedGRMFD'
+        assert self.mfd.__class__.__name__ == 'TruncatedGRMFD', msg
+        # Compute moment
+        area = self.get_fault_surface_area() * 1e6  # area in m^2
+        rigidity *= 1e9  # rigidity in Pa
+        slip_rate *= 1e-3  # slip rate in m
+        mo = rigidity * area * slip_rate
+        # Update the MFD
+        min_mag = self.mfd.min_mag
+        max_mag = self.mfd.max_mag
+        bin_w = self.mfd.bin_width
+        b_val = self.mfd.b_val
+        self.mfd = mfd.TruncatedGRMFD.from_moment(min_mag, max_mag, bin_w,
+                                                  b_val, mo)
