@@ -19,9 +19,9 @@ Module :mod:`openquake.hazardlib.source.kite_fault` defines
 """
 import copy
 import numpy as np
-from typing import Tuple, Optional
+from typing import Tuple
 from openquake.hazardlib import mfd
-from openquake.hazardlib.geo import Point, Line, Polygon
+from openquake.hazardlib.geo import Point, Polygon
 from openquake.hazardlib.geo.mesh import Mesh
 from openquake.hazardlib.geo.surface.kite_fault import (
         get_profiles_from_simple_fault_data)
@@ -37,7 +37,7 @@ class KiteFaultSource(ParametricSeismicSource):
     """
 
     code = b'K'
-    MODIFICATIONS = {}
+    MODIFICATIONS = {'adjust_mfd_from_slip'}
 
     def __init__(self, source_id, name, tectonic_region_type, mfd,
                  rupture_mesh_spacing, magnitude_scaling_relationship,
@@ -72,9 +72,9 @@ class KiteFaultSource(ParametricSeismicSource):
                         floating_y_step):
 
         # Get profiles
-        profiles = get_profiles_from_simple_fault_data(fault_trace, 
-                upper_seismogenic_depth, lower_seismogenic_depth, dip, 
-                rupture_mesh_spacing)
+        profiles = get_profiles_from_simple_fault_data(
+            fault_trace, upper_seismogenic_depth, lower_seismogenic_depth,
+            dip, rupture_mesh_spacing)
 
         # Creating Kite Source
         self = cls(source_id, name, tectonic_region_type, mfd,
