@@ -26,7 +26,7 @@ import math
 import itertools
 import json
 from openquake.baselib import general, hdf5
-from openquake.hazardlib import geo, contexts
+from openquake.hazardlib import geo
 from openquake.hazardlib.geo.nodalplane import NodalPlane
 from openquake.hazardlib.geo.mesh import (
     Mesh, RectangularMesh, surface_to_arrays)
@@ -43,19 +43,45 @@ F32 = numpy.float32
 F64 = numpy.float64
 TWO16 = 2 ** 16
 TWO32 = 2 ** 32
-pmf_dt = numpy.dtype([('prob', float), ('occ', U32)])
-events_dt = numpy.dtype([('id', U32), ('rup_id', U32), ('rlz_id', U16)])
-rupture_dt = numpy.dtype([('seed', U32),
-                          ('mag', F32),
-                          ('rake', F32),
-                          ('lon', F32),
-                          ('lat', F32),
-                          ('dep', F32),
-                          ('multiplicity', U32),
-                          ('trt', hdf5.vstr),
-                          ('kind', hdf5.vstr),
-                          ('mesh', hdf5.vstr),
-                          ('extra', hdf5.vstr)])
+
+pmf_dt = numpy.dtype([
+    ('prob', float),
+    ('occ', U32)])
+
+events_dt = numpy.dtype([
+    ('id', U32),
+    ('rup_id', U32),
+    ('rlz_id', U16)])
+
+rup_dt = numpy.dtype([
+    ('seed', U32),
+    ('mag', F32),
+    ('rake', F32),
+    ('lon', F32),
+    ('lat', F32),
+    ('dep', F32),
+    ('multiplicity', U32),
+    ('trt', hdf5.vstr),
+    ('kind', hdf5.vstr),
+    ('mesh', hdf5.vstr),
+    ('extra', hdf5.vstr)])
+
+rupture_dt = numpy.dtype([
+    ('id', U32),
+    ('seed', U32),
+    ('source_id', '<S16'),
+    ('trt_smr', U16),
+    ('code', U8),
+    ('n_occ', U32),
+    ('mag', F32), ('rake', F32),
+    ('occurrence_rate', F32),
+    ('minlon', F32),
+    ('minlat', F32),
+    ('maxlon', F32),
+    ('maxlat', F32),
+    ('hypo', (F32, 3)),
+    ('geom_id', U32),
+    ('e0', U32)])
 
 code2cls = {}
 
@@ -67,9 +93,10 @@ def to_csv_array(ruptures):
     """
     if not code2cls:
         code2cls.update(BaseRupture.init())
-    arr = numpy.zeros(len(ruptures), rupture_dt)
+    arr = numpy.zeros(len(ruptures), rup_dt)
     for rec, rup in zip(arr, ruptures):
-        arrays = surface_to_arrays(rup.surface)  # shape (3, s1, s2)
+        # s0=number of multi surfaces, s1=number of rows, s2=number of columns
+        arrays = surface_to_arrays(rup.surface)  # shape (s0, 3, s1, s2)
         rec['seed'] = rup.rup_id
         rec['mag'] = rup.mag
         rec['rake'] = rup.rake
@@ -94,19 +121,17 @@ def to_csv_array(ruptures):
     return arr
 
 
-def from_array(aw):
-    """
-    :returns: a list of ruptures from an ArrayWrapper
-    """
-    rups = []
-    names = aw.array.dtype.names
-    for rec in aw.array:
-        dic = dict(zip(names, rec))
-        dic['trt'] = aw.trts[int(dic.pop('et_id'))]
-        dic['hypo'] = dic.pop('lon'), dic.pop('lat'), dic.pop('dep')
-        dic.update(json.loads(dic.pop('extra')))
-        rups.append(_get_rupture(dic))
-    return rups
+def to_arrays(geom):
+    arrays = []
+    num_surfaces = int(geom[0])
+    start = num_surfaces * 2 + 1
+    for i in range(1, 2 * num_surfaces, 2):
+        s1, s2 = int(geom[i]), int(geom[i + 1])
+        size = s1 * s2 * 3
+        array = geom[start:start + size].reshape(3, s1, s2)
+        arrays.append(array)
+        start += size
+    return arrays
 
 
 def _get_rupture(rec, geom=None, trt=None):
@@ -119,15 +144,7 @@ def _get_rupture(rec, geom=None, trt=None):
         geom = numpy.concatenate([[1], [len(rec['lons']), 1], points])
 
     # build surface
-    arrays = []
-    num_surfaces = int(geom[0])
-    start = num_surfaces * 2 + 1
-    for i in range(1, 2 * num_surfaces, 2):
-        s1, s2 = int(geom[i]), int(geom[i + 1])
-        size = s1 * s2 * 3
-        array = geom[start:start + size].reshape(3, s1, s2)
-        arrays.append(array)
-        start += size
+    arrays = to_arrays(geom)
     mesh = arrays[0]
     rupture_cls, surface_cls = code2cls[rec['code']]
     surface = object.__new__(surface_cls)
@@ -281,9 +298,6 @@ class BaseRupture(metaclass=abc.ABCMeta):
         """
         return 1
 
-    get_probability_no_exceedance = (
-        contexts.RuptureContext.get_probability_no_exceedance)
-
     def sample_number_of_occurrences(self, n=1):
         """
         Randomly sample number of occurrences from temporal occurrence model
@@ -412,19 +426,6 @@ class ParametricProbabilisticRupture(BaseRupture):
         r = self.occurrence_rate * self.temporal_occurrence_model.time_span
         return numpy.random.poisson(r, n)
 
-    def get_probability_no_exceedance(self, poes):
-        """
-        See :meth:`superclass method
-        <.rupture.BaseRupture.get_probability_no_exceedance>`
-        for spec of input and result values.
-
-        Uses
-        :meth:`openquake.hazardlib.tom.PoissonTOM.get_probability_no_exceedance`
-        """
-        tom = self.temporal_occurrence_model
-        rate = self.occurrence_rate
-        return tom.get_probability_no_exceedance(rate, poes)
-
     def get_dppvalue(self, site):
         """
         Get the directivity prediction value, DPP at
@@ -697,20 +698,29 @@ class ExportedRupture(object):
 class EBRupture(object):
     """
     An event based rupture. It is a wrapper over a hazardlib rupture
-    object, containing an array of site indices affected by the rupture,
-    as well as the IDs of the corresponding seismic events.
+    object.
+
+    :param rupture: the underlying rupture
+    :param str source_id: ID of the source that generated the rupture
+    :param int trt_smr: an integer describing TRT and source model realization
+    :param int n_occ: number of occurrences of the rupture
+    :param int e0: initial event ID (default 0)
+    :param bool scenario: True for scenario ruptures, default False
     """
-    def __init__(self, rupture, source_id, et_id, n_occ, id=None, e0=0):
-        # NB: when reading an exported ruptures.xml the rup_id will be 0
-        # for the first rupture; it used to be the seed instead
-        assert rupture.rup_id >= 0  # sanity check
+    def __init__(self, rupture, source_id, trt_smr, n_occ=1,
+                 id=None, e0=0, scenario=False):
+        assert rupture.rup_id > 0  # sanity check
         self.rupture = rupture
         self.source_id = source_id
-        self.et_id = et_id
+        self.trt_smr = trt_smr
         self.n_occ = n_occ
         self.id = id  # id of the rupture on the DataStore
         self.e0 = e0
-        self.scenario = False
+        self.scenario = scenario
+
+    @property
+    def tectonic_region_type(self):
+        return self.rupture.tectonic_region_type
 
     @property
     def rup_id(self):
@@ -731,7 +741,7 @@ class EBRupture(object):
             splits = numpy.array_split(all_eids, len(rlzs))
             for rlz_id, eids in zip(rlzs, splits):
                 dic[rlz_id] = eids
-        else:
+        else:  # event_based
             j = 0
             histo = general.random_histogram(
                 self.n_occ, len(rlzs), self.rup_id)
@@ -746,49 +756,6 @@ class EBRupture(object):
         """
         return numpy.arange(self.n_occ, dtype=U32)
 
-    def export(self, events_by_ses):
-        """
-        Yield :class:`Rupture` objects, with all the
-        attributes set, suitable for export in XML format.
-        """
-        rupture = self.rupture
-        new = ExportedRupture(self.id, self.n_occ, events_by_ses)
-        if isinstance(rupture.surface, geo.ComplexFaultSurface):
-            new.typology = 'complexFaultsurface'
-        elif isinstance(rupture.surface, geo.SimpleFaultSurface):
-            new.typology = 'simpleFaultsurface'
-        elif isinstance(rupture.surface, geo.GriddedSurface):
-            new.typology = 'griddedRupture'
-        elif isinstance(rupture.surface, geo.MultiSurface):
-            new.typology = 'multiPlanesRupture'
-        else:
-            new.typology = 'singlePlaneRupture'
-        new.is_from_fault_source = iffs = isinstance(
-            rupture.surface, (geo.ComplexFaultSurface,
-                              geo.SimpleFaultSurface))
-        new.is_gridded_surface = igs = isinstance(
-            rupture.surface, geo.GriddedSurface)
-        new.is_multi_surface = ims = isinstance(
-            rupture.surface, geo.MultiSurface)
-        new.lons, new.lats, new.depths = get_geom(
-            rupture.surface, iffs, ims, igs)
-        new.surface = rupture.surface
-        new.strike = rupture.surface.get_strike()
-        new.dip = rupture.surface.get_dip()
-        new.rake = rupture.rake
-        new.hypocenter = rupture.hypocenter
-        new.tectonic_region_type = rupture.tectonic_region_type
-        new.magnitude = new.mag = rupture.mag
-        new.top_left_corner = None if iffs or ims or igs else (
-            new.lons[0], new.lats[0], new.depths[0])
-        new.top_right_corner = None if iffs or ims or igs else (
-            new.lons[1], new.lats[1], new.depths[1])
-        new.bottom_left_corner = None if iffs or ims or igs else (
-            new.lons[2], new.lats[2], new.depths[2])
-        new.bottom_right_corner = None if iffs or ims or igs else (
-            new.lons[3], new.lats[3], new.depths[3])
-        return new
-
     def __repr__(self):
         return '<%s %d[%d]>' % (
             self.__class__.__name__, self.rup_id, self.n_occ)
@@ -811,10 +778,10 @@ class RuptureProxy(object):
         """
         :returns:
             heuristic weight for the underlying rupture, depending on the
-            number of occurrences, number of samples and number of sites
+            number of occurrences and number of presumably affected sites
         """
         return self['n_occ'] * (
-            100 if self.nsites is None else max(self.nsites, 100))
+            10 if self.nsites is None else max(self.nsites, 10))
 
     def __getitem__(self, name):
         return self.rec[name]
@@ -826,7 +793,59 @@ class RuptureProxy(object):
         """
         # not implemented: rupture_slip_direction
         rupture = _get_rupture(self.rec, self.geom, trt)
-        ebr = EBRupture(rupture, self.rec['source_id'], self.rec['et_id'],
-                        self.rec['n_occ'], self.rec['id'], self.rec['e0'])
-        ebr.scenario = self.scenario
+        ebr = EBRupture(rupture, self.rec['source_id'], self.rec['trt_smr'],
+                        self.rec['n_occ'], self.rec['id'], self.rec['e0'],
+                        self.scenario)
         return ebr
+
+
+def get_ruptures(fname_csv):
+    """
+    Read ruptures in CSV format and return an ArrayWrapper.
+
+    :param fname_csv: path to the CSV file
+    """
+    if not BaseRupture._code:
+        BaseRupture.init()  # initialize rupture codes
+    code = BaseRupture.str2code
+    aw = hdf5.read_csv(fname_csv, rup_dt)
+    rups = []
+    geoms = []
+    n_occ = 1
+    for u, row in enumerate(aw.array):
+        hypo = row['lon'], row['lat'], row['dep']
+        dic = json.loads(row['extra'])
+        meshes = F32(json.loads(row['mesh']))  # num_surfaces 3D arrays
+        num_surfaces = len(meshes)
+        shapes = []
+        points = []
+        minlons = []
+        maxlons = []
+        minlats = []
+        maxlats = []
+        for mesh in meshes:
+            shapes.extend(mesh.shape[1:])
+            points.extend(mesh.flatten())  # lons + lats + deps
+            minlons.append(mesh[0].min())
+            minlats.append(mesh[1].min())
+            maxlons.append(mesh[0].max())
+            maxlats.append(mesh[1].max())
+        rec = numpy.zeros(1, rupture_dt)[0]
+        rec['seed'] = row['seed']
+        rec['minlon'] = minlon = min(minlons)
+        rec['minlat'] = minlat = min(minlats)
+        rec['maxlon'] = maxlon = max(maxlons)
+        rec['maxlat'] = maxlat = max(maxlats)
+        rec['mag'] = row['mag']
+        rec['hypo'] = hypo
+        rate = dic.get('occurrence_rate', numpy.nan)
+        tup = (u, row['seed'], 'no-source', aw.trts.index(row['trt']),
+               code[row['kind']], n_occ, row['mag'], row['rake'], rate,
+               minlon, minlat, maxlon, maxlat, hypo, u, 0)
+        rups.append(tup)
+        geoms.append(numpy.concatenate([[num_surfaces], shapes, points]))
+    if not rups:
+        return ()
+    dic = dict(geom=numpy.array(geoms, object), trts=aw.trts)
+    # NB: PMFs for nonparametric ruptures are missing
+    return hdf5.ArrayWrapper(numpy.array(rups, rupture_dt), dic)
