@@ -81,8 +81,11 @@ class SimpleFaultSource(ParametricSeismicSource):
         for the lowest magnitude value.
     """
     code = b'S'
-    MODIFICATIONS = {'set_geometry', 'adjust_dip', 'set_dip',
-                     'adjust_mfd_from_slip'}
+    MODIFICATIONS = {'adjust_dip', 'adjust_mfd_from_slip',
+                     'set_bGR', 'set_dip', 'set_geometry',
+                     'set_lower_seismogenic_depth', 'set_msr',
+                     'set_slip_rate', 'set_mmax_truncatedGR',
+                     'recompute_mmax'}
 
     def __init__(self, source_id, name, tectonic_region_type,
                  mfd, rupture_mesh_spacing,
@@ -281,30 +284,6 @@ class SimpleFaultSource(ParametricSeismicSource):
         self.dip = dip
         self.rupture_mesh_spacing = spacing
 
-    def modify_adjust_mfd_from_slip(self, slip_rate, rigidity):
-        """
-        :slip_rate:
-            A float defining slip rate [in mm]
-        :rigidity:
-            A float defining material rigidity [in GPa]
-        """
-        # Check that the current src has a TruncatedGRMFD MFD
-        msg = 'This modification works only when the source MFD is a '
-        msg += 'TruncatedGRMFD'
-        assert self.mfd.__class__.__name__ == 'TruncatedGRMFD', msg
-        # Compute moment
-        area = self.get_fault_surface_area() * 1e6  # area in m^2
-        rigidity *= 1e9  # rigidity in Pa
-        slip_rate *= 1e-3  # slip rate in m
-        mo = rigidity * area * slip_rate
-        # Update the MFD
-        min_mag = self.mfd.min_mag
-        max_mag = self.mfd.max_mag
-        bin_w = self.mfd.bin_width
-        b_val = self.mfd.b_val
-        self.mfd = mfd.TruncatedGRMFD.from_moment(min_mag, max_mag, bin_w,
-                                                  b_val, mo)
-
     def modify_adjust_dip(self, increment):
         """
         Modifies the dip by an incremental value
@@ -319,6 +298,18 @@ class SimpleFaultSource(ParametricSeismicSource):
             self.rupture_mesh_spacing)
         self.dip += increment
 
+    def modify_set_lower_seismogenic_depth(self, lsd):
+        """
+        Modifies the lower seismogenic depth
+
+        :param float lsd:
+            New value of the lsd [km]
+        """
+        SimpleFaultSurface.check_fault_data(
+            self.fault_trace, self.upper_seismogenic_depth, lsd,
+            self.dip, self.rupture_mesh_spacing)
+        self.lower_seismogenic_depth = lsd
+
     def modify_set_dip(self, dip):
         """
         Modifies the dip to the specified value
