@@ -155,6 +155,24 @@ def build_complex_fault_geometry(fault_source):
     return Node("complexFaultGeometry", nodes=edge_nodes)
 
 
+@obj_to_node.add('KiteSurface')
+def build_kite_surface(ksurface):
+    """
+    Returns the KiteSurface instance as a Node
+
+    :param ksurface:
+        Kite fault source model as an instance of the :class:
+        `openquake.hazardlib.source.kite_fault.KiteFaultSource`
+    :returns:
+        Instance of :class:`openquake.baselib.node.Node`
+    """
+    profile_nodes = []
+    for profile in ksurface.profiles:
+        node = build_linestring_node(profile, with_depth=True)
+        profile_nodes.append(Node("profile", nodes=[node]))
+    return Node("kiteSurface", nodes=profile_nodes)
+
+
 @obj_to_node.add('EvenlyDiscretizedMFD')
 def build_evenly_discretised_mfd(mfd):
     """
@@ -577,7 +595,7 @@ def build_complex_fault_source_node(fault_source):
     Parses a complex fault source to a Node class
 
     :param fault_source:
-        Simple fault source as instance of :class:
+        Complex fault source as instance of :class:
         `openquake.hazardlib.source.complex_fault.ComplexFaultSource`
     :returns:
         Instance of :class:`openquake.baselib.node.Node`
@@ -593,6 +611,70 @@ def build_complex_fault_source_node(fault_source):
                 nodes=source_nodes)
 
 
+@obj_to_node.add('KiteFaultSource')
+def build_kite_fault_source_node(fault_source):
+    """
+    Parses a kite fault source to a Node class
+
+    :param fault_source:
+        Kite fault source as instance of :class:
+        `openquake.hazardlib.source.kite_fault.KiteFaultSource`
+    :returns:
+        Instance of :class:`openquake.baselib.node.Node`
+    """
+
+    # Parse geometry
+    source_nodes = [build_kite_surface(fault_source)]
+    # Parse common fault source attributes
+    source_nodes.extend(get_fault_source_nodes(fault_source))
+    return Node("kiteFaultSource",
+                get_source_attributes(fault_source),
+                nodes=source_nodes)
+
+
+@obj_to_node.add('MultiFaultSource')
+def build_multi_fault_source_node(multi_fault_source):
+    """
+    Parses a MultiFaultSource to a Node class
+
+    :param multi_fault_source:
+        Multi fault source as instance of :class:
+        `openquake.hazardlib.source.multi_fault.MultiFaultSource`
+    :returns:
+        Instance of :class:`openquake.baselib.node.Node`
+    """
+    rup_nodes = []  # multiPlanesRupture
+    for rup_idxs, pmf_, mag, rake in zip(
+            multi_fault_source.rupture_idxs,
+            multi_fault_source.pmfs,
+            multi_fault_source.mags,
+            multi_fault_source.rakes):
+        probs = ' '.join(map(str, [pair[0] for pair in pmf_.data]))
+        nodes = [Node('magnitude', text=str(mag)),
+                 Node('sectionIndexes', {'indexes': ','.join(rup_idxs)}),
+                 Node('rake', text=str(rake))]
+        rup_node = Node('multiPlanesRupture', {'probs_occur': probs},
+                        nodes=nodes)
+        rup_nodes.append(rup_node)
+    return Node("multiFaultSource",
+                get_source_attributes(multi_fault_source),
+                nodes=rup_nodes)
+
+
+@obj_to_node.add('FaultSection')
+def build_section(section):
+    """
+    Parses a FaultSection instance to a Node class
+
+    :param section:
+        A FaultSection instance
+    :returns:
+        Instance of :class:`openquake.baselib.node.Node`
+    """
+    nodes = [obj_to_node(section.surface)]
+    return Node("section", {'id': section.sec_id}, nodes=nodes)
+
+
 @obj_to_node.add('SourceGroup')
 def build_source_group(source_group):
     source_nodes = [obj_to_node(src) for src in source_group.sources]
@@ -631,7 +713,6 @@ def build_source_model_node(source_model):
     return Node('sourceModel', attrs, nodes=nodes)
 
 
-# usage: hdf5write(datastore.hdf5, csm)
 @obj_to_node.add('CompositeSourceModel')
 def build_source_model(csm):
     nodes = [obj_to_node(sg) for sg in csm.src_groups]
@@ -663,6 +744,8 @@ def write_source_model(dest, sources_or_groups, name=None,
         Source model in different formats
     :param name:
         Name of the source model (if missing, extracted from the filename)
+    :returns:
+        the list of generated filenames
     """
     if isinstance(sources_or_groups, nrml.SourceModel):
         groups = sources_or_groups.src_groups
@@ -683,6 +766,7 @@ def write_source_model(dest, sources_or_groups, name=None,
         del attrs['investigation_time']
     nodes = list(map(obj_to_node, groups))
     ddict = extract_ddict(groups)
+    out = [dest]
     if ddict:
         # remove duplicate content from nodes
         for grp_node in nodes:
@@ -702,14 +786,25 @@ def write_source_model(dest, sources_or_groups, name=None,
                         h[key][:] = v
                     else:
                         h[key] = v
+        out.append(dest5)
 
-    source_model = Node("sourceModel", attrs, nodes=nodes)
+    # produce a geometryModel if there are MultiFaultSources
+    sections = {}
+    for group in groups:
+        for src in group:
+            if hasattr(src, 'sections'):
+                sections.update(src.sections)
+    sections = {sid: sections[sid] for sid in sections}
+    smodel = Node("sourceModel", attrs, nodes=nodes)
     with open(dest, 'wb') as f:
-        nrml.write([source_model], f, '%s')
-    if ddict:
-        return [dest, dest5]
-    else:
-        return [dest]
+        nrml.write([smodel], f, '%s')
+    if sections:
+        secnodes = [obj_to_node(sec) for sec in sections.values()]
+        gmodel = Node("geometryModel", attrs, nodes=secnodes)
+        with open(dest[:-4] + '_sections.xml', 'wb') as f:
+            nrml.write([gmodel], f, '%s')
+            out.append(f.name)
+    return out
 
 
 def tomldump(obj, fileobj=None):
